{-# LANGUAGE PatternSynonyms, ScopedTypeVariables, ViewPatterns #-}


-- | Copyright  : Will Thompson and Iñaki García Etxebarria
-- License    : LGPL-2.1
-- Maintainer : Iñaki García Etxebarria

#if (MIN_VERSION_haskell_gi_overloading(1,0,0) && !defined(__HADDOCK_VERSION__))
#define ENABLE_OVERLOADING
#endif

module GI.Gdk.Constants
    ( 
    pattern PRIORITY_REDRAW                 ,
    pattern MODIFIER_MASK                   ,
    pattern KEY_zstroke                     ,
    pattern KEY_zerosuperior                ,
    pattern KEY_zerosubscript               ,
    pattern KEY_zcaron                      ,
    pattern KEY_zacute                      ,
    pattern KEY_zabovedot                   ,
    pattern KEY_z                           ,
    pattern KEY_ytilde                      ,
    pattern KEY_yhook                       ,
    pattern KEY_ygrave                      ,
    pattern KEY_yen                         ,
    pattern KEY_ydiaeresis                  ,
    pattern KEY_ycircumflex                 ,
    pattern KEY_ybelowdot                   ,
    pattern KEY_yacute                      ,
    pattern KEY_y                           ,
    pattern KEY_xabovedot                   ,
    pattern KEY_x                           ,
    pattern KEY_wgrave                      ,
    pattern KEY_wdiaeresis                  ,
    pattern KEY_wcircumflex                 ,
    pattern KEY_wacute                      ,
    pattern KEY_w                           ,
    pattern KEY_vt                          ,
    pattern KEY_voicedsound                 ,
    pattern KEY_vertconnector               ,
    pattern KEY_vertbar                     ,
    pattern KEY_variation                   ,
    pattern KEY_v                           ,
    pattern KEY_utilde                      ,
    pattern KEY_uring                       ,
    pattern KEY_uptack                      ,
    pattern KEY_upstile                     ,
    pattern KEY_upshoe                      ,
    pattern KEY_uprightcorner               ,
    pattern KEY_upleftcorner                ,
    pattern KEY_upcaret                     ,
    pattern KEY_uparrow                     ,
    pattern KEY_uogonek                     ,
    pattern KEY_union                       ,
    pattern KEY_underscore                  ,
    pattern KEY_underbar                    ,
    pattern KEY_umacron                     ,
    pattern KEY_uhorntilde                  ,
    pattern KEY_uhornhook                   ,
    pattern KEY_uhorngrave                  ,
    pattern KEY_uhornbelowdot               ,
    pattern KEY_uhornacute                  ,
    pattern KEY_uhorn                       ,
    pattern KEY_uhook                       ,
    pattern KEY_ugrave                      ,
    pattern KEY_udoubleacute                ,
    pattern KEY_udiaeresis                  ,
    pattern KEY_ucircumflex                 ,
    pattern KEY_ubreve                      ,
    pattern KEY_ubelowdot                   ,
    pattern KEY_uacute                      ,
    pattern KEY_u                           ,
    pattern KEY_twothirds                   ,
    pattern KEY_twosuperior                 ,
    pattern KEY_twosubscript                ,
    pattern KEY_twofifths                   ,
    pattern KEY_tslash                      ,
    pattern KEY_trademarkincircle           ,
    pattern KEY_trademark                   ,
    pattern KEY_topvertsummationconnector   ,
    pattern KEY_topt                        ,
    pattern KEY_toprightsummation           ,
    pattern KEY_toprightsqbracket           ,
    pattern KEY_toprightparens              ,
    pattern KEY_topleftsummation            ,
    pattern KEY_topleftsqbracket            ,
    pattern KEY_topleftradical              ,
    pattern KEY_topleftparens               ,
    pattern KEY_topintegral                 ,
    pattern KEY_tintegral                   ,
    pattern KEY_threesuperior               ,
    pattern KEY_threesubscript              ,
    pattern KEY_threequarters               ,
    pattern KEY_threefifths                 ,
    pattern KEY_threeeighths                ,
    pattern KEY_thorn                       ,
    pattern KEY_thinspace                   ,
    pattern KEY_therefore                   ,
    pattern KEY_telephonerecorder           ,
    pattern KEY_telephone                   ,
    pattern KEY_tcedilla                    ,
    pattern KEY_tcaron                      ,
    pattern KEY_tabovedot                   ,
    pattern KEY_t                           ,
    pattern KEY_stricteq                    ,
    pattern KEY_sterling                    ,
    pattern KEY_ssharp                      ,
    pattern KEY_squareroot                  ,
    pattern KEY_space                       ,
    pattern KEY_soliddiamond                ,
    pattern KEY_slash                       ,
    pattern KEY_sixsuperior                 ,
    pattern KEY_sixsubscript                ,
    pattern KEY_singlelowquotemark          ,
    pattern KEY_similarequal                ,
    pattern KEY_signifblank                 ,
    pattern KEY_signaturemark               ,
    pattern KEY_sevensuperior               ,
    pattern KEY_sevensubscript              ,
    pattern KEY_seveneighths                ,
    pattern KEY_semivoicedsound             ,
    pattern KEY_semicolon                   ,
    pattern KEY_section                     ,
    pattern KEY_seconds                     ,
    pattern KEY_script_switch               ,
    pattern KEY_scircumflex                 ,
    pattern KEY_schwa                       ,
    pattern KEY_scedilla                    ,
    pattern KEY_scaron                      ,
    pattern KEY_sacute                      ,
    pattern KEY_sabovedot                   ,
    pattern KEY_s                           ,
    pattern KEY_righttack                   ,
    pattern KEY_rightt                      ,
    pattern KEY_rightsinglequotemark        ,
    pattern KEY_rightshoe                   ,
    pattern KEY_rightpointer                ,
    pattern KEY_rightopentriangle           ,
    pattern KEY_rightmiddlesummation        ,
    pattern KEY_rightmiddlecurlybrace       ,
    pattern KEY_rightdoublequotemark        ,
    pattern KEY_rightcaret                  ,
    pattern KEY_rightarrow                  ,
    pattern KEY_rightanglebracket           ,
    pattern KEY_registered                  ,
    pattern KEY_rcedilla                    ,
    pattern KEY_rcaron                      ,
    pattern KEY_radical                     ,
    pattern KEY_racute                      ,
    pattern KEY_r                           ,
    pattern KEY_quoteright                  ,
    pattern KEY_quoteleft                   ,
    pattern KEY_quotedbl                    ,
    pattern KEY_questiondown                ,
    pattern KEY_question                    ,
    pattern KEY_quad                        ,
    pattern KEY_q                           ,
    pattern KEY_punctspace                  ,
    pattern KEY_prolongedsound              ,
    pattern KEY_prescription                ,
    pattern KEY_plusminus                   ,
    pattern KEY_plus                        ,
    pattern KEY_phonographcopyright         ,
    pattern KEY_permille                    ,
    pattern KEY_periodcentered              ,
    pattern KEY_period                      ,
    pattern KEY_percent                     ,
    pattern KEY_partialderivative           ,
    pattern KEY_partdifferential            ,
    pattern KEY_parenright                  ,
    pattern KEY_parenleft                   ,
    pattern KEY_paragraph                   ,
    pattern KEY_pabovedot                   ,
    pattern KEY_p                           ,
    pattern KEY_overline                    ,
    pattern KEY_overbar                     ,
    pattern KEY_otilde                      ,
    pattern KEY_oslash                      ,
    pattern KEY_ordmasculine                ,
    pattern KEY_ordfeminine                 ,
    pattern KEY_opentribulletup             ,
    pattern KEY_opentribulletdown           ,
    pattern KEY_openstar                    ,
    pattern KEY_openrectbullet              ,
    pattern KEY_ooblique                    ,
    pattern KEY_onethird                    ,
    pattern KEY_onesuperior                 ,
    pattern KEY_onesubscript                ,
    pattern KEY_onesixth                    ,
    pattern KEY_onequarter                  ,
    pattern KEY_onehalf                     ,
    pattern KEY_onefifth                    ,
    pattern KEY_oneeighth                   ,
    pattern KEY_omacron                     ,
    pattern KEY_ohorntilde                  ,
    pattern KEY_ohornhook                   ,
    pattern KEY_ohorngrave                  ,
    pattern KEY_ohornbelowdot               ,
    pattern KEY_ohornacute                  ,
    pattern KEY_ohorn                       ,
    pattern KEY_ohook                       ,
    pattern KEY_ograve                      ,
    pattern KEY_ogonek                      ,
    pattern KEY_oe                          ,
    pattern KEY_odoubleacute                ,
    pattern KEY_odiaeresis                  ,
    pattern KEY_ocircumflextilde            ,
    pattern KEY_ocircumflexhook             ,
    pattern KEY_ocircumflexgrave            ,
    pattern KEY_ocircumflexbelowdot         ,
    pattern KEY_ocircumflexacute            ,
    pattern KEY_ocircumflex                 ,
    pattern KEY_ocaron                      ,
    pattern KEY_obelowdot                   ,
    pattern KEY_obarred                     ,
    pattern KEY_oacute                      ,
    pattern KEY_o                           ,
    pattern KEY_numerosign                  ,
    pattern KEY_numbersign                  ,
    pattern KEY_ntilde                      ,
    pattern KEY_notsign                     ,
    pattern KEY_notidentical                ,
    pattern KEY_notequal                    ,
    pattern KEY_notelementof                ,
    pattern KEY_notapproxeq                 ,
    pattern KEY_nobreakspace                ,
    pattern KEY_nl                          ,
    pattern KEY_ninesuperior                ,
    pattern KEY_ninesubscript               ,
    pattern KEY_ncedilla                    ,
    pattern KEY_ncaron                      ,
    pattern KEY_nacute                      ,
    pattern KEY_nabla                       ,
    pattern KEY_n                           ,
    pattern KEY_musicalsharp                ,
    pattern KEY_musicalflat                 ,
    pattern KEY_multiply                    ,
    pattern KEY_mu                          ,
    pattern KEY_minutes                     ,
    pattern KEY_minus                       ,
    pattern KEY_masculine                   ,
    pattern KEY_marker                      ,
    pattern KEY_maltesecross                ,
    pattern KEY_malesymbol                  ,
    pattern KEY_macron                      ,
    pattern KEY_mabovedot                   ,
    pattern KEY_m                           ,
    pattern KEY_lstroke                     ,
    pattern KEY_lowrightcorner              ,
    pattern KEY_lowleftcorner               ,
    pattern KEY_logicalor                   ,
    pattern KEY_logicaland                  ,
    pattern KEY_lf                          ,
    pattern KEY_lessthanequal               ,
    pattern KEY_less                        ,
    pattern KEY_lefttack                    ,
    pattern KEY_leftt                       ,
    pattern KEY_leftsinglequotemark         ,
    pattern KEY_leftshoe                    ,
    pattern KEY_leftradical                 ,
    pattern KEY_leftpointer                 ,
    pattern KEY_leftopentriangle            ,
    pattern KEY_leftmiddlecurlybrace        ,
    pattern KEY_leftdoublequotemark         ,
    pattern KEY_leftcaret                   ,
    pattern KEY_leftarrow                   ,
    pattern KEY_leftanglebracket            ,
    pattern KEY_lcedilla                    ,
    pattern KEY_lcaron                      ,
    pattern KEY_lbelowdot                   ,
    pattern KEY_latincross                  ,
    pattern KEY_lacute                      ,
    pattern KEY_l                           ,
    pattern KEY_kra                         ,
    pattern KEY_kcedilla                    ,
    pattern KEY_kappa                       ,
    pattern KEY_kana_yu                     ,
    pattern KEY_kana_yo                     ,
    pattern KEY_kana_ya                     ,
    pattern KEY_kana_u                      ,
    pattern KEY_kana_tu                     ,
    pattern KEY_kana_tsu                    ,
    pattern KEY_kana_switch                 ,
    pattern KEY_kana_openingbracket         ,
    pattern KEY_kana_o                      ,
    pattern KEY_kana_middledot              ,
    pattern KEY_kana_i                      ,
    pattern KEY_kana_fullstop               ,
    pattern KEY_kana_e                      ,
    pattern KEY_kana_conjunctive            ,
    pattern KEY_kana_comma                  ,
    pattern KEY_kana_closingbracket         ,
    pattern KEY_kana_a                      ,
    pattern KEY_kana_YU                     ,
    pattern KEY_kana_YO                     ,
    pattern KEY_kana_YA                     ,
    pattern KEY_kana_WO                     ,
    pattern KEY_kana_WA                     ,
    pattern KEY_kana_U                      ,
    pattern KEY_kana_TU                     ,
    pattern KEY_kana_TSU                    ,
    pattern KEY_kana_TO                     ,
    pattern KEY_kana_TI                     ,
    pattern KEY_kana_TE                     ,
    pattern KEY_kana_TA                     ,
    pattern KEY_kana_SU                     ,
    pattern KEY_kana_SO                     ,
    pattern KEY_kana_SHI                    ,
    pattern KEY_kana_SE                     ,
    pattern KEY_kana_SA                     ,
    pattern KEY_kana_RU                     ,
    pattern KEY_kana_RO                     ,
    pattern KEY_kana_RI                     ,
    pattern KEY_kana_RE                     ,
    pattern KEY_kana_RA                     ,
    pattern KEY_kana_O                      ,
    pattern KEY_kana_NU                     ,
    pattern KEY_kana_NO                     ,
    pattern KEY_kana_NI                     ,
    pattern KEY_kana_NE                     ,
    pattern KEY_kana_NA                     ,
    pattern KEY_kana_N                      ,
    pattern KEY_kana_MU                     ,
    pattern KEY_kana_MO                     ,
    pattern KEY_kana_MI                     ,
    pattern KEY_kana_ME                     ,
    pattern KEY_kana_MA                     ,
    pattern KEY_kana_KU                     ,
    pattern KEY_kana_KO                     ,
    pattern KEY_kana_KI                     ,
    pattern KEY_kana_KE                     ,
    pattern KEY_kana_KA                     ,
    pattern KEY_kana_I                      ,
    pattern KEY_kana_HU                     ,
    pattern KEY_kana_HO                     ,
    pattern KEY_kana_HI                     ,
    pattern KEY_kana_HE                     ,
    pattern KEY_kana_HA                     ,
    pattern KEY_kana_FU                     ,
    pattern KEY_kana_E                      ,
    pattern KEY_kana_CHI                    ,
    pattern KEY_kana_A                      ,
    pattern KEY_k                           ,
    pattern KEY_jot                         ,
    pattern KEY_jcircumflex                 ,
    pattern KEY_j                           ,
    pattern KEY_itilde                      ,
    pattern KEY_iogonek                     ,
    pattern KEY_intersection                ,
    pattern KEY_integral                    ,
    pattern KEY_infinity                    ,
    pattern KEY_includes                    ,
    pattern KEY_includedin                  ,
    pattern KEY_implies                     ,
    pattern KEY_imacron                     ,
    pattern KEY_ihook                       ,
    pattern KEY_igrave                      ,
    pattern KEY_ifonlyif                    ,
    pattern KEY_idotless                    ,
    pattern KEY_idiaeresis                  ,
    pattern KEY_identical                   ,
    pattern KEY_icircumflex                 ,
    pattern KEY_ibreve                      ,
    pattern KEY_ibelowdot                   ,
    pattern KEY_iacute                      ,
    pattern KEY_iTouch                      ,
    pattern KEY_i                           ,
    pattern KEY_hyphen                      ,
    pattern KEY_ht                          ,
    pattern KEY_hstroke                     ,
    pattern KEY_horizlinescan9              ,
    pattern KEY_horizlinescan7              ,
    pattern KEY_horizlinescan5              ,
    pattern KEY_horizlinescan3              ,
    pattern KEY_horizlinescan1              ,
    pattern KEY_horizconnector              ,
    pattern KEY_hexagram                    ,
    pattern KEY_hebrew_zayin                ,
    pattern KEY_hebrew_zain                 ,
    pattern KEY_hebrew_zadi                 ,
    pattern KEY_hebrew_zade                 ,
    pattern KEY_hebrew_yod                  ,
    pattern KEY_hebrew_waw                  ,
    pattern KEY_hebrew_teth                 ,
    pattern KEY_hebrew_tet                  ,
    pattern KEY_hebrew_taw                  ,
    pattern KEY_hebrew_taf                  ,
    pattern KEY_hebrew_shin                 ,
    pattern KEY_hebrew_samekh               ,
    pattern KEY_hebrew_samech               ,
    pattern KEY_hebrew_resh                 ,
    pattern KEY_hebrew_qoph                 ,
    pattern KEY_hebrew_pe                   ,
    pattern KEY_hebrew_nun                  ,
    pattern KEY_hebrew_mem                  ,
    pattern KEY_hebrew_lamed                ,
    pattern KEY_hebrew_kuf                  ,
    pattern KEY_hebrew_kaph                 ,
    pattern KEY_hebrew_het                  ,
    pattern KEY_hebrew_he                   ,
    pattern KEY_hebrew_gimmel               ,
    pattern KEY_hebrew_gimel                ,
    pattern KEY_hebrew_finalzadi            ,
    pattern KEY_hebrew_finalzade            ,
    pattern KEY_hebrew_finalpe              ,
    pattern KEY_hebrew_finalnun             ,
    pattern KEY_hebrew_finalmem             ,
    pattern KEY_hebrew_finalkaph            ,
    pattern KEY_hebrew_doublelowline        ,
    pattern KEY_hebrew_daleth               ,
    pattern KEY_hebrew_dalet                ,
    pattern KEY_hebrew_chet                 ,
    pattern KEY_hebrew_beth                 ,
    pattern KEY_hebrew_bet                  ,
    pattern KEY_hebrew_ayin                 ,
    pattern KEY_hebrew_aleph                ,
    pattern KEY_heart                       ,
    pattern KEY_hcircumflex                 ,
    pattern KEY_hairspace                   ,
    pattern KEY_h                           ,
    pattern KEY_guillemotright              ,
    pattern KEY_guillemotleft               ,
    pattern KEY_guillemetright              ,
    pattern KEY_guillemetleft               ,
    pattern KEY_greaterthanequal            ,
    pattern KEY_greater                     ,
    pattern KEY_grave                       ,
    pattern KEY_gcircumflex                 ,
    pattern KEY_gcedilla                    ,
    pattern KEY_gcaron                      ,
    pattern KEY_gbreve                      ,
    pattern KEY_gabovedot                   ,
    pattern KEY_g                           ,
    pattern KEY_function                    ,
    pattern KEY_fourthroot                  ,
    pattern KEY_foursuperior                ,
    pattern KEY_foursubscript               ,
    pattern KEY_fourfifths                  ,
    pattern KEY_fivesuperior                ,
    pattern KEY_fivesubscript               ,
    pattern KEY_fivesixths                  ,
    pattern KEY_fiveeighths                 ,
    pattern KEY_filledtribulletup           ,
    pattern KEY_filledtribulletdown         ,
    pattern KEY_filledrighttribullet        ,
    pattern KEY_filledrectbullet            ,
    pattern KEY_filledlefttribullet         ,
    pattern KEY_figdash                     ,
    pattern KEY_ff                          ,
    pattern KEY_femalesymbol                ,
    pattern KEY_fabovedot                   ,
    pattern KEY_f                           ,
    pattern KEY_ezh                         ,
    pattern KEY_exclamdown                  ,
    pattern KEY_exclam                      ,
    pattern KEY_etilde                      ,
    pattern KEY_eth                         ,
    pattern KEY_equal                       ,
    pattern KEY_eogonek                     ,
    pattern KEY_enspace                     ,
    pattern KEY_enopensquarebullet          ,
    pattern KEY_enopencircbullet            ,
    pattern KEY_eng                         ,
    pattern KEY_enfilledsqbullet            ,
    pattern KEY_enfilledcircbullet          ,
    pattern KEY_endash                      ,
    pattern KEY_emspace                     ,
    pattern KEY_emptyset                    ,
    pattern KEY_emopenrectangle             ,
    pattern KEY_emopencircle                ,
    pattern KEY_emfilledrect                ,
    pattern KEY_emfilledcircle              ,
    pattern KEY_emdash                      ,
    pattern KEY_emacron                     ,
    pattern KEY_em4space                    ,
    pattern KEY_em3space                    ,
    pattern KEY_ellipsis                    ,
    pattern KEY_elementof                   ,
    pattern KEY_eightsuperior               ,
    pattern KEY_eightsubscript              ,
    pattern KEY_ehook                       ,
    pattern KEY_egrave                      ,
    pattern KEY_ediaeresis                  ,
    pattern KEY_ecircumflextilde            ,
    pattern KEY_ecircumflexhook             ,
    pattern KEY_ecircumflexgrave            ,
    pattern KEY_ecircumflexbelowdot         ,
    pattern KEY_ecircumflexacute            ,
    pattern KEY_ecircumflex                 ,
    pattern KEY_ecaron                      ,
    pattern KEY_ebelowdot                   ,
    pattern KEY_eacute                      ,
    pattern KEY_eabovedot                   ,
    pattern KEY_e                           ,
    pattern KEY_dstroke                     ,
    pattern KEY_downtack                    ,
    pattern KEY_downstile                   ,
    pattern KEY_downshoe                    ,
    pattern KEY_downcaret                   ,
    pattern KEY_downarrow                   ,
    pattern KEY_doublelowquotemark          ,
    pattern KEY_doubledagger                ,
    pattern KEY_doubleacute                 ,
    pattern KEY_doubbaselinedot             ,
    pattern KEY_dollar                      ,
    pattern KEY_division                    ,
    pattern KEY_dintegral                   ,
    pattern KEY_digitspace                  ,
    pattern KEY_diamond                     ,
    pattern KEY_diaeresis                   ,
    pattern KEY_degree                      ,
    pattern KEY_decimalpoint                ,
    pattern KEY_dead_voiced_sound           ,
    pattern KEY_dead_u                      ,
    pattern KEY_dead_tilde                  ,
    pattern KEY_dead_stroke                 ,
    pattern KEY_dead_small_schwa            ,
    pattern KEY_dead_semivoiced_sound       ,
    pattern KEY_dead_schwa                  ,
    pattern KEY_dead_psili                  ,
    pattern KEY_dead_perispomeni            ,
    pattern KEY_dead_ogonek                 ,
    pattern KEY_dead_o                      ,
    pattern KEY_dead_macron                 ,
    pattern KEY_dead_lowline                ,
    pattern KEY_dead_longsolidusoverlay     ,
    pattern KEY_dead_iota                   ,
    pattern KEY_dead_invertedbreve          ,
    pattern KEY_dead_i                      ,
    pattern KEY_dead_horn                   ,
    pattern KEY_dead_hook                   ,
    pattern KEY_dead_hamza                  ,
    pattern KEY_dead_greek                  ,
    pattern KEY_dead_grave                  ,
    pattern KEY_dead_e                      ,
    pattern KEY_dead_doublegrave            ,
    pattern KEY_dead_doubleacute            ,
    pattern KEY_dead_diaeresis              ,
    pattern KEY_dead_dasia                  ,
    pattern KEY_dead_currency               ,
    pattern KEY_dead_circumflex             ,
    pattern KEY_dead_cedilla                ,
    pattern KEY_dead_caron                  ,
    pattern KEY_dead_capital_schwa          ,
    pattern KEY_dead_breve                  ,
    pattern KEY_dead_belowverticalline      ,
    pattern KEY_dead_belowtilde             ,
    pattern KEY_dead_belowring              ,
    pattern KEY_dead_belowmacron            ,
    pattern KEY_dead_belowdot               ,
    pattern KEY_dead_belowdiaeresis         ,
    pattern KEY_dead_belowcomma             ,
    pattern KEY_dead_belowcircumflex        ,
    pattern KEY_dead_belowbreve             ,
    pattern KEY_dead_acute                  ,
    pattern KEY_dead_aboveverticalline      ,
    pattern KEY_dead_abovering              ,
    pattern KEY_dead_abovereversedcomma     ,
    pattern KEY_dead_abovedot               ,
    pattern KEY_dead_abovecomma             ,
    pattern KEY_dead_a                      ,
    pattern KEY_dead_U                      ,
    pattern KEY_dead_SCHWA                  ,
    pattern KEY_dead_O                      ,
    pattern KEY_dead_I                      ,
    pattern KEY_dead_E                      ,
    pattern KEY_dead_A                      ,
    pattern KEY_dcaron                      ,
    pattern KEY_dagger                      ,
    pattern KEY_dabovedot                   ,
    pattern KEY_d                           ,
    pattern KEY_cursor                      ,
    pattern KEY_currency                    ,
    pattern KEY_cuberoot                    ,
    pattern KEY_crossinglines               ,
    pattern KEY_cr                          ,
    pattern KEY_copyright                   ,
    pattern KEY_containsas                  ,
    pattern KEY_comma                       ,
    pattern KEY_combining_tilde             ,
    pattern KEY_combining_hook              ,
    pattern KEY_combining_grave             ,
    pattern KEY_combining_belowdot          ,
    pattern KEY_combining_acute             ,
    pattern KEY_colon                       ,
    pattern KEY_club                        ,
    pattern KEY_circle                      ,
    pattern KEY_checkmark                   ,
    pattern KEY_checkerboard                ,
    pattern KEY_ch                          ,
    pattern KEY_cent                        ,
    pattern KEY_cedilla                     ,
    pattern KEY_ccircumflex                 ,
    pattern KEY_ccedilla                    ,
    pattern KEY_ccaron                      ,
    pattern KEY_caron                       ,
    pattern KEY_caret                       ,
    pattern KEY_careof                      ,
    pattern KEY_cacute                      ,
    pattern KEY_cabovedot                   ,
    pattern KEY_c_h                         ,
    pattern KEY_c                           ,
    pattern KEY_brokenbar                   ,
    pattern KEY_breve                       ,
    pattern KEY_braille_dots_8              ,
    pattern KEY_braille_dots_78             ,
    pattern KEY_braille_dots_7              ,
    pattern KEY_braille_dots_68             ,
    pattern KEY_braille_dots_678            ,
    pattern KEY_braille_dots_67             ,
    pattern KEY_braille_dots_6              ,
    pattern KEY_braille_dots_58             ,
    pattern KEY_braille_dots_578            ,
    pattern KEY_braille_dots_57             ,
    pattern KEY_braille_dots_568            ,
    pattern KEY_braille_dots_5678           ,
    pattern KEY_braille_dots_567            ,
    pattern KEY_braille_dots_56             ,
    pattern KEY_braille_dots_5              ,
    pattern KEY_braille_dots_48             ,
    pattern KEY_braille_dots_478            ,
    pattern KEY_braille_dots_47             ,
    pattern KEY_braille_dots_468            ,
    pattern KEY_braille_dots_4678           ,
    pattern KEY_braille_dots_467            ,
    pattern KEY_braille_dots_46             ,
    pattern KEY_braille_dots_458            ,
    pattern KEY_braille_dots_4578           ,
    pattern KEY_braille_dots_457            ,
    pattern KEY_braille_dots_4568           ,
    pattern KEY_braille_dots_45678          ,
    pattern KEY_braille_dots_4567           ,
    pattern KEY_braille_dots_456            ,
    pattern KEY_braille_dots_45             ,
    pattern KEY_braille_dots_4              ,
    pattern KEY_braille_dots_38             ,
    pattern KEY_braille_dots_378            ,
    pattern KEY_braille_dots_37             ,
    pattern KEY_braille_dots_368            ,
    pattern KEY_braille_dots_3678           ,
    pattern KEY_braille_dots_367            ,
    pattern KEY_braille_dots_36             ,
    pattern KEY_braille_dots_358            ,
    pattern KEY_braille_dots_3578           ,
    pattern KEY_braille_dots_357            ,
    pattern KEY_braille_dots_3568           ,
    pattern KEY_braille_dots_35678          ,
    pattern KEY_braille_dots_3567           ,
    pattern KEY_braille_dots_356            ,
    pattern KEY_braille_dots_35             ,
    pattern KEY_braille_dots_348            ,
    pattern KEY_braille_dots_3478           ,
    pattern KEY_braille_dots_347            ,
    pattern KEY_braille_dots_3468           ,
    pattern KEY_braille_dots_34678          ,
    pattern KEY_braille_dots_3467           ,
    pattern KEY_braille_dots_346            ,
    pattern KEY_braille_dots_3458           ,
    pattern KEY_braille_dots_34578          ,
    pattern KEY_braille_dots_3457           ,
    pattern KEY_braille_dots_34568          ,
    pattern KEY_braille_dots_345678         ,
    pattern KEY_braille_dots_34567          ,
    pattern KEY_braille_dots_3456           ,
    pattern KEY_braille_dots_345            ,
    pattern KEY_braille_dots_34             ,
    pattern KEY_braille_dots_3              ,
    pattern KEY_braille_dots_28             ,
    pattern KEY_braille_dots_278            ,
    pattern KEY_braille_dots_27             ,
    pattern KEY_braille_dots_268            ,
    pattern KEY_braille_dots_2678           ,
    pattern KEY_braille_dots_267            ,
    pattern KEY_braille_dots_26             ,
    pattern KEY_braille_dots_258            ,
    pattern KEY_braille_dots_2578           ,
    pattern KEY_braille_dots_257            ,
    pattern KEY_braille_dots_2568           ,
    pattern KEY_braille_dots_25678          ,
    pattern KEY_braille_dots_2567           ,
    pattern KEY_braille_dots_256            ,
    pattern KEY_braille_dots_25             ,
    pattern KEY_braille_dots_248            ,
    pattern KEY_braille_dots_2478           ,
    pattern KEY_braille_dots_247            ,
    pattern KEY_braille_dots_2468           ,
    pattern KEY_braille_dots_24678          ,
    pattern KEY_braille_dots_2467           ,
    pattern KEY_braille_dots_246            ,
    pattern KEY_braille_dots_2458           ,
    pattern KEY_braille_dots_24578          ,
    pattern KEY_braille_dots_2457           ,
    pattern KEY_braille_dots_24568          ,
    pattern KEY_braille_dots_245678         ,
    pattern KEY_braille_dots_24567          ,
    pattern KEY_braille_dots_2456           ,
    pattern KEY_braille_dots_245            ,
    pattern KEY_braille_dots_24             ,
    pattern KEY_braille_dots_238            ,
    pattern KEY_braille_dots_2378           ,
    pattern KEY_braille_dots_237            ,
    pattern KEY_braille_dots_2368           ,
    pattern KEY_braille_dots_23678          ,
    pattern KEY_braille_dots_2367           ,
    pattern KEY_braille_dots_236            ,
    pattern KEY_braille_dots_2358           ,
    pattern KEY_braille_dots_23578          ,
    pattern KEY_braille_dots_2357           ,
    pattern KEY_braille_dots_23568          ,
    pattern KEY_braille_dots_235678         ,
    pattern KEY_braille_dots_23567          ,
    pattern KEY_braille_dots_2356           ,
    pattern KEY_braille_dots_235            ,
    pattern KEY_braille_dots_2348           ,
    pattern KEY_braille_dots_23478          ,
    pattern KEY_braille_dots_2347           ,
    pattern KEY_braille_dots_23468          ,
    pattern KEY_braille_dots_234678         ,
    pattern KEY_braille_dots_23467          ,
    pattern KEY_braille_dots_2346           ,
    pattern KEY_braille_dots_23458          ,
    pattern KEY_braille_dots_234578         ,
    pattern KEY_braille_dots_23457          ,
    pattern KEY_braille_dots_234568         ,
    pattern KEY_braille_dots_2345678        ,
    pattern KEY_braille_dots_234567         ,
    pattern KEY_braille_dots_23456          ,
    pattern KEY_braille_dots_2345           ,
    pattern KEY_braille_dots_234            ,
    pattern KEY_braille_dots_23             ,
    pattern KEY_braille_dots_2              ,
    pattern KEY_braille_dots_18             ,
    pattern KEY_braille_dots_178            ,
    pattern KEY_braille_dots_17             ,
    pattern KEY_braille_dots_168            ,
    pattern KEY_braille_dots_1678           ,
    pattern KEY_braille_dots_167            ,
    pattern KEY_braille_dots_16             ,
    pattern KEY_braille_dots_158            ,
    pattern KEY_braille_dots_1578           ,
    pattern KEY_braille_dots_157            ,
    pattern KEY_braille_dots_1568           ,
    pattern KEY_braille_dots_15678          ,
    pattern KEY_braille_dots_1567           ,
    pattern KEY_braille_dots_156            ,
    pattern KEY_braille_dots_15             ,
    pattern KEY_braille_dots_148            ,
    pattern KEY_braille_dots_1478           ,
    pattern KEY_braille_dots_147            ,
    pattern KEY_braille_dots_1468           ,
    pattern KEY_braille_dots_14678          ,
    pattern KEY_braille_dots_1467           ,
    pattern KEY_braille_dots_146            ,
    pattern KEY_braille_dots_1458           ,
    pattern KEY_braille_dots_14578          ,
    pattern KEY_braille_dots_1457           ,
    pattern KEY_braille_dots_14568          ,
    pattern KEY_braille_dots_145678         ,
    pattern KEY_braille_dots_14567          ,
    pattern KEY_braille_dots_1456           ,
    pattern KEY_braille_dots_145            ,
    pattern KEY_braille_dots_14             ,
    pattern KEY_braille_dots_138            ,
    pattern KEY_braille_dots_1378           ,
    pattern KEY_braille_dots_137            ,
    pattern KEY_braille_dots_1368           ,
    pattern KEY_braille_dots_13678          ,
    pattern KEY_braille_dots_1367           ,
    pattern KEY_braille_dots_136            ,
    pattern KEY_braille_dots_1358           ,
    pattern KEY_braille_dots_13578          ,
    pattern KEY_braille_dots_1357           ,
    pattern KEY_braille_dots_13568          ,
    pattern KEY_braille_dots_135678         ,
    pattern KEY_braille_dots_13567          ,
    pattern KEY_braille_dots_1356           ,
    pattern KEY_braille_dots_135            ,
    pattern KEY_braille_dots_1348           ,
    pattern KEY_braille_dots_13478          ,
    pattern KEY_braille_dots_1347           ,
    pattern KEY_braille_dots_13468          ,
    pattern KEY_braille_dots_134678         ,
    pattern KEY_braille_dots_13467          ,
    pattern KEY_braille_dots_1346           ,
    pattern KEY_braille_dots_13458          ,
    pattern KEY_braille_dots_134578         ,
    pattern KEY_braille_dots_13457          ,
    pattern KEY_braille_dots_134568         ,
    pattern KEY_braille_dots_1345678        ,
    pattern KEY_braille_dots_134567         ,
    pattern KEY_braille_dots_13456          ,
    pattern KEY_braille_dots_1345           ,
    pattern KEY_braille_dots_134            ,
    pattern KEY_braille_dots_13             ,
    pattern KEY_braille_dots_128            ,
    pattern KEY_braille_dots_1278           ,
    pattern KEY_braille_dots_127            ,
    pattern KEY_braille_dots_1268           ,
    pattern KEY_braille_dots_12678          ,
    pattern KEY_braille_dots_1267           ,
    pattern KEY_braille_dots_126            ,
    pattern KEY_braille_dots_1258           ,
    pattern KEY_braille_dots_12578          ,
    pattern KEY_braille_dots_1257           ,
    pattern KEY_braille_dots_12568          ,
    pattern KEY_braille_dots_125678         ,
    pattern KEY_braille_dots_12567          ,
    pattern KEY_braille_dots_1256           ,
    pattern KEY_braille_dots_125            ,
    pattern KEY_braille_dots_1248           ,
    pattern KEY_braille_dots_12478          ,
    pattern KEY_braille_dots_1247           ,
    pattern KEY_braille_dots_12468          ,
    pattern KEY_braille_dots_124678         ,
    pattern KEY_braille_dots_12467          ,
    pattern KEY_braille_dots_1246           ,
    pattern KEY_braille_dots_12458          ,
    pattern KEY_braille_dots_124578         ,
    pattern KEY_braille_dots_12457          ,
    pattern KEY_braille_dots_124568         ,
    pattern KEY_braille_dots_1245678        ,
    pattern KEY_braille_dots_124567         ,
    pattern KEY_braille_dots_12456          ,
    pattern KEY_braille_dots_1245           ,
    pattern KEY_braille_dots_124            ,
    pattern KEY_braille_dots_1238           ,
    pattern KEY_braille_dots_12378          ,
    pattern KEY_braille_dots_1237           ,
    pattern KEY_braille_dots_12368          ,
    pattern KEY_braille_dots_123678         ,
    pattern KEY_braille_dots_12367          ,
    pattern KEY_braille_dots_1236           ,
    pattern KEY_braille_dots_12358          ,
    pattern KEY_braille_dots_123578         ,
    pattern KEY_braille_dots_12357          ,
    pattern KEY_braille_dots_123568         ,
    pattern KEY_braille_dots_1235678        ,
    pattern KEY_braille_dots_123567         ,
    pattern KEY_braille_dots_12356          ,
    pattern KEY_braille_dots_1235           ,
    pattern KEY_braille_dots_12348          ,
    pattern KEY_braille_dots_123478         ,
    pattern KEY_braille_dots_12347          ,
    pattern KEY_braille_dots_123468         ,
    pattern KEY_braille_dots_1234678        ,
    pattern KEY_braille_dots_123467         ,
    pattern KEY_braille_dots_12346          ,
    pattern KEY_braille_dots_123458         ,
    pattern KEY_braille_dots_1234578        ,
    pattern KEY_braille_dots_123457         ,
    pattern KEY_braille_dots_1234568        ,
    pattern KEY_braille_dots_12345678       ,
    pattern KEY_braille_dots_1234567        ,
    pattern KEY_braille_dots_123456         ,
    pattern KEY_braille_dots_12345          ,
    pattern KEY_braille_dots_1234           ,
    pattern KEY_braille_dots_123            ,
    pattern KEY_braille_dots_12             ,
    pattern KEY_braille_dots_1              ,
    pattern KEY_braille_dot_9               ,
    pattern KEY_braille_dot_8               ,
    pattern KEY_braille_dot_7               ,
    pattern KEY_braille_dot_6               ,
    pattern KEY_braille_dot_5               ,
    pattern KEY_braille_dot_4               ,
    pattern KEY_braille_dot_3               ,
    pattern KEY_braille_dot_2               ,
    pattern KEY_braille_dot_10              ,
    pattern KEY_braille_dot_1               ,
    pattern KEY_braille_blank               ,
    pattern KEY_bracketright                ,
    pattern KEY_bracketleft                 ,
    pattern KEY_braceright                  ,
    pattern KEY_braceleft                   ,
    pattern KEY_botvertsummationconnector   ,
    pattern KEY_bott                        ,
    pattern KEY_botrightsummation           ,
    pattern KEY_botrightsqbracket           ,
    pattern KEY_botrightparens              ,
    pattern KEY_botleftsummation            ,
    pattern KEY_botleftsqbracket            ,
    pattern KEY_botleftparens               ,
    pattern KEY_botintegral                 ,
    pattern KEY_blank                       ,
    pattern KEY_because                     ,
    pattern KEY_bar                         ,
    pattern KEY_ballotcross                 ,
    pattern KEY_backslash                   ,
    pattern KEY_babovedot                   ,
    pattern KEY_b                           ,
    pattern KEY_atilde                      ,
    pattern KEY_at                          ,
    pattern KEY_asterisk                    ,
    pattern KEY_asciitilde                  ,
    pattern KEY_asciicircum                 ,
    pattern KEY_aring                       ,
    pattern KEY_approximate                 ,
    pattern KEY_approxeq                    ,
    pattern KEY_apostrophe                  ,
    pattern KEY_aogonek                     ,
    pattern KEY_ampersand                   ,
    pattern KEY_amacron                     ,
    pattern KEY_ahook                       ,
    pattern KEY_agrave                      ,
    pattern KEY_ae                          ,
    pattern KEY_adiaeresis                  ,
    pattern KEY_acute                       ,
    pattern KEY_acircumflextilde            ,
    pattern KEY_acircumflexhook             ,
    pattern KEY_acircumflexgrave            ,
    pattern KEY_acircumflexbelowdot         ,
    pattern KEY_acircumflexacute            ,
    pattern KEY_acircumflex                 ,
    pattern KEY_abrevetilde                 ,
    pattern KEY_abrevehook                  ,
    pattern KEY_abrevegrave                 ,
    pattern KEY_abrevebelowdot              ,
    pattern KEY_abreveacute                 ,
    pattern KEY_abreve                      ,
    pattern KEY_abovedot                    ,
    pattern KEY_abelowdot                   ,
    pattern KEY_aacute                      ,
    pattern KEY_a                           ,
    pattern KEY_Zstroke                     ,
    pattern KEY_ZoomReset                   ,
    pattern KEY_ZoomOut                     ,
    pattern KEY_ZoomIn                      ,
    pattern KEY_Zenkaku_Hankaku             ,
    pattern KEY_Zenkaku                     ,
    pattern KEY_Zen_Koho                    ,
    pattern KEY_Zcaron                      ,
    pattern KEY_Zacute                      ,
    pattern KEY_Zabovedot                   ,
    pattern KEY_Z                           ,
    pattern KEY_Ytilde                      ,
    pattern KEY_Yhook                       ,
    pattern KEY_Ygrave                      ,
    pattern KEY_Yellow                      ,
    pattern KEY_Ydiaeresis                  ,
    pattern KEY_Ycircumflex                 ,
    pattern KEY_Ybelowdot                   ,
    pattern KEY_Yacute                      ,
    pattern KEY_Y                           ,
    pattern KEY_Xfer                        ,
    pattern KEY_Xabovedot                   ,
    pattern KEY_X                           ,
    pattern KEY_Word                        ,
    pattern KEY_WonSign                     ,
    pattern KEY_WindowClear                 ,
    pattern KEY_WheelButton                 ,
    pattern KEY_Wgrave                      ,
    pattern KEY_WebCam                      ,
    pattern KEY_Wdiaeresis                  ,
    pattern KEY_Wcircumflex                 ,
    pattern KEY_WakeUp                      ,
    pattern KEY_Wacute                      ,
    pattern KEY_WWW                         ,
    pattern KEY_WWAN                        ,
    pattern KEY_WPSButton                   ,
    pattern KEY_WLAN                        ,
    pattern KEY_W                           ,
    pattern KEY_VoidSymbol                  ,
    pattern KEY_Voicemail                   ,
    pattern KEY_VoiceCommand                ,
    pattern KEY_View                        ,
    pattern KEY_VideoPhone                  ,
    pattern KEY_Video                       ,
    pattern KEY_VendorHome                  ,
    pattern KEY_VOD                         ,
    pattern KEY_V                           ,
    pattern KEY_Utilde                      ,
    pattern KEY_UserPB                      ,
    pattern KEY_User2KB                     ,
    pattern KEY_User1KB                     ,
    pattern KEY_Uring                       ,
    pattern KEY_Up                          ,
    pattern KEY_Uogonek                     ,
    pattern KEY_Unmute                      ,
    pattern KEY_Ungrab                      ,
    pattern KEY_Undo                        ,
    pattern KEY_Umacron                     ,
    pattern KEY_Ukranian_yi                 ,
    pattern KEY_Ukranian_je                 ,
    pattern KEY_Ukranian_i                  ,
    pattern KEY_Ukranian_YI                 ,
    pattern KEY_Ukranian_JE                 ,
    pattern KEY_Ukranian_I                  ,
    pattern KEY_Ukrainian_yi                ,
    pattern KEY_Ukrainian_ie                ,
    pattern KEY_Ukrainian_i                 ,
    pattern KEY_Ukrainian_ghe_with_upturn   ,
    pattern KEY_Ukrainian_YI                ,
    pattern KEY_Ukrainian_IE                ,
    pattern KEY_Ukrainian_I                 ,
    pattern KEY_Ukrainian_GHE_WITH_UPTURN   ,
    pattern KEY_Uhorntilde                  ,
    pattern KEY_Uhornhook                   ,
    pattern KEY_Uhorngrave                  ,
    pattern KEY_Uhornbelowdot               ,
    pattern KEY_Uhornacute                  ,
    pattern KEY_Uhorn                       ,
    pattern KEY_Uhook                       ,
    pattern KEY_Ugrave                      ,
    pattern KEY_Udoubleacute                ,
    pattern KEY_Udiaeresis                  ,
    pattern KEY_Ucircumflex                 ,
    pattern KEY_Ubreve                      ,
    pattern KEY_Ubelowdot                   ,
    pattern KEY_Uacute                      ,
    pattern KEY_UWB                         ,
    pattern KEY_U                           ,
    pattern KEY_Tslash                      ,
    pattern KEY_Travel                      ,
    pattern KEY_TraditionalSonar            ,
    pattern KEY_Touroku                     ,
    pattern KEY_TouchpadToggle              ,
    pattern KEY_TouchpadOn                  ,
    pattern KEY_TouchpadOff                 ,
    pattern KEY_TopMenu                     ,
    pattern KEY_Tools                       ,
    pattern KEY_ToDoList                    ,
    pattern KEY_Time                        ,
    pattern KEY_Thorn                       ,
    pattern KEY_Thai_yoying                 ,
    pattern KEY_Thai_yoyak                  ,
    pattern KEY_Thai_wowaen                 ,
    pattern KEY_Thai_totao                  ,
    pattern KEY_Thai_topatak                ,
    pattern KEY_Thai_thothung               ,
    pattern KEY_Thai_thothong               ,
    pattern KEY_Thai_thothan                ,
    pattern KEY_Thai_thothahan              ,
    pattern KEY_Thai_thophuthao             ,
    pattern KEY_Thai_thonangmontho          ,
    pattern KEY_Thai_thanthakhat            ,
    pattern KEY_Thai_sosua                  ,
    pattern KEY_Thai_soso                   ,
    pattern KEY_Thai_sosala                 ,
    pattern KEY_Thai_sorusi                 ,
    pattern KEY_Thai_sarauu                 ,
    pattern KEY_Thai_sarauee                ,
    pattern KEY_Thai_saraue                 ,
    pattern KEY_Thai_sarau                  ,
    pattern KEY_Thai_sarao                  ,
    pattern KEY_Thai_saraii                 ,
    pattern KEY_Thai_sarai                  ,
    pattern KEY_Thai_sarae                  ,
    pattern KEY_Thai_saraam                 ,
    pattern KEY_Thai_saraaimaimuan          ,
    pattern KEY_Thai_saraaimaimalai         ,
    pattern KEY_Thai_saraae                 ,
    pattern KEY_Thai_saraaa                 ,
    pattern KEY_Thai_saraa                  ,
    pattern KEY_Thai_ru                     ,
    pattern KEY_Thai_rorua                  ,
    pattern KEY_Thai_popla                  ,
    pattern KEY_Thai_phosamphao             ,
    pattern KEY_Thai_phophung               ,
    pattern KEY_Thai_phophan                ,
    pattern KEY_Thai_phinthu                ,
    pattern KEY_Thai_paiyannoi              ,
    pattern KEY_Thai_oang                   ,
    pattern KEY_Thai_nonu                   ,
    pattern KEY_Thai_nonen                  ,
    pattern KEY_Thai_nikhahit               ,
    pattern KEY_Thai_ngongu                 ,
    pattern KEY_Thai_moma                   ,
    pattern KEY_Thai_maiyamok               ,
    pattern KEY_Thai_maitri                 ,
    pattern KEY_Thai_maitho                 ,
    pattern KEY_Thai_maitaikhu              ,
    pattern KEY_Thai_maihanakat_maitho      ,
    pattern KEY_Thai_maihanakat             ,
    pattern KEY_Thai_maiek                  ,
    pattern KEY_Thai_maichattawa            ,
    pattern KEY_Thai_lu                     ,
    pattern KEY_Thai_loling                 ,
    pattern KEY_Thai_lochula                ,
    pattern KEY_Thai_leksun                 ,
    pattern KEY_Thai_leksong                ,
    pattern KEY_Thai_leksi                  ,
    pattern KEY_Thai_leksam                 ,
    pattern KEY_Thai_lekpaet                ,
    pattern KEY_Thai_leknung                ,
    pattern KEY_Thai_lekkao                 ,
    pattern KEY_Thai_lekhok                 ,
    pattern KEY_Thai_lekha                  ,
    pattern KEY_Thai_lekchet                ,
    pattern KEY_Thai_lakkhangyao            ,
    pattern KEY_Thai_kokai                  ,
    pattern KEY_Thai_khorakhang             ,
    pattern KEY_Thai_khokhwai               ,
    pattern KEY_Thai_khokhuat               ,
    pattern KEY_Thai_khokhon                ,
    pattern KEY_Thai_khokhai                ,
    pattern KEY_Thai_honokhuk               ,
    pattern KEY_Thai_hohip                  ,
    pattern KEY_Thai_fofan                  ,
    pattern KEY_Thai_fofa                   ,
    pattern KEY_Thai_dodek                  ,
    pattern KEY_Thai_dochada                ,
    pattern KEY_Thai_chochoe                ,
    pattern KEY_Thai_choching               ,
    pattern KEY_Thai_chochang               ,
    pattern KEY_Thai_chochan                ,
    pattern KEY_Thai_bobaimai               ,
    pattern KEY_Thai_baht                   ,
    pattern KEY_Terminate_Server            ,
    pattern KEY_Terminal                    ,
    pattern KEY_Tcedilla                    ,
    pattern KEY_Tcaron                      ,
    pattern KEY_Taskmanager                 ,
    pattern KEY_TaskPane                    ,
    pattern KEY_Tabovedot                   ,
    pattern KEY_Tab                         ,
    pattern KEY_THORN                       ,
    pattern KEY_T                           ,
    pattern KEY_Sys_Req                     ,
    pattern KEY_Switch_VT_9                 ,
    pattern KEY_Switch_VT_8                 ,
    pattern KEY_Switch_VT_7                 ,
    pattern KEY_Switch_VT_6                 ,
    pattern KEY_Switch_VT_5                 ,
    pattern KEY_Switch_VT_4                 ,
    pattern KEY_Switch_VT_3                 ,
    pattern KEY_Switch_VT_2                 ,
    pattern KEY_Switch_VT_12                ,
    pattern KEY_Switch_VT_11                ,
    pattern KEY_Switch_VT_10                ,
    pattern KEY_Switch_VT_1                 ,
    pattern KEY_Suspend                     ,
    pattern KEY_Support                     ,
    pattern KEY_Super_R                     ,
    pattern KEY_Super_L                     ,
    pattern KEY_Subtitle                    ,
    pattern KEY_StopRecord                  ,
    pattern KEY_Stop                        ,
    pattern KEY_StickyKeys_Enable           ,
    pattern KEY_Start                       ,
    pattern KEY_Standby                     ,
    pattern KEY_SplitScreen                 ,
    pattern KEY_SpellCheck                  ,
    pattern KEY_Spell                       ,
    pattern KEY_Sos                         ,
    pattern KEY_SlowReverse                 ,
    pattern KEY_SlowKeys_Enable             ,
    pattern KEY_Sleep                       ,
    pattern KEY_Sinh_ya                     ,
    pattern KEY_Sinh_va                     ,
    pattern KEY_Sinh_uu2                    ,
    pattern KEY_Sinh_uu                     ,
    pattern KEY_Sinh_u2                     ,
    pattern KEY_Sinh_u                      ,
    pattern KEY_Sinh_ttha                   ,
    pattern KEY_Sinh_tta                    ,
    pattern KEY_Sinh_thha                   ,
    pattern KEY_Sinh_tha                    ,
    pattern KEY_Sinh_ssha                   ,
    pattern KEY_Sinh_sha                    ,
    pattern KEY_Sinh_sa                     ,
    pattern KEY_Sinh_ruu2                   ,
    pattern KEY_Sinh_ru2                    ,
    pattern KEY_Sinh_rii                    ,
    pattern KEY_Sinh_ri                     ,
    pattern KEY_Sinh_ra                     ,
    pattern KEY_Sinh_pha                    ,
    pattern KEY_Sinh_pa                     ,
    pattern KEY_Sinh_oo2                    ,
    pattern KEY_Sinh_oo                     ,
    pattern KEY_Sinh_o2                     ,
    pattern KEY_Sinh_o                      ,
    pattern KEY_Sinh_nya                    ,
    pattern KEY_Sinh_nna                    ,
    pattern KEY_Sinh_nja                    ,
    pattern KEY_Sinh_nga                    ,
    pattern KEY_Sinh_ng2                    ,
    pattern KEY_Sinh_ng                     ,
    pattern KEY_Sinh_ndha                   ,
    pattern KEY_Sinh_ndda                   ,
    pattern KEY_Sinh_na                     ,
    pattern KEY_Sinh_mba                    ,
    pattern KEY_Sinh_ma                     ,
    pattern KEY_Sinh_luu2                   ,
    pattern KEY_Sinh_luu                    ,
    pattern KEY_Sinh_lu2                    ,
    pattern KEY_Sinh_lu                     ,
    pattern KEY_Sinh_lla                    ,
    pattern KEY_Sinh_la                     ,
    pattern KEY_Sinh_kunddaliya             ,
    pattern KEY_Sinh_kha                    ,
    pattern KEY_Sinh_ka                     ,
    pattern KEY_Sinh_jnya                   ,
    pattern KEY_Sinh_jha                    ,
    pattern KEY_Sinh_ja                     ,
    pattern KEY_Sinh_ii2                    ,
    pattern KEY_Sinh_ii                     ,
    pattern KEY_Sinh_i2                     ,
    pattern KEY_Sinh_i                      ,
    pattern KEY_Sinh_ha                     ,
    pattern KEY_Sinh_h2                     ,
    pattern KEY_Sinh_gha                    ,
    pattern KEY_Sinh_ga                     ,
    pattern KEY_Sinh_fa                     ,
    pattern KEY_Sinh_ee2                    ,
    pattern KEY_Sinh_ee                     ,
    pattern KEY_Sinh_e2                     ,
    pattern KEY_Sinh_e                      ,
    pattern KEY_Sinh_dhha                   ,
    pattern KEY_Sinh_dha                    ,
    pattern KEY_Sinh_ddha                   ,
    pattern KEY_Sinh_dda                    ,
    pattern KEY_Sinh_cha                    ,
    pattern KEY_Sinh_ca                     ,
    pattern KEY_Sinh_bha                    ,
    pattern KEY_Sinh_ba                     ,
    pattern KEY_Sinh_au2                    ,
    pattern KEY_Sinh_au                     ,
    pattern KEY_Sinh_al                     ,
    pattern KEY_Sinh_ai2                    ,
    pattern KEY_Sinh_ai                     ,
    pattern KEY_Sinh_aee2                   ,
    pattern KEY_Sinh_aee                    ,
    pattern KEY_Sinh_ae2                    ,
    pattern KEY_Sinh_ae                     ,
    pattern KEY_Sinh_aa2                    ,
    pattern KEY_Sinh_aa                     ,
    pattern KEY_Sinh_a                      ,
    pattern KEY_SingleRangeRadar            ,
    pattern KEY_SingleCandidate             ,
    pattern KEY_SidevuSonar                 ,
    pattern KEY_Shop                        ,
    pattern KEY_Shift_R                     ,
    pattern KEY_Shift_Lock                  ,
    pattern KEY_Shift_L                     ,
    pattern KEY_Serbian_tshe                ,
    pattern KEY_Serbian_nje                 ,
    pattern KEY_Serbian_lje                 ,
    pattern KEY_Serbian_je                  ,
    pattern KEY_Serbian_dze                 ,
    pattern KEY_Serbian_dje                 ,
    pattern KEY_Serbian_TSHE                ,
    pattern KEY_Serbian_NJE                 ,
    pattern KEY_Serbian_LJE                 ,
    pattern KEY_Serbian_JE                  ,
    pattern KEY_Serbian_DZE                 ,
    pattern KEY_Serbian_DJE                 ,
    pattern KEY_Send                        ,
    pattern KEY_SelectiveScreenshot         ,
    pattern KEY_SelectButton                ,
    pattern KEY_Select                      ,
    pattern KEY_Search                      ,
    pattern KEY_Scroll_Lock                 ,
    pattern KEY_ScrollUp                    ,
    pattern KEY_ScrollDown                  ,
    pattern KEY_ScrollClick                 ,
    pattern KEY_Screensaver                 ,
    pattern KEY_ScreenSaver                 ,
    pattern KEY_Scircumflex                 ,
    pattern KEY_Scedilla                    ,
    pattern KEY_Scaron                      ,
    pattern KEY_Save                        ,
    pattern KEY_Sacute                      ,
    pattern KEY_Sabovedot                   ,
    pattern KEY_SCHWA                       ,
    pattern KEY_S                           ,
    pattern KEY_RupeeSign                   ,
    pattern KEY_RotationPB                  ,
    pattern KEY_RotationLockToggle          ,
    pattern KEY_RotationKB                  ,
    pattern KEY_RotateWindows               ,
    pattern KEY_RootMenu                    ,
    pattern KEY_Romaji                      ,
    pattern KEY_RockerUp                    ,
    pattern KEY_RockerEnter                 ,
    pattern KEY_RockerDown                  ,
    pattern KEY_RightUp                     ,
    pattern KEY_RightDown                   ,
    pattern KEY_Right                       ,
    pattern KEY_Return                      ,
    pattern KEY_Reply                       ,
    pattern KEY_RepeatKeys_Enable           ,
    pattern KEY_Reload                      ,
    pattern KEY_RefreshRateToggle           ,
    pattern KEY_Refresh                     ,
    pattern KEY_Redo                        ,
    pattern KEY_Red                         ,
    pattern KEY_Rcedilla                    ,
    pattern KEY_Rcaron                      ,
    pattern KEY_RadarOverlay                ,
    pattern KEY_Racute                      ,
    pattern KEY_RFKill                      ,
    pattern KEY_R9                          ,
    pattern KEY_R8                          ,
    pattern KEY_R7                          ,
    pattern KEY_R6                          ,
    pattern KEY_R5                          ,
    pattern KEY_R4                          ,
    pattern KEY_R3                          ,
    pattern KEY_R2                          ,
    pattern KEY_R15                         ,
    pattern KEY_R14                         ,
    pattern KEY_R13                         ,
    pattern KEY_R12                         ,
    pattern KEY_R11                         ,
    pattern KEY_R10                         ,
    pattern KEY_R1                          ,
    pattern KEY_R                           ,
    pattern KEY_Q                           ,
    pattern KEY_PrivacyScreenToggle         ,
    pattern KEY_Prior                       ,
    pattern KEY_Print                       ,
    pattern KEY_PreviousElement             ,
    pattern KEY_PreviousCandidate           ,
    pattern KEY_Prev_Virtual_Screen         ,
    pattern KEY_Prev_VMode                  ,
    pattern KEY_Presentation                ,
    pattern KEY_PowerOff                    ,
    pattern KEY_PowerDown                   ,
    pattern KEY_Pointer_UpRight             ,
    pattern KEY_Pointer_UpLeft              ,
    pattern KEY_Pointer_Up                  ,
    pattern KEY_Pointer_Right               ,
    pattern KEY_Pointer_Left                ,
    pattern KEY_Pointer_EnableKeys          ,
    pattern KEY_Pointer_Drag_Dflt           ,
    pattern KEY_Pointer_Drag5               ,
    pattern KEY_Pointer_Drag4               ,
    pattern KEY_Pointer_Drag3               ,
    pattern KEY_Pointer_Drag2               ,
    pattern KEY_Pointer_Drag1               ,
    pattern KEY_Pointer_DownRight           ,
    pattern KEY_Pointer_DownLeft            ,
    pattern KEY_Pointer_Down                ,
    pattern KEY_Pointer_DfltBtnPrev         ,
    pattern KEY_Pointer_DfltBtnNext         ,
    pattern KEY_Pointer_DblClick_Dflt       ,
    pattern KEY_Pointer_DblClick5           ,
    pattern KEY_Pointer_DblClick4           ,
    pattern KEY_Pointer_DblClick3           ,
    pattern KEY_Pointer_DblClick2           ,
    pattern KEY_Pointer_DblClick1           ,
    pattern KEY_Pointer_Button_Dflt         ,
    pattern KEY_Pointer_Button5             ,
    pattern KEY_Pointer_Button4             ,
    pattern KEY_Pointer_Button3             ,
    pattern KEY_Pointer_Button2             ,
    pattern KEY_Pointer_Button1             ,
    pattern KEY_Pointer_Accelerate          ,
    pattern KEY_Pictures                    ,
    pattern KEY_PickupPhone                 ,
    pattern KEY_Phone                       ,
    pattern KEY_PesetaSign                  ,
    pattern KEY_PauseRecord                 ,
    pattern KEY_Pause                       ,
    pattern KEY_Paste                       ,
    pattern KEY_Page_Up                     ,
    pattern KEY_Page_Down                   ,
    pattern KEY_Pabovedot                   ,
    pattern KEY_P                           ,
    pattern KEY_Overlay2_Enable             ,
    pattern KEY_Overlay1_Enable             ,
    pattern KEY_Otilde                      ,
    pattern KEY_Oslash                      ,
    pattern KEY_Option                      ,
    pattern KEY_OpenURL                     ,
    pattern KEY_Open                        ,
    pattern KEY_Ooblique                    ,
    pattern KEY_OnScreenKeyboard            ,
    pattern KEY_Omacron                     ,
    pattern KEY_Ohorntilde                  ,
    pattern KEY_Ohornhook                   ,
    pattern KEY_Ohorngrave                  ,
    pattern KEY_Ohornbelowdot               ,
    pattern KEY_Ohornacute                  ,
    pattern KEY_Ohorn                       ,
    pattern KEY_Ohook                       ,
    pattern KEY_Ograve                      ,
    pattern KEY_OfficeHome                  ,
    pattern KEY_Odoubleacute                ,
    pattern KEY_Odiaeresis                  ,
    pattern KEY_Ocircumflextilde            ,
    pattern KEY_Ocircumflexhook             ,
    pattern KEY_Ocircumflexgrave            ,
    pattern KEY_Ocircumflexbelowdot         ,
    pattern KEY_Ocircumflexacute            ,
    pattern KEY_Ocircumflex                 ,
    pattern KEY_Ocaron                      ,
    pattern KEY_Obelowdot                   ,
    pattern KEY_Obarred                     ,
    pattern KEY_Oacute                      ,
    pattern KEY_OE                          ,
    pattern KEY_O                           ,
    pattern KEY_NumericStar                 ,
    pattern KEY_NumericPound                ,
    pattern KEY_NumericD                    ,
    pattern KEY_NumericC                    ,
    pattern KEY_NumericB                    ,
    pattern KEY_NumericA                    ,
    pattern KEY_Numeric9                    ,
    pattern KEY_Numeric8                    ,
    pattern KEY_Numeric7                    ,
    pattern KEY_Numeric6                    ,
    pattern KEY_Numeric5                    ,
    pattern KEY_Numeric4                    ,
    pattern KEY_Numeric3                    ,
    pattern KEY_Numeric2                    ,
    pattern KEY_Numeric12                   ,
    pattern KEY_Numeric11                   ,
    pattern KEY_Numeric1                    ,
    pattern KEY_Numeric0                    ,
    pattern KEY_Num_Lock                    ,
    pattern KEY_Ntilde                      ,
    pattern KEY_NotificationCenter          ,
    pattern KEY_Next_Virtual_Screen         ,
    pattern KEY_Next_VMode                  ,
    pattern KEY_NextFavorite                ,
    pattern KEY_NextElement                 ,
    pattern KEY_Next                        ,
    pattern KEY_News                        ,
    pattern KEY_NewSheqelSign               ,
    pattern KEY_New                         ,
    pattern KEY_Ncedilla                    ,
    pattern KEY_Ncaron                      ,
    pattern KEY_NavInfo                     ,
    pattern KEY_NavChart                    ,
    pattern KEY_NairaSign                   ,
    pattern KEY_Nacute                      ,
    pattern KEY_N                           ,
    pattern KEY_MySites                     ,
    pattern KEY_MyComputer                  ,
    pattern KEY_Music                       ,
    pattern KEY_MultipleCandidate           ,
    pattern KEY_Multi_key                   ,
    pattern KEY_Muhenkan                    ,
    pattern KEY_MouseKeys_Enable            ,
    pattern KEY_MouseKeys_Accel_Enable      ,
    pattern KEY_MonBrightnessUp             ,
    pattern KEY_MonBrightnessDown           ,
    pattern KEY_MonBrightnessCycle          ,
    pattern KEY_Mode_switch                 ,
    pattern KEY_ModeLock                    ,
    pattern KEY_MillSign                    ,
    pattern KEY_Meta_R                      ,
    pattern KEY_Meta_L                      ,
    pattern KEY_Messenger                   ,
    pattern KEY_MenuPB                      ,
    pattern KEY_MenuKB                      ,
    pattern KEY_Menu                        ,
    pattern KEY_Memo                        ,
    pattern KEY_Meeting                     ,
    pattern KEY_MediaTopMenu                ,
    pattern KEY_MediaRepeat                 ,
    pattern KEY_Massyo                      ,
    pattern KEY_Market                      ,
    pattern KEY_MarkWaypoint                ,
    pattern KEY_MailForward                 ,
    pattern KEY_Mail                        ,
    pattern KEY_Mae_Koho                    ,
    pattern KEY_MacroRecordStop             ,
    pattern KEY_MacroRecordStart            ,
    pattern KEY_MacroPresetCycle            ,
    pattern KEY_MacroPreset3                ,
    pattern KEY_MacroPreset2                ,
    pattern KEY_MacroPreset1                ,
    pattern KEY_Macro9                      ,
    pattern KEY_Macro8                      ,
    pattern KEY_Macro7                      ,
    pattern KEY_Macro6                      ,
    pattern KEY_Macro5                      ,
    pattern KEY_Macro4                      ,
    pattern KEY_Macro30                     ,
    pattern KEY_Macro3                      ,
    pattern KEY_Macro29                     ,
    pattern KEY_Macro28                     ,
    pattern KEY_Macro27                     ,
    pattern KEY_Macro26                     ,
    pattern KEY_Macro25                     ,
    pattern KEY_Macro24                     ,
    pattern KEY_Macro23                     ,
    pattern KEY_Macro22                     ,
    pattern KEY_Macro21                     ,
    pattern KEY_Macro20                     ,
    pattern KEY_Macro2                      ,
    pattern KEY_Macro19                     ,
    pattern KEY_Macro18                     ,
    pattern KEY_Macro17                     ,
    pattern KEY_Macro16                     ,
    pattern KEY_Macro15                     ,
    pattern KEY_Macro14                     ,
    pattern KEY_Macro13                     ,
    pattern KEY_Macro12                     ,
    pattern KEY_Macro11                     ,
    pattern KEY_Macro10                     ,
    pattern KEY_Macro1                      ,
    pattern KEY_Macedonia_kje               ,
    pattern KEY_Macedonia_gje               ,
    pattern KEY_Macedonia_dse               ,
    pattern KEY_Macedonia_KJE               ,
    pattern KEY_Macedonia_GJE               ,
    pattern KEY_Macedonia_DSE               ,
    pattern KEY_Mabovedot                   ,
    pattern KEY_M                           ,
    pattern KEY_Lstroke                     ,
    pattern KEY_LogWindowTree               ,
    pattern KEY_LogOff                      ,
    pattern KEY_LogGrabInfo                 ,
    pattern KEY_LiraSign                    ,
    pattern KEY_Linefeed                    ,
    pattern KEY_LightsToggle                ,
    pattern KEY_LightBulb                   ,
    pattern KEY_LeftUp                      ,
    pattern KEY_LeftDown                    ,
    pattern KEY_Left                        ,
    pattern KEY_Lcedilla                    ,
    pattern KEY_Lcaron                      ,
    pattern KEY_Lbelowdot                   ,
    pattern KEY_LaunchF                     ,
    pattern KEY_LaunchE                     ,
    pattern KEY_LaunchD                     ,
    pattern KEY_LaunchC                     ,
    pattern KEY_LaunchB                     ,
    pattern KEY_LaunchA                     ,
    pattern KEY_Launch9                     ,
    pattern KEY_Launch8                     ,
    pattern KEY_Launch7                     ,
    pattern KEY_Launch6                     ,
    pattern KEY_Launch5                     ,
    pattern KEY_Launch4                     ,
    pattern KEY_Launch3                     ,
    pattern KEY_Launch2                     ,
    pattern KEY_Launch1                     ,
    pattern KEY_Launch0                     ,
    pattern KEY_Last_Virtual_Screen         ,
    pattern KEY_Lacute                      ,
    pattern KEY_L9                          ,
    pattern KEY_L8                          ,
    pattern KEY_L7                          ,
    pattern KEY_L6                          ,
    pattern KEY_L5                          ,
    pattern KEY_L4                          ,
    pattern KEY_L3                          ,
    pattern KEY_L2                          ,
    pattern KEY_L10                         ,
    pattern KEY_L1                          ,
    pattern KEY_L                           ,
    pattern KEY_Korean_Won                  ,
    pattern KEY_Keyboard                    ,
    pattern KEY_Kcedilla                    ,
    pattern KEY_KbdLightOnOff               ,
    pattern KEY_KbdLcdMenu5                 ,
    pattern KEY_KbdLcdMenu4                 ,
    pattern KEY_KbdLcdMenu3                 ,
    pattern KEY_KbdLcdMenu2                 ,
    pattern KEY_KbdLcdMenu1                 ,
    pattern KEY_KbdInputAssistPrevgroup     ,
    pattern KEY_KbdInputAssistPrev          ,
    pattern KEY_KbdInputAssistNextgroup     ,
    pattern KEY_KbdInputAssistNext          ,
    pattern KEY_KbdInputAssistCancel        ,
    pattern KEY_KbdInputAssistAccept        ,
    pattern KEY_KbdBrightnessUp             ,
    pattern KEY_KbdBrightnessDown           ,
    pattern KEY_Katakana                    ,
    pattern KEY_Kanji_Bangou                ,
    pattern KEY_Kanji                       ,
    pattern KEY_Kana_Shift                  ,
    pattern KEY_Kana_Lock                   ,
    pattern KEY_KP_Up                       ,
    pattern KEY_KP_Tab                      ,
    pattern KEY_KP_Subtract                 ,
    pattern KEY_KP_Space                    ,
    pattern KEY_KP_Separator                ,
    pattern KEY_KP_Right                    ,
    pattern KEY_KP_Prior                    ,
    pattern KEY_KP_Page_Up                  ,
    pattern KEY_KP_Page_Down                ,
    pattern KEY_KP_Next                     ,
    pattern KEY_KP_Multiply                 ,
    pattern KEY_KP_Left                     ,
    pattern KEY_KP_Insert                   ,
    pattern KEY_KP_Home                     ,
    pattern KEY_KP_F4                       ,
    pattern KEY_KP_F3                       ,
    pattern KEY_KP_F2                       ,
    pattern KEY_KP_F1                       ,
    pattern KEY_KP_Equal                    ,
    pattern KEY_KP_Enter                    ,
    pattern KEY_KP_End                      ,
    pattern KEY_KP_Down                     ,
    pattern KEY_KP_Divide                   ,
    pattern KEY_KP_Delete                   ,
    pattern KEY_KP_Decimal                  ,
    pattern KEY_KP_Begin                    ,
    pattern KEY_KP_Add                      ,
    pattern KEY_KP_9                        ,
    pattern KEY_KP_8                        ,
    pattern KEY_KP_7                        ,
    pattern KEY_KP_6                        ,
    pattern KEY_KP_5                        ,
    pattern KEY_KP_4                        ,
    pattern KEY_KP_3                        ,
    pattern KEY_KP_2                        ,
    pattern KEY_KP_1                        ,
    pattern KEY_KP_0                        ,
    pattern KEY_K                           ,
    pattern KEY_Journal                     ,
    pattern KEY_Jcircumflex                 ,
    pattern KEY_J                           ,
    pattern KEY_Itilde                      ,
    pattern KEY_Iogonek                     ,
    pattern KEY_Insert                      ,
    pattern KEY_Info                        ,
    pattern KEY_Images                      ,
    pattern KEY_Imacron                     ,
    pattern KEY_Ihook                       ,
    pattern KEY_Igrave                      ,
    pattern KEY_Idiaeresis                  ,
    pattern KEY_Icircumflex                 ,
    pattern KEY_Ibreve                      ,
    pattern KEY_Ibelowdot                   ,
    pattern KEY_Iacute                      ,
    pattern KEY_Iabovedot                   ,
    pattern KEY_ISO_Set_Margin_Right        ,
    pattern KEY_ISO_Set_Margin_Left         ,
    pattern KEY_ISO_Release_Margin_Right    ,
    pattern KEY_ISO_Release_Margin_Left     ,
    pattern KEY_ISO_Release_Both_Margins    ,
    pattern KEY_ISO_Prev_Group_Lock         ,
    pattern KEY_ISO_Prev_Group              ,
    pattern KEY_ISO_Partial_Space_Right     ,
    pattern KEY_ISO_Partial_Space_Left      ,
    pattern KEY_ISO_Partial_Line_Up         ,
    pattern KEY_ISO_Partial_Line_Down       ,
    pattern KEY_ISO_Next_Group_Lock         ,
    pattern KEY_ISO_Next_Group              ,
    pattern KEY_ISO_Move_Line_Up            ,
    pattern KEY_ISO_Move_Line_Down          ,
    pattern KEY_ISO_Lock                    ,
    pattern KEY_ISO_Level5_Shift            ,
    pattern KEY_ISO_Level5_Lock             ,
    pattern KEY_ISO_Level5_Latch            ,
    pattern KEY_ISO_Level3_Shift            ,
    pattern KEY_ISO_Level3_Lock             ,
    pattern KEY_ISO_Level3_Latch            ,
    pattern KEY_ISO_Level2_Latch            ,
    pattern KEY_ISO_Left_Tab                ,
    pattern KEY_ISO_Last_Group_Lock         ,
    pattern KEY_ISO_Last_Group              ,
    pattern KEY_ISO_Group_Shift             ,
    pattern KEY_ISO_Group_Lock              ,
    pattern KEY_ISO_Group_Latch             ,
    pattern KEY_ISO_First_Group_Lock        ,
    pattern KEY_ISO_First_Group             ,
    pattern KEY_ISO_Fast_Cursor_Up          ,
    pattern KEY_ISO_Fast_Cursor_Right       ,
    pattern KEY_ISO_Fast_Cursor_Left        ,
    pattern KEY_ISO_Fast_Cursor_Down        ,
    pattern KEY_ISO_Enter                   ,
    pattern KEY_ISO_Emphasize               ,
    pattern KEY_ISO_Discontinuous_Underline ,
    pattern KEY_ISO_Continuous_Underline    ,
    pattern KEY_ISO_Center_Object           ,
    pattern KEY_I                           ,
    pattern KEY_Hyper_R                     ,
    pattern KEY_Hyper_L                     ,
    pattern KEY_Hstroke                     ,
    pattern KEY_HotLinks                    ,
    pattern KEY_HomePage                    ,
    pattern KEY_Home                        ,
    pattern KEY_History                     ,
    pattern KEY_Hiragana_Katakana           ,
    pattern KEY_Hiragana                    ,
    pattern KEY_Hibernate                   ,
    pattern KEY_Henkan_Mode                 ,
    pattern KEY_Henkan                      ,
    pattern KEY_Help                        ,
    pattern KEY_Hebrew_switch               ,
    pattern KEY_Hcircumflex                 ,
    pattern KEY_Hankaku                     ,
    pattern KEY_HangupPhone                 ,
    pattern KEY_Hangul_switch               ,
    pattern KEY_Hangul_YeorinHieuh          ,
    pattern KEY_Hangul_YU                   ,
    pattern KEY_Hangul_YO                   ,
    pattern KEY_Hangul_YI                   ,
    pattern KEY_Hangul_YEO                  ,
    pattern KEY_Hangul_YE                   ,
    pattern KEY_Hangul_YAE                  ,
    pattern KEY_Hangul_YA                   ,
    pattern KEY_Hangul_WI                   ,
    pattern KEY_Hangul_WEO                  ,
    pattern KEY_Hangul_WE                   ,
    pattern KEY_Hangul_WAE                  ,
    pattern KEY_Hangul_WA                   ,
    pattern KEY_Hangul_U                    ,
    pattern KEY_Hangul_Tieut                ,
    pattern KEY_Hangul_SunkyeongeumPieub    ,
    pattern KEY_Hangul_SunkyeongeumPhieuf   ,
    pattern KEY_Hangul_SunkyeongeumMieum    ,
    pattern KEY_Hangul_Start                ,
    pattern KEY_Hangul_SsangSios            ,
    pattern KEY_Hangul_SsangPieub           ,
    pattern KEY_Hangul_SsangKiyeog          ,
    pattern KEY_Hangul_SsangJieuj           ,
    pattern KEY_Hangul_SsangDikeud          ,
    pattern KEY_Hangul_Special              ,
    pattern KEY_Hangul_Sios                 ,
    pattern KEY_Hangul_SingleCandidate      ,
    pattern KEY_Hangul_Romaja               ,
    pattern KEY_Hangul_RieulYeorinHieuh     ,
    pattern KEY_Hangul_RieulTieut           ,
    pattern KEY_Hangul_RieulSios            ,
    pattern KEY_Hangul_RieulPieub           ,
    pattern KEY_Hangul_RieulPhieuf          ,
    pattern KEY_Hangul_RieulMieum           ,
    pattern KEY_Hangul_RieulKiyeog          ,
    pattern KEY_Hangul_RieulHieuh           ,
    pattern KEY_Hangul_Rieul                ,
    pattern KEY_Hangul_PreviousCandidate    ,
    pattern KEY_Hangul_PreHanja             ,
    pattern KEY_Hangul_PostHanja            ,
    pattern KEY_Hangul_PieubSios            ,
    pattern KEY_Hangul_Pieub                ,
    pattern KEY_Hangul_Phieuf               ,
    pattern KEY_Hangul_PanSios              ,
    pattern KEY_Hangul_OE                   ,
    pattern KEY_Hangul_O                    ,
    pattern KEY_Hangul_NieunJieuj           ,
    pattern KEY_Hangul_NieunHieuh           ,
    pattern KEY_Hangul_Nieun                ,
    pattern KEY_Hangul_MultipleCandidate    ,
    pattern KEY_Hangul_Mieum                ,
    pattern KEY_Hangul_KkogjiDalrinIeung    ,
    pattern KEY_Hangul_KiyeogSios           ,
    pattern KEY_Hangul_Kiyeog               ,
    pattern KEY_Hangul_Khieuq               ,
    pattern KEY_Hangul_Jieuj                ,
    pattern KEY_Hangul_Jeonja               ,
    pattern KEY_Hangul_Jamo                 ,
    pattern KEY_Hangul_J_YeorinHieuh        ,
    pattern KEY_Hangul_J_Tieut              ,
    pattern KEY_Hangul_J_SsangSios          ,
    pattern KEY_Hangul_J_SsangKiyeog        ,
    pattern KEY_Hangul_J_Sios               ,
    pattern KEY_Hangul_J_RieulTieut         ,
    pattern KEY_Hangul_J_RieulSios          ,
    pattern KEY_Hangul_J_RieulPieub         ,
    pattern KEY_Hangul_J_RieulPhieuf        ,
    pattern KEY_Hangul_J_RieulMieum         ,
    pattern KEY_Hangul_J_RieulKiyeog        ,
    pattern KEY_Hangul_J_RieulHieuh         ,
    pattern KEY_Hangul_J_Rieul              ,
    pattern KEY_Hangul_J_PieubSios          ,
    pattern KEY_Hangul_J_Pieub              ,
    pattern KEY_Hangul_J_Phieuf             ,
    pattern KEY_Hangul_J_PanSios            ,
    pattern KEY_Hangul_J_NieunJieuj         ,
    pattern KEY_Hangul_J_NieunHieuh         ,
    pattern KEY_Hangul_J_Nieun              ,
    pattern KEY_Hangul_J_Mieum              ,
    pattern KEY_Hangul_J_KkogjiDalrinIeung  ,
    pattern KEY_Hangul_J_KiyeogSios         ,
    pattern KEY_Hangul_J_Kiyeog             ,
    pattern KEY_Hangul_J_Khieuq             ,
    pattern KEY_Hangul_J_Jieuj              ,
    pattern KEY_Hangul_J_Ieung              ,
    pattern KEY_Hangul_J_Hieuh              ,
    pattern KEY_Hangul_J_Dikeud             ,
    pattern KEY_Hangul_J_Cieuc              ,
    pattern KEY_Hangul_Ieung                ,
    pattern KEY_Hangul_I                    ,
    pattern KEY_Hangul_Hieuh                ,
    pattern KEY_Hangul_Hanja                ,
    pattern KEY_Hangul_End                  ,
    pattern KEY_Hangul_EU                   ,
    pattern KEY_Hangul_EO                   ,
    pattern KEY_Hangul_E                    ,
    pattern KEY_Hangul_Dikeud               ,
    pattern KEY_Hangul_Codeinput            ,
    pattern KEY_Hangul_Cieuc                ,
    pattern KEY_Hangul_Banja                ,
    pattern KEY_Hangul_AraeAE               ,
    pattern KEY_Hangul_AraeA                ,
    pattern KEY_Hangul_AE                   ,
    pattern KEY_Hangul_A                    ,
    pattern KEY_Hangul                      ,
    pattern KEY_H                           ,
    pattern KEY_Green                       ,
    pattern KEY_Greek_zeta                  ,
    pattern KEY_Greek_xi                    ,
    pattern KEY_Greek_upsilondieresis       ,
    pattern KEY_Greek_upsilonaccentdieresis ,
    pattern KEY_Greek_upsilonaccent         ,
    pattern KEY_Greek_upsilon               ,
    pattern KEY_Greek_theta                 ,
    pattern KEY_Greek_tau                   ,
    pattern KEY_Greek_switch                ,
    pattern KEY_Greek_sigma                 ,
    pattern KEY_Greek_rho                   ,
    pattern KEY_Greek_psi                   ,
    pattern KEY_Greek_pi                    ,
    pattern KEY_Greek_phi                   ,
    pattern KEY_Greek_omicronaccent         ,
    pattern KEY_Greek_omicron               ,
    pattern KEY_Greek_omegaaccent           ,
    pattern KEY_Greek_omega                 ,
    pattern KEY_Greek_nu                    ,
    pattern KEY_Greek_mu                    ,
    pattern KEY_Greek_lamda                 ,
    pattern KEY_Greek_lambda                ,
    pattern KEY_Greek_kappa                 ,
    pattern KEY_Greek_iotadieresis          ,
    pattern KEY_Greek_iotaaccentdieresis    ,
    pattern KEY_Greek_iotaaccent            ,
    pattern KEY_Greek_iota                  ,
    pattern KEY_Greek_horizbar              ,
    pattern KEY_Greek_gamma                 ,
    pattern KEY_Greek_finalsmallsigma       ,
    pattern KEY_Greek_etaaccent             ,
    pattern KEY_Greek_eta                   ,
    pattern KEY_Greek_epsilonaccent         ,
    pattern KEY_Greek_epsilon               ,
    pattern KEY_Greek_delta                 ,
    pattern KEY_Greek_chi                   ,
    pattern KEY_Greek_beta                  ,
    pattern KEY_Greek_alphaaccent           ,
    pattern KEY_Greek_alpha                 ,
    pattern KEY_Greek_accentdieresis        ,
    pattern KEY_Greek_ZETA                  ,
    pattern KEY_Greek_XI                    ,
    pattern KEY_Greek_UPSILONdieresis       ,
    pattern KEY_Greek_UPSILONaccent         ,
    pattern KEY_Greek_UPSILON               ,
    pattern KEY_Greek_THETA                 ,
    pattern KEY_Greek_TAU                   ,
    pattern KEY_Greek_SIGMA                 ,
    pattern KEY_Greek_RHO                   ,
    pattern KEY_Greek_PSI                   ,
    pattern KEY_Greek_PI                    ,
    pattern KEY_Greek_PHI                   ,
    pattern KEY_Greek_OMICRONaccent         ,
    pattern KEY_Greek_OMICRON               ,
    pattern KEY_Greek_OMEGAaccent           ,
    pattern KEY_Greek_OMEGA                 ,
    pattern KEY_Greek_NU                    ,
    pattern KEY_Greek_MU                    ,
    pattern KEY_Greek_LAMDA                 ,
    pattern KEY_Greek_LAMBDA                ,
    pattern KEY_Greek_KAPPA                 ,
    pattern KEY_Greek_IOTAdieresis          ,
    pattern KEY_Greek_IOTAdiaeresis         ,
    pattern KEY_Greek_IOTAaccent            ,
    pattern KEY_Greek_IOTA                  ,
    pattern KEY_Greek_GAMMA                 ,
    pattern KEY_Greek_ETAaccent             ,
    pattern KEY_Greek_ETA                   ,
    pattern KEY_Greek_EPSILONaccent         ,
    pattern KEY_Greek_EPSILON               ,
    pattern KEY_Greek_DELTA                 ,
    pattern KEY_Greek_CHI                   ,
    pattern KEY_Greek_BETA                  ,
    pattern KEY_Greek_ALPHAaccent           ,
    pattern KEY_Greek_ALPHA                 ,
    pattern KEY_GraphicsEditor              ,
    pattern KEY_Go                          ,
    pattern KEY_Georgian_zhar               ,
    pattern KEY_Georgian_zen                ,
    pattern KEY_Georgian_xan                ,
    pattern KEY_Georgian_we                 ,
    pattern KEY_Georgian_vin                ,
    pattern KEY_Georgian_un                 ,
    pattern KEY_Georgian_tar                ,
    pattern KEY_Georgian_tan                ,
    pattern KEY_Georgian_shin               ,
    pattern KEY_Georgian_san                ,
    pattern KEY_Georgian_rae                ,
    pattern KEY_Georgian_qar                ,
    pattern KEY_Georgian_phar               ,
    pattern KEY_Georgian_par                ,
    pattern KEY_Georgian_on                 ,
    pattern KEY_Georgian_nar                ,
    pattern KEY_Georgian_man                ,
    pattern KEY_Georgian_las                ,
    pattern KEY_Georgian_khar               ,
    pattern KEY_Georgian_kan                ,
    pattern KEY_Georgian_jil                ,
    pattern KEY_Georgian_jhan               ,
    pattern KEY_Georgian_in                 ,
    pattern KEY_Georgian_hoe                ,
    pattern KEY_Georgian_hie                ,
    pattern KEY_Georgian_he                 ,
    pattern KEY_Georgian_har                ,
    pattern KEY_Georgian_hae                ,
    pattern KEY_Georgian_ghan               ,
    pattern KEY_Georgian_gan                ,
    pattern KEY_Georgian_fi                 ,
    pattern KEY_Georgian_en                 ,
    pattern KEY_Georgian_don                ,
    pattern KEY_Georgian_cil                ,
    pattern KEY_Georgian_chin               ,
    pattern KEY_Georgian_char               ,
    pattern KEY_Georgian_can                ,
    pattern KEY_Georgian_ban                ,
    pattern KEY_Georgian_an                 ,
    pattern KEY_Gcircumflex                 ,
    pattern KEY_Gcedilla                    ,
    pattern KEY_Gcaron                      ,
    pattern KEY_Gbreve                      ,
    pattern KEY_Game                        ,
    pattern KEY_Gabovedot                   ,
    pattern KEY_G                           ,
    pattern KEY_FullScreen                  ,
    pattern KEY_FrameForward                ,
    pattern KEY_FrameBack                   ,
    pattern KEY_Forward                     ,
    pattern KEY_Fn_Esc                      ,
    pattern KEY_FnRightShift                ,
    pattern KEY_Fn                          ,
    pattern KEY_FishingChart                ,
    pattern KEY_First_Virtual_Screen        ,
    pattern KEY_Find                        ,
    pattern KEY_Finance                     ,
    pattern KEY_Favorites                   ,
    pattern KEY_FastReverse                 ,
    pattern KEY_Farsi_yeh                   ,
    pattern KEY_Farsi_9                     ,
    pattern KEY_Farsi_8                     ,
    pattern KEY_Farsi_7                     ,
    pattern KEY_Farsi_6                     ,
    pattern KEY_Farsi_5                     ,
    pattern KEY_Farsi_4                     ,
    pattern KEY_Farsi_3                     ,
    pattern KEY_Farsi_2                     ,
    pattern KEY_Farsi_1                     ,
    pattern KEY_Farsi_0                     ,
    pattern KEY_Fabovedot                   ,
    pattern KEY_FFrancSign                  ,
    pattern KEY_F9                          ,
    pattern KEY_F8                          ,
    pattern KEY_F7                          ,
    pattern KEY_F6                          ,
    pattern KEY_F5                          ,
    pattern KEY_F4                          ,
    pattern KEY_F35                         ,
    pattern KEY_F34                         ,
    pattern KEY_F33                         ,
    pattern KEY_F32                         ,
    pattern KEY_F31                         ,
    pattern KEY_F30                         ,
    pattern KEY_F3                          ,
    pattern KEY_F29                         ,
    pattern KEY_F28                         ,
    pattern KEY_F27                         ,
    pattern KEY_F26                         ,
    pattern KEY_F25                         ,
    pattern KEY_F24                         ,
    pattern KEY_F23                         ,
    pattern KEY_F22                         ,
    pattern KEY_F21                         ,
    pattern KEY_F20                         ,
    pattern KEY_F2                          ,
    pattern KEY_F19                         ,
    pattern KEY_F18                         ,
    pattern KEY_F17                         ,
    pattern KEY_F16                         ,
    pattern KEY_F15                         ,
    pattern KEY_F14                         ,
    pattern KEY_F13                         ,
    pattern KEY_F12                         ,
    pattern KEY_F11                         ,
    pattern KEY_F10                         ,
    pattern KEY_F1                          ,
    pattern KEY_F                           ,
    pattern KEY_Explorer                    ,
    pattern KEY_Execute                     ,
    pattern KEY_Excel                       ,
    pattern KEY_EuroSign                    ,
    pattern KEY_Etilde                      ,
    pattern KEY_Eth                         ,
    pattern KEY_Escape                      ,
    pattern KEY_Eogonek                     ,
    pattern KEY_End                         ,
    pattern KEY_EmojiPicker                 ,
    pattern KEY_Emacron                     ,
    pattern KEY_Eject                       ,
    pattern KEY_Eisu_toggle                 ,
    pattern KEY_Eisu_Shift                  ,
    pattern KEY_Ehook                       ,
    pattern KEY_Egrave                      ,
    pattern KEY_Editor                      ,
    pattern KEY_Ediaeresis                  ,
    pattern KEY_EcuSign                     ,
    pattern KEY_Ecircumflextilde            ,
    pattern KEY_Ecircumflexhook             ,
    pattern KEY_Ecircumflexgrave            ,
    pattern KEY_Ecircumflexbelowdot         ,
    pattern KEY_Ecircumflexacute            ,
    pattern KEY_Ecircumflex                 ,
    pattern KEY_Ecaron                      ,
    pattern KEY_Ebelowdot                   ,
    pattern KEY_Eacute                      ,
    pattern KEY_Eabovedot                   ,
    pattern KEY_EZH                         ,
    pattern KEY_ETH                         ,
    pattern KEY_ENG                         ,
    pattern KEY_E                           ,
    pattern KEY_DualRangeRadar              ,
    pattern KEY_Dstroke                     ,
    pattern KEY_Down                        ,
    pattern KEY_DongSign                    ,
    pattern KEY_Documents                   ,
    pattern KEY_DoNotDisturb                ,
    pattern KEY_DisplayToggle               ,
    pattern KEY_DisplayOff                  ,
    pattern KEY_Display                     ,
    pattern KEY_Dictate                     ,
    pattern KEY_Delete                      ,
    pattern KEY_Dcaron                      ,
    pattern KEY_Database                    ,
    pattern KEY_Data                        ,
    pattern KEY_Dabovedot                   ,
    pattern KEY_DVD                         ,
    pattern KEY_DOS                         ,
    pattern KEY_D                           ,
    pattern KEY_Cyrillic_zhe_descender      ,
    pattern KEY_Cyrillic_zhe                ,
    pattern KEY_Cyrillic_ze                 ,
    pattern KEY_Cyrillic_yu                 ,
    pattern KEY_Cyrillic_yeru               ,
    pattern KEY_Cyrillic_ya                 ,
    pattern KEY_Cyrillic_ve                 ,
    pattern KEY_Cyrillic_u_straight_bar     ,
    pattern KEY_Cyrillic_u_straight         ,
    pattern KEY_Cyrillic_u_macron           ,
    pattern KEY_Cyrillic_u                  ,
    pattern KEY_Cyrillic_tse                ,
    pattern KEY_Cyrillic_te                 ,
    pattern KEY_Cyrillic_softsign           ,
    pattern KEY_Cyrillic_shorti             ,
    pattern KEY_Cyrillic_shha               ,
    pattern KEY_Cyrillic_shcha              ,
    pattern KEY_Cyrillic_sha                ,
    pattern KEY_Cyrillic_schwa              ,
    pattern KEY_Cyrillic_pe                 ,
    pattern KEY_Cyrillic_o_bar              ,
    pattern KEY_Cyrillic_o                  ,
    pattern KEY_Cyrillic_nje                ,
    pattern KEY_Cyrillic_lje                ,
    pattern KEY_Cyrillic_ka_vertstroke      ,
    pattern KEY_Cyrillic_ka_descender       ,
    pattern KEY_Cyrillic_ka                 ,
    pattern KEY_Cyrillic_je                 ,
    pattern KEY_Cyrillic_io                 ,
    pattern KEY_Cyrillic_ie                 ,
    pattern KEY_Cyrillic_i_macron           ,
    pattern KEY_Cyrillic_i                  ,
    pattern KEY_Cyrillic_hardsign           ,
    pattern KEY_Cyrillic_ha_descender       ,
    pattern KEY_Cyrillic_ha                 ,
    pattern KEY_Cyrillic_ghe_bar            ,
    pattern KEY_Cyrillic_ghe                ,
    pattern KEY_Cyrillic_es                 ,
    pattern KEY_Cyrillic_er                 ,
    pattern KEY_Cyrillic_en_descender       ,
    pattern KEY_Cyrillic_en                 ,
    pattern KEY_Cyrillic_em                 ,
    pattern KEY_Cyrillic_el                 ,
    pattern KEY_Cyrillic_ef                 ,
    pattern KEY_Cyrillic_e                  ,
    pattern KEY_Cyrillic_dzhe               ,
    pattern KEY_Cyrillic_de                 ,
    pattern KEY_Cyrillic_che_vertstroke     ,
    pattern KEY_Cyrillic_che_descender      ,
    pattern KEY_Cyrillic_che                ,
    pattern KEY_Cyrillic_be                 ,
    pattern KEY_Cyrillic_a                  ,
    pattern KEY_Cyrillic_ZHE_descender      ,
    pattern KEY_Cyrillic_ZHE                ,
    pattern KEY_Cyrillic_ZE                 ,
    pattern KEY_Cyrillic_YU                 ,
    pattern KEY_Cyrillic_YERU               ,
    pattern KEY_Cyrillic_YA                 ,
    pattern KEY_Cyrillic_VE                 ,
    pattern KEY_Cyrillic_U_straight_bar     ,
    pattern KEY_Cyrillic_U_straight         ,
    pattern KEY_Cyrillic_U_macron           ,
    pattern KEY_Cyrillic_U                  ,
    pattern KEY_Cyrillic_TSE                ,
    pattern KEY_Cyrillic_TE                 ,
    pattern KEY_Cyrillic_SOFTSIGN           ,
    pattern KEY_Cyrillic_SHORTI             ,
    pattern KEY_Cyrillic_SHHA               ,
    pattern KEY_Cyrillic_SHCHA              ,
    pattern KEY_Cyrillic_SHA                ,
    pattern KEY_Cyrillic_SCHWA              ,
    pattern KEY_Cyrillic_PE                 ,
    pattern KEY_Cyrillic_O_bar              ,
    pattern KEY_Cyrillic_O                  ,
    pattern KEY_Cyrillic_NJE                ,
    pattern KEY_Cyrillic_LJE                ,
    pattern KEY_Cyrillic_KA_vertstroke      ,
    pattern KEY_Cyrillic_KA_descender       ,
    pattern KEY_Cyrillic_KA                 ,
    pattern KEY_Cyrillic_JE                 ,
    pattern KEY_Cyrillic_I_macron           ,
    pattern KEY_Cyrillic_IO                 ,
    pattern KEY_Cyrillic_IE                 ,
    pattern KEY_Cyrillic_I                  ,
    pattern KEY_Cyrillic_HA_descender       ,
    pattern KEY_Cyrillic_HARDSIGN           ,
    pattern KEY_Cyrillic_HA                 ,
    pattern KEY_Cyrillic_GHE_bar            ,
    pattern KEY_Cyrillic_GHE                ,
    pattern KEY_Cyrillic_ES                 ,
    pattern KEY_Cyrillic_ER                 ,
    pattern KEY_Cyrillic_EN_descender       ,
    pattern KEY_Cyrillic_EN                 ,
    pattern KEY_Cyrillic_EM                 ,
    pattern KEY_Cyrillic_EL                 ,
    pattern KEY_Cyrillic_EF                 ,
    pattern KEY_Cyrillic_E                  ,
    pattern KEY_Cyrillic_DZHE               ,
    pattern KEY_Cyrillic_DE                 ,
    pattern KEY_Cyrillic_CHE_vertstroke     ,
    pattern KEY_Cyrillic_CHE_descender      ,
    pattern KEY_Cyrillic_CHE                ,
    pattern KEY_Cyrillic_BE                 ,
    pattern KEY_Cyrillic_A                  ,
    pattern KEY_CycleAngle                  ,
    pattern KEY_Cut                         ,
    pattern KEY_CruzeiroSign                ,
    pattern KEY_Copy                        ,
    pattern KEY_Control_R                   ,
    pattern KEY_Control_L                   ,
    pattern KEY_ControlPanel                ,
    pattern KEY_ContrastAdjust              ,
    pattern KEY_ContextMenu                 ,
    pattern KEY_Community                   ,
    pattern KEY_ColonSign                   ,
    pattern KEY_Codeinput                   ,
    pattern KEY_Close                       ,
    pattern KEY_ClearvuSonar                ,
    pattern KEY_ClearGrab                   ,
    pattern KEY_Clear                       ,
    pattern KEY_ChannelUp                   ,
    pattern KEY_ChannelDown                 ,
    pattern KEY_Ch                          ,
    pattern KEY_Ccircumflex                 ,
    pattern KEY_Ccedilla                    ,
    pattern KEY_Ccaron                      ,
    pattern KEY_Caps_Lock                   ,
    pattern KEY_Cancel                      ,
    pattern KEY_CameraZoomOut               ,
    pattern KEY_CameraZoomIn                ,
    pattern KEY_CameraUp                    ,
    pattern KEY_CameraRight                 ,
    pattern KEY_CameraLeft                  ,
    pattern KEY_CameraFocus                 ,
    pattern KEY_CameraDown                  ,
    pattern KEY_CameraAccessToggle          ,
    pattern KEY_CameraAccessEnable          ,
    pattern KEY_CameraAccessDisable         ,
    pattern KEY_Calendar                    ,
    pattern KEY_Calculator                  ,
    pattern KEY_Cacute                      ,
    pattern KEY_Cabovedot                   ,
    pattern KEY_C_h                         ,
    pattern KEY_C_H                         ,
    pattern KEY_CH                          ,
    pattern KEY_CD                          ,
    pattern KEY_C                           ,
    pattern KEY_Byelorussian_shortu         ,
    pattern KEY_Byelorussian_SHORTU         ,
    pattern KEY_Buttonconfig                ,
    pattern KEY_BrightnessMin               ,
    pattern KEY_BrightnessMax               ,
    pattern KEY_BrightnessAuto              ,
    pattern KEY_BrightnessAdjust            ,
    pattern KEY_Break                       ,
    pattern KEY_BounceKeys_Enable           ,
    pattern KEY_Book                        ,
    pattern KEY_Bluetooth                   ,
    pattern KEY_Blue                        ,
    pattern KEY_Begin                       ,
    pattern KEY_Battery                     ,
    pattern KEY_BackSpace                   ,
    pattern KEY_BackForward                 ,
    pattern KEY_Back                        ,
    pattern KEY_Babovedot                   ,
    pattern KEY_B                           ,
    pattern KEY_Away                        ,
    pattern KEY_AutopilotEngageToggle       ,
    pattern KEY_AudioStop                   ,
    pattern KEY_AudioRewind                 ,
    pattern KEY_AudioRepeat                 ,
    pattern KEY_AudioRecord                 ,
    pattern KEY_AudioRandomPlay             ,
    pattern KEY_AudioRaiseVolume            ,
    pattern KEY_AudioPrev                   ,
    pattern KEY_AudioPreset                 ,
    pattern KEY_AudioPlay                   ,
    pattern KEY_AudioPause                  ,
    pattern KEY_AudioNext                   ,
    pattern KEY_AudioMute                   ,
    pattern KEY_AudioMicMute                ,
    pattern KEY_AudioMedia                  ,
    pattern KEY_AudioLowerVolume            ,
    pattern KEY_AudioForward                ,
    pattern KEY_AudioDesc                   ,
    pattern KEY_AudioCycleTrack             ,
    pattern KEY_Audio                       ,
    pattern KEY_AudibleBell_Enable          ,
    pattern KEY_AttendantToggle             ,
    pattern KEY_AttendantOn                 ,
    pattern KEY_AttendantOff                ,
    pattern KEY_Atilde                      ,
    pattern KEY_Assistant                   ,
    pattern KEY_AspectRatio                 ,
    pattern KEY_Armenian_zhe                ,
    pattern KEY_Armenian_za                 ,
    pattern KEY_Armenian_yentamna           ,
    pattern KEY_Armenian_yech               ,
    pattern KEY_Armenian_vyun               ,
    pattern KEY_Armenian_vo                 ,
    pattern KEY_Armenian_vev                ,
    pattern KEY_Armenian_verjaket           ,
    pattern KEY_Armenian_tyun               ,
    pattern KEY_Armenian_tso                ,
    pattern KEY_Armenian_tsa                ,
    pattern KEY_Armenian_to                 ,
    pattern KEY_Armenian_tche               ,
    pattern KEY_Armenian_shesht             ,
    pattern KEY_Armenian_sha                ,
    pattern KEY_Armenian_separation_mark    ,
    pattern KEY_Armenian_se                 ,
    pattern KEY_Armenian_re                 ,
    pattern KEY_Armenian_ra                 ,
    pattern KEY_Armenian_question           ,
    pattern KEY_Armenian_pyur               ,
    pattern KEY_Armenian_pe                 ,
    pattern KEY_Armenian_paruyk             ,
    pattern KEY_Armenian_o                  ,
    pattern KEY_Armenian_nu                 ,
    pattern KEY_Armenian_men                ,
    pattern KEY_Armenian_lyun               ,
    pattern KEY_Armenian_ligature_ew        ,
    pattern KEY_Armenian_khe                ,
    pattern KEY_Armenian_ken                ,
    pattern KEY_Armenian_ke                 ,
    pattern KEY_Armenian_je                 ,
    pattern KEY_Armenian_ini                ,
    pattern KEY_Armenian_hyphen             ,
    pattern KEY_Armenian_ho                 ,
    pattern KEY_Armenian_hi                 ,
    pattern KEY_Armenian_gim                ,
    pattern KEY_Armenian_ghat               ,
    pattern KEY_Armenian_full_stop          ,
    pattern KEY_Armenian_fe                 ,
    pattern KEY_Armenian_exclam             ,
    pattern KEY_Armenian_e                  ,
    pattern KEY_Armenian_dza                ,
    pattern KEY_Armenian_da                 ,
    pattern KEY_Armenian_cha                ,
    pattern KEY_Armenian_but                ,
    pattern KEY_Armenian_ben                ,
    pattern KEY_Armenian_ayb                ,
    pattern KEY_Armenian_at                 ,
    pattern KEY_Armenian_apostrophe         ,
    pattern KEY_Armenian_amanak             ,
    pattern KEY_Armenian_accent             ,
    pattern KEY_Armenian_ZHE                ,
    pattern KEY_Armenian_ZA                 ,
    pattern KEY_Armenian_YECH               ,
    pattern KEY_Armenian_VYUN               ,
    pattern KEY_Armenian_VO                 ,
    pattern KEY_Armenian_VEV                ,
    pattern KEY_Armenian_TYUN               ,
    pattern KEY_Armenian_TSO                ,
    pattern KEY_Armenian_TSA                ,
    pattern KEY_Armenian_TO                 ,
    pattern KEY_Armenian_TCHE               ,
    pattern KEY_Armenian_SHA                ,
    pattern KEY_Armenian_SE                 ,
    pattern KEY_Armenian_RE                 ,
    pattern KEY_Armenian_RA                 ,
    pattern KEY_Armenian_PYUR               ,
    pattern KEY_Armenian_PE                 ,
    pattern KEY_Armenian_O                  ,
    pattern KEY_Armenian_NU                 ,
    pattern KEY_Armenian_MEN                ,
    pattern KEY_Armenian_LYUN               ,
    pattern KEY_Armenian_KHE                ,
    pattern KEY_Armenian_KEN                ,
    pattern KEY_Armenian_KE                 ,
    pattern KEY_Armenian_JE                 ,
    pattern KEY_Armenian_INI                ,
    pattern KEY_Armenian_HO                 ,
    pattern KEY_Armenian_HI                 ,
    pattern KEY_Armenian_GIM                ,
    pattern KEY_Armenian_GHAT               ,
    pattern KEY_Armenian_FE                 ,
    pattern KEY_Armenian_E                  ,
    pattern KEY_Armenian_DZA                ,
    pattern KEY_Armenian_DA                 ,
    pattern KEY_Armenian_CHA                ,
    pattern KEY_Armenian_BEN                ,
    pattern KEY_Armenian_AYB                ,
    pattern KEY_Armenian_AT                 ,
    pattern KEY_Aring                       ,
    pattern KEY_Arabic_zain                 ,
    pattern KEY_Arabic_zah                  ,
    pattern KEY_Arabic_yeh_baree            ,
    pattern KEY_Arabic_yeh                  ,
    pattern KEY_Arabic_waw                  ,
    pattern KEY_Arabic_veh                  ,
    pattern KEY_Arabic_tteh                 ,
    pattern KEY_Arabic_theh                 ,
    pattern KEY_Arabic_thal                 ,
    pattern KEY_Arabic_tehmarbuta           ,
    pattern KEY_Arabic_teh                  ,
    pattern KEY_Arabic_tcheh                ,
    pattern KEY_Arabic_tatweel              ,
    pattern KEY_Arabic_tah                  ,
    pattern KEY_Arabic_switch               ,
    pattern KEY_Arabic_superscript_alef     ,
    pattern KEY_Arabic_sukun                ,
    pattern KEY_Arabic_sheen                ,
    pattern KEY_Arabic_shadda               ,
    pattern KEY_Arabic_semicolon            ,
    pattern KEY_Arabic_seen                 ,
    pattern KEY_Arabic_sad                  ,
    pattern KEY_Arabic_rreh                 ,
    pattern KEY_Arabic_ra                   ,
    pattern KEY_Arabic_question_mark        ,
    pattern KEY_Arabic_qaf                  ,
    pattern KEY_Arabic_percent              ,
    pattern KEY_Arabic_peh                  ,
    pattern KEY_Arabic_noon_ghunna          ,
    pattern KEY_Arabic_noon                 ,
    pattern KEY_Arabic_meem                 ,
    pattern KEY_Arabic_maddaonalef          ,
    pattern KEY_Arabic_madda_above          ,
    pattern KEY_Arabic_lam                  ,
    pattern KEY_Arabic_khah                 ,
    pattern KEY_Arabic_keheh                ,
    pattern KEY_Arabic_kasratan             ,
    pattern KEY_Arabic_kasra                ,
    pattern KEY_Arabic_kaf                  ,
    pattern KEY_Arabic_jeh                  ,
    pattern KEY_Arabic_jeem                 ,
    pattern KEY_Arabic_heh_goal             ,
    pattern KEY_Arabic_heh_doachashmee      ,
    pattern KEY_Arabic_heh                  ,
    pattern KEY_Arabic_hamzaunderalef       ,
    pattern KEY_Arabic_hamzaonyeh           ,
    pattern KEY_Arabic_hamzaonwaw           ,
    pattern KEY_Arabic_hamzaonalef          ,
    pattern KEY_Arabic_hamza_below          ,
    pattern KEY_Arabic_hamza_above          ,
    pattern KEY_Arabic_hamza                ,
    pattern KEY_Arabic_hah                  ,
    pattern KEY_Arabic_ha                   ,
    pattern KEY_Arabic_ghain                ,
    pattern KEY_Arabic_gaf                  ,
    pattern KEY_Arabic_fullstop             ,
    pattern KEY_Arabic_feh                  ,
    pattern KEY_Arabic_fathatan             ,
    pattern KEY_Arabic_fatha                ,
    pattern KEY_Arabic_farsi_yeh            ,
    pattern KEY_Arabic_ddal                 ,
    pattern KEY_Arabic_dammatan             ,
    pattern KEY_Arabic_damma                ,
    pattern KEY_Arabic_dal                  ,
    pattern KEY_Arabic_dad                  ,
    pattern KEY_Arabic_comma                ,
    pattern KEY_Arabic_beh                  ,
    pattern KEY_Arabic_alefmaksura          ,
    pattern KEY_Arabic_alef                 ,
    pattern KEY_Arabic_ain                  ,
    pattern KEY_Arabic_9                    ,
    pattern KEY_Arabic_8                    ,
    pattern KEY_Arabic_7                    ,
    pattern KEY_Arabic_6                    ,
    pattern KEY_Arabic_5                    ,
    pattern KEY_Arabic_4                    ,
    pattern KEY_Arabic_3                    ,
    pattern KEY_Arabic_2                    ,
    pattern KEY_Arabic_1                    ,
    pattern KEY_Arabic_0                    ,
    pattern KEY_ApplicationRight            ,
    pattern KEY_ApplicationLeft             ,
    pattern KEY_AppSelect                   ,
    pattern KEY_Aogonek                     ,
    pattern KEY_Amacron                     ,
    pattern KEY_Alt_R                       ,
    pattern KEY_Alt_L                       ,
    pattern KEY_Ahook                       ,
    pattern KEY_Agrave                      ,
    pattern KEY_Adiaeresis                  ,
    pattern KEY_Addressbook                 ,
    pattern KEY_AddFavorite                 ,
    pattern KEY_Acircumflextilde            ,
    pattern KEY_Acircumflexhook             ,
    pattern KEY_Acircumflexgrave            ,
    pattern KEY_Acircumflexbelowdot         ,
    pattern KEY_Acircumflexacute            ,
    pattern KEY_Acircumflex                 ,
    pattern KEY_Accessibility               ,
    pattern KEY_AccessX_Feedback_Enable     ,
    pattern KEY_AccessX_Enable              ,
    pattern KEY_Abrevetilde                 ,
    pattern KEY_Abrevehook                  ,
    pattern KEY_Abrevegrave                 ,
    pattern KEY_Abrevebelowdot              ,
    pattern KEY_Abreveacute                 ,
    pattern KEY_Abreve                      ,
    pattern KEY_Abelowdot                   ,
    pattern KEY_Aacute                      ,
    pattern KEY_ALSToggle                   ,
    pattern KEY_AE                          ,
    pattern KEY_A                           ,
    pattern KEY_9                           ,
    pattern KEY_8                           ,
    pattern KEY_7                           ,
    pattern KEY_6                           ,
    pattern KEY_5                           ,
    pattern KEY_4                           ,
    pattern KEY_3DMode                      ,
    pattern KEY_3270_Test                   ,
    pattern KEY_3270_Setup                  ,
    pattern KEY_3270_Rule                   ,
    pattern KEY_3270_Right2                 ,
    pattern KEY_3270_Reset                  ,
    pattern KEY_3270_Record                 ,
    pattern KEY_3270_Quit                   ,
    pattern KEY_3270_PrintScreen            ,
    pattern KEY_3270_Play                   ,
    pattern KEY_3270_PA3                    ,
    pattern KEY_3270_PA2                    ,
    pattern KEY_3270_PA1                    ,
    pattern KEY_3270_Left2                  ,
    pattern KEY_3270_KeyClick               ,
    pattern KEY_3270_Jump                   ,
    pattern KEY_3270_Ident                  ,
    pattern KEY_3270_FieldMark              ,
    pattern KEY_3270_ExSelect               ,
    pattern KEY_3270_EraseInput             ,
    pattern KEY_3270_EraseEOF               ,
    pattern KEY_3270_Enter                  ,
    pattern KEY_3270_Duplicate              ,
    pattern KEY_3270_DeleteWord             ,
    pattern KEY_3270_CursorSelect           ,
    pattern KEY_3270_CursorBlink            ,
    pattern KEY_3270_Copy                   ,
    pattern KEY_3270_ChangeScreen           ,
    pattern KEY_3270_BackTab                ,
    pattern KEY_3270_Attn                   ,
    pattern KEY_3270_AltCursor              ,
    pattern KEY_3                           ,
    pattern KEY_2                           ,
    pattern KEY_10ChannelsUp                ,
    pattern KEY_10ChannelsDown              ,
    pattern KEY_1                           ,
    pattern KEY_0                           ,
    pattern EVENT_STOP                      ,
    pattern EVENT_PROPAGATE                 ,
    pattern CURRENT_TIME                    ,
    pattern BUTTON_SECONDARY                ,
    pattern BUTTON_PRIMARY                  ,
    pattern BUTTON_MIDDLE                   ,
    pattern ACTION_ALL                      ,

    ) where

import Data.GI.Base.ShortPrelude
import qualified Data.GI.Base.ShortPrelude as SP
import qualified Data.GI.Base.Overloading as O
import qualified Prelude as P

import qualified Data.GI.Base.Attributes as GI.Attributes
import qualified Data.GI.Base.BasicTypes as B.Types
import qualified Data.GI.Base.ManagedPtr as B.ManagedPtr
import qualified Data.GI.Base.GArray as B.GArray
import qualified Data.GI.Base.GClosure as B.GClosure
import qualified Data.GI.Base.GError as B.GError
import qualified Data.GI.Base.GHashTable as B.GHT
import qualified Data.GI.Base.GVariant as B.GVariant
import qualified Data.GI.Base.GValue as B.GValue
import qualified Data.GI.Base.GParamSpec as B.GParamSpec
import qualified Data.GI.Base.CallStack as B.CallStack
import qualified Data.GI.Base.Properties as B.Properties
import qualified Data.GI.Base.Signals as B.Signals
import qualified Control.Monad.IO.Class as MIO
import qualified Data.Coerce as Coerce
import qualified Data.Text as T
import qualified Data.Kind as DK
import qualified Data.ByteString.Char8 as B
import qualified Data.Map as Map
import qualified Foreign.Ptr as FP
import qualified GHC.OverloadedLabels as OL
import qualified GHC.Records as R
import qualified Data.Word as DW
import qualified Data.Int as DI
import qualified System.Posix.Types as SPT
import qualified Foreign.C.Types as FCT

-- Workaround for https://gitlab.haskell.org/ghc/ghc/-/issues/23392
#if MIN_VERSION_base(4,18,0)

#else

#endif

-- | This is the priority that the idle handler processing surface updates
-- is given in the main loop.
pattern PRIORITY_REDRAW = 120 :: Int32

-- | A mask covering all entries in @GdkModifierType@.
pattern MODIFIER_MASK = 469769999 :: Int32

-- | /No description available in the introspection data./
pattern KEY_zstroke = 16777654 :: Word32

-- | /No description available in the introspection data./
pattern KEY_zerosuperior = 16785520 :: Word32

-- | /No description available in the introspection data./
pattern KEY_zerosubscript = 16785536 :: Word32

-- | /No description available in the introspection data./
pattern KEY_zcaron = 446 :: Word32

-- | /No description available in the introspection data./
pattern KEY_zacute = 444 :: Word32

-- | /No description available in the introspection data./
pattern KEY_zabovedot = 447 :: Word32

-- | /No description available in the introspection data./
pattern KEY_z = 122 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ytilde = 16785145 :: Word32

-- | /No description available in the introspection data./
pattern KEY_yhook = 16785143 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ygrave = 16785139 :: Word32

-- | /No description available in the introspection data./
pattern KEY_yen = 165 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ydiaeresis = 255 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ycircumflex = 16777591 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ybelowdot = 16785141 :: Word32

-- | /No description available in the introspection data./
pattern KEY_yacute = 253 :: Word32

-- | /No description available in the introspection data./
pattern KEY_y = 121 :: Word32

-- | /No description available in the introspection data./
pattern KEY_xabovedot = 16785035 :: Word32

-- | /No description available in the introspection data./
pattern KEY_x = 120 :: Word32

-- | /No description available in the introspection data./
pattern KEY_wgrave = 16785025 :: Word32

-- | /No description available in the introspection data./
pattern KEY_wdiaeresis = 16785029 :: Word32

-- | /No description available in the introspection data./
pattern KEY_wcircumflex = 16777589 :: Word32

-- | /No description available in the introspection data./
pattern KEY_wacute = 16785027 :: Word32

-- | /No description available in the introspection data./
pattern KEY_w = 119 :: Word32

-- | /No description available in the introspection data./
pattern KEY_vt = 2537 :: Word32

-- | /No description available in the introspection data./
pattern KEY_voicedsound = 1246 :: Word32

-- | /No description available in the introspection data./
pattern KEY_vertconnector = 2214 :: Word32

-- | /No description available in the introspection data./
pattern KEY_vertbar = 2552 :: Word32

-- | /No description available in the introspection data./
pattern KEY_variation = 2241 :: Word32

-- | /No description available in the introspection data./
pattern KEY_v = 118 :: Word32

-- | /No description available in the introspection data./
pattern KEY_utilde = 1021 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uring = 505 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uptack = 3022 :: Word32

-- | /No description available in the introspection data./
pattern KEY_upstile = 3027 :: Word32

-- | /No description available in the introspection data./
pattern KEY_upshoe = 3011 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uprightcorner = 2539 :: Word32

-- | /No description available in the introspection data./
pattern KEY_upleftcorner = 2540 :: Word32

-- | /No description available in the introspection data./
pattern KEY_upcaret = 2985 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uparrow = 2300 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uogonek = 1017 :: Word32

-- | /No description available in the introspection data./
pattern KEY_union = 2269 :: Word32

-- | /No description available in the introspection data./
pattern KEY_underscore = 95 :: Word32

-- | /No description available in the introspection data./
pattern KEY_underbar = 3014 :: Word32

-- | /No description available in the introspection data./
pattern KEY_umacron = 1022 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uhorntilde = 16785135 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uhornhook = 16785133 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uhorngrave = 16785131 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uhornbelowdot = 16785137 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uhornacute = 16785129 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uhorn = 16777648 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uhook = 16785127 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ugrave = 249 :: Word32

-- | /No description available in the introspection data./
pattern KEY_udoubleacute = 507 :: Word32

-- | /No description available in the introspection data./
pattern KEY_udiaeresis = 252 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ucircumflex = 251 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ubreve = 765 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ubelowdot = 16785125 :: Word32

-- | /No description available in the introspection data./
pattern KEY_uacute = 250 :: Word32

-- | /No description available in the introspection data./
pattern KEY_u = 117 :: Word32

-- | /No description available in the introspection data./
pattern KEY_twothirds = 2737 :: Word32

-- | /No description available in the introspection data./
pattern KEY_twosuperior = 178 :: Word32

-- | /No description available in the introspection data./
pattern KEY_twosubscript = 16785538 :: Word32

-- | /No description available in the introspection data./
pattern KEY_twofifths = 2739 :: Word32

-- | /No description available in the introspection data./
pattern KEY_tslash = 956 :: Word32

-- | /No description available in the introspection data./
pattern KEY_trademarkincircle = 2763 :: Word32

-- | /No description available in the introspection data./
pattern KEY_trademark = 2761 :: Word32

-- | /No description available in the introspection data./
pattern KEY_topvertsummationconnector = 2227 :: Word32

-- | /No description available in the introspection data./
pattern KEY_topt = 2551 :: Word32

-- | /No description available in the introspection data./
pattern KEY_toprightsummation = 2229 :: Word32

-- | /No description available in the introspection data./
pattern KEY_toprightsqbracket = 2217 :: Word32

-- | /No description available in the introspection data./
pattern KEY_toprightparens = 2221 :: Word32

-- | /No description available in the introspection data./
pattern KEY_topleftsummation = 2225 :: Word32

-- | /No description available in the introspection data./
pattern KEY_topleftsqbracket = 2215 :: Word32

-- | /No description available in the introspection data./
pattern KEY_topleftradical = 2210 :: Word32

-- | /No description available in the introspection data./
pattern KEY_topleftparens = 2219 :: Word32

-- | /No description available in the introspection data./
pattern KEY_topintegral = 2212 :: Word32

-- | /No description available in the introspection data./
pattern KEY_tintegral = 16785965 :: Word32

-- | /No description available in the introspection data./
pattern KEY_threesuperior = 179 :: Word32

-- | /No description available in the introspection data./
pattern KEY_threesubscript = 16785539 :: Word32

-- | /No description available in the introspection data./
pattern KEY_threequarters = 190 :: Word32

-- | /No description available in the introspection data./
pattern KEY_threefifths = 2740 :: Word32

-- | /No description available in the introspection data./
pattern KEY_threeeighths = 2756 :: Word32

-- | /No description available in the introspection data./
pattern KEY_thorn = 254 :: Word32

-- | /No description available in the introspection data./
pattern KEY_thinspace = 2727 :: Word32

-- | /No description available in the introspection data./
pattern KEY_therefore = 2240 :: Word32

-- | /No description available in the introspection data./
pattern KEY_telephonerecorder = 2810 :: Word32

-- | /No description available in the introspection data./
pattern KEY_telephone = 2809 :: Word32

-- | /No description available in the introspection data./
pattern KEY_tcedilla = 510 :: Word32

-- | /No description available in the introspection data./
pattern KEY_tcaron = 443 :: Word32

-- | /No description available in the introspection data./
pattern KEY_tabovedot = 16785003 :: Word32

-- | /No description available in the introspection data./
pattern KEY_t = 116 :: Word32

-- | /No description available in the introspection data./
pattern KEY_stricteq = 16786019 :: Word32

-- | /No description available in the introspection data./
pattern KEY_sterling = 163 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ssharp = 223 :: Word32

-- | /No description available in the introspection data./
pattern KEY_squareroot = 16785946 :: Word32

-- | /No description available in the introspection data./
pattern KEY_space = 32 :: Word32

-- | /No description available in the introspection data./
pattern KEY_soliddiamond = 2528 :: Word32

-- | /No description available in the introspection data./
pattern KEY_slash = 47 :: Word32

-- | /No description available in the introspection data./
pattern KEY_sixsuperior = 16785526 :: Word32

-- | /No description available in the introspection data./
pattern KEY_sixsubscript = 16785542 :: Word32

-- | /No description available in the introspection data./
pattern KEY_singlelowquotemark = 2813 :: Word32

-- | /No description available in the introspection data./
pattern KEY_similarequal = 2249 :: Word32

-- | /No description available in the introspection data./
pattern KEY_signifblank = 2732 :: Word32

-- | /No description available in the introspection data./
pattern KEY_signaturemark = 2762 :: Word32

-- | /No description available in the introspection data./
pattern KEY_sevensuperior = 16785527 :: Word32

-- | /No description available in the introspection data./
pattern KEY_sevensubscript = 16785543 :: Word32

-- | /No description available in the introspection data./
pattern KEY_seveneighths = 2758 :: Word32

-- | /No description available in the introspection data./
pattern KEY_semivoicedsound = 1247 :: Word32

-- | /No description available in the introspection data./
pattern KEY_semicolon = 59 :: Word32

-- | /No description available in the introspection data./
pattern KEY_section = 167 :: Word32

-- | /No description available in the introspection data./
pattern KEY_seconds = 2775 :: Word32

-- | /No description available in the introspection data./
pattern KEY_script_switch = 65406 :: Word32

-- | /No description available in the introspection data./
pattern KEY_scircumflex = 766 :: Word32

-- | /No description available in the introspection data./
pattern KEY_schwa = 16777817 :: Word32

-- | /No description available in the introspection data./
pattern KEY_scedilla = 442 :: Word32

-- | /No description available in the introspection data./
pattern KEY_scaron = 441 :: Word32

-- | /No description available in the introspection data./
pattern KEY_sacute = 438 :: Word32

-- | /No description available in the introspection data./
pattern KEY_sabovedot = 16784993 :: Word32

-- | /No description available in the introspection data./
pattern KEY_s = 115 :: Word32

-- | /No description available in the introspection data./
pattern KEY_righttack = 3068 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rightt = 2549 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rightsinglequotemark = 2769 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rightshoe = 3032 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rightpointer = 2795 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rightopentriangle = 2765 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rightmiddlesummation = 2231 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rightmiddlecurlybrace = 2224 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rightdoublequotemark = 2771 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rightcaret = 2982 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rightarrow = 2301 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rightanglebracket = 2750 :: Word32

-- | /No description available in the introspection data./
pattern KEY_registered = 174 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rcedilla = 947 :: Word32

-- | /No description available in the introspection data./
pattern KEY_rcaron = 504 :: Word32

-- | /No description available in the introspection data./
pattern KEY_radical = 2262 :: Word32

-- | /No description available in the introspection data./
pattern KEY_racute = 480 :: Word32

-- | /No description available in the introspection data./
pattern KEY_r = 114 :: Word32

-- | /No description available in the introspection data./
pattern KEY_quoteright = 39 :: Word32

-- | /No description available in the introspection data./
pattern KEY_quoteleft = 96 :: Word32

-- | /No description available in the introspection data./
pattern KEY_quotedbl = 34 :: Word32

-- | /No description available in the introspection data./
pattern KEY_questiondown = 191 :: Word32

-- | /No description available in the introspection data./
pattern KEY_question = 63 :: Word32

-- | /No description available in the introspection data./
pattern KEY_quad = 3020 :: Word32

-- | /No description available in the introspection data./
pattern KEY_q = 113 :: Word32

-- | /No description available in the introspection data./
pattern KEY_punctspace = 2726 :: Word32

-- | /No description available in the introspection data./
pattern KEY_prolongedsound = 1200 :: Word32

-- | /No description available in the introspection data./
pattern KEY_prescription = 2772 :: Word32

-- | /No description available in the introspection data./
pattern KEY_plusminus = 177 :: Word32

-- | /No description available in the introspection data./
pattern KEY_plus = 43 :: Word32

-- | /No description available in the introspection data./
pattern KEY_phonographcopyright = 2811 :: Word32

-- | /No description available in the introspection data./
pattern KEY_permille = 2773 :: Word32

-- | /No description available in the introspection data./
pattern KEY_periodcentered = 183 :: Word32

-- | /No description available in the introspection data./
pattern KEY_period = 46 :: Word32

-- | /No description available in the introspection data./
pattern KEY_percent = 37 :: Word32

-- | /No description available in the introspection data./
pattern KEY_partialderivative = 2287 :: Word32

-- | /No description available in the introspection data./
pattern KEY_partdifferential = 16785922 :: Word32

-- | /No description available in the introspection data./
pattern KEY_parenright = 41 :: Word32

-- | /No description available in the introspection data./
pattern KEY_parenleft = 40 :: Word32

-- | /No description available in the introspection data./
pattern KEY_paragraph = 182 :: Word32

-- | /No description available in the introspection data./
pattern KEY_pabovedot = 16784983 :: Word32

-- | /No description available in the introspection data./
pattern KEY_p = 112 :: Word32

-- | /No description available in the introspection data./
pattern KEY_overline = 1150 :: Word32

-- | /No description available in the introspection data./
pattern KEY_overbar = 3008 :: Word32

-- | /No description available in the introspection data./
pattern KEY_otilde = 245 :: Word32

-- | /No description available in the introspection data./
pattern KEY_oslash = 248 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ordmasculine = 186 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ordfeminine = 170 :: Word32

-- | /No description available in the introspection data./
pattern KEY_opentribulletup = 2787 :: Word32

-- | /No description available in the introspection data./
pattern KEY_opentribulletdown = 2788 :: Word32

-- | /No description available in the introspection data./
pattern KEY_openstar = 2789 :: Word32

-- | /No description available in the introspection data./
pattern KEY_openrectbullet = 2786 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ooblique = 248 :: Word32

-- | /No description available in the introspection data./
pattern KEY_onethird = 2736 :: Word32

-- | /No description available in the introspection data./
pattern KEY_onesuperior = 185 :: Word32

-- | /No description available in the introspection data./
pattern KEY_onesubscript = 16785537 :: Word32

-- | /No description available in the introspection data./
pattern KEY_onesixth = 2742 :: Word32

-- | /No description available in the introspection data./
pattern KEY_onequarter = 188 :: Word32

-- | /No description available in the introspection data./
pattern KEY_onehalf = 189 :: Word32

-- | /No description available in the introspection data./
pattern KEY_onefifth = 2738 :: Word32

-- | /No description available in the introspection data./
pattern KEY_oneeighth = 2755 :: Word32

-- | /No description available in the introspection data./
pattern KEY_omacron = 1010 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ohorntilde = 16785121 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ohornhook = 16785119 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ohorngrave = 16785117 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ohornbelowdot = 16785123 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ohornacute = 16785115 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ohorn = 16777633 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ohook = 16785103 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ograve = 242 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ogonek = 434 :: Word32

-- | /No description available in the introspection data./
pattern KEY_oe = 5053 :: Word32

-- | /No description available in the introspection data./
pattern KEY_odoubleacute = 501 :: Word32

-- | /No description available in the introspection data./
pattern KEY_odiaeresis = 246 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ocircumflextilde = 16785111 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ocircumflexhook = 16785109 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ocircumflexgrave = 16785107 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ocircumflexbelowdot = 16785113 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ocircumflexacute = 16785105 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ocircumflex = 244 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ocaron = 16777682 :: Word32

-- | /No description available in the introspection data./
pattern KEY_obelowdot = 16785101 :: Word32

-- | /No description available in the introspection data./
pattern KEY_obarred = 16777845 :: Word32

-- | /No description available in the introspection data./
pattern KEY_oacute = 243 :: Word32

-- | /No description available in the introspection data./
pattern KEY_o = 111 :: Word32

-- | /No description available in the introspection data./
pattern KEY_numerosign = 1712 :: Word32

-- | /No description available in the introspection data./
pattern KEY_numbersign = 35 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ntilde = 241 :: Word32

-- | /No description available in the introspection data./
pattern KEY_notsign = 172 :: Word32

-- | /No description available in the introspection data./
pattern KEY_notidentical = 16786018 :: Word32

-- | /No description available in the introspection data./
pattern KEY_notequal = 2237 :: Word32

-- | /No description available in the introspection data./
pattern KEY_notelementof = 16785929 :: Word32

-- | /No description available in the introspection data./
pattern KEY_notapproxeq = 16785991 :: Word32

-- | /No description available in the introspection data./
pattern KEY_nobreakspace = 160 :: Word32

-- | /No description available in the introspection data./
pattern KEY_nl = 2536 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ninesuperior = 16785529 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ninesubscript = 16785545 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ncedilla = 1009 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ncaron = 498 :: Word32

-- | /No description available in the introspection data./
pattern KEY_nacute = 497 :: Word32

-- | /No description available in the introspection data./
pattern KEY_nabla = 2245 :: Word32

-- | /No description available in the introspection data./
pattern KEY_n = 110 :: Word32

-- | /No description available in the introspection data./
pattern KEY_musicalsharp = 2805 :: Word32

-- | /No description available in the introspection data./
pattern KEY_musicalflat = 2806 :: Word32

-- | /No description available in the introspection data./
pattern KEY_multiply = 215 :: Word32

-- | /No description available in the introspection data./
pattern KEY_mu = 181 :: Word32

-- | /No description available in the introspection data./
pattern KEY_minutes = 2774 :: Word32

-- | /No description available in the introspection data./
pattern KEY_minus = 45 :: Word32

-- | /No description available in the introspection data./
pattern KEY_masculine = 186 :: Word32

-- | /No description available in the introspection data./
pattern KEY_marker = 2751 :: Word32

-- | /No description available in the introspection data./
pattern KEY_maltesecross = 2800 :: Word32

-- | /No description available in the introspection data./
pattern KEY_malesymbol = 2807 :: Word32

-- | /No description available in the introspection data./
pattern KEY_macron = 175 :: Word32

-- | /No description available in the introspection data./
pattern KEY_mabovedot = 16784961 :: Word32

-- | /No description available in the introspection data./
pattern KEY_m = 109 :: Word32

-- | /No description available in the introspection data./
pattern KEY_lstroke = 435 :: Word32

-- | /No description available in the introspection data./
pattern KEY_lowrightcorner = 2538 :: Word32

-- | /No description available in the introspection data./
pattern KEY_lowleftcorner = 2541 :: Word32

-- | /No description available in the introspection data./
pattern KEY_logicalor = 2271 :: Word32

-- | /No description available in the introspection data./
pattern KEY_logicaland = 2270 :: Word32

-- | /No description available in the introspection data./
pattern KEY_lf = 2533 :: Word32

-- | /No description available in the introspection data./
pattern KEY_lessthanequal = 2236 :: Word32

-- | /No description available in the introspection data./
pattern KEY_less = 60 :: Word32

-- | /No description available in the introspection data./
pattern KEY_lefttack = 3036 :: Word32

-- | /No description available in the introspection data./
pattern KEY_leftt = 2548 :: Word32

-- | /No description available in the introspection data./
pattern KEY_leftsinglequotemark = 2768 :: Word32

-- | /No description available in the introspection data./
pattern KEY_leftshoe = 3034 :: Word32

-- | /No description available in the introspection data./
pattern KEY_leftradical = 2209 :: Word32

-- | /No description available in the introspection data./
pattern KEY_leftpointer = 2794 :: Word32

-- | /No description available in the introspection data./
pattern KEY_leftopentriangle = 2764 :: Word32

-- | /No description available in the introspection data./
pattern KEY_leftmiddlecurlybrace = 2223 :: Word32

-- | /No description available in the introspection data./
pattern KEY_leftdoublequotemark = 2770 :: Word32

-- | /No description available in the introspection data./
pattern KEY_leftcaret = 2979 :: Word32

-- | /No description available in the introspection data./
pattern KEY_leftarrow = 2299 :: Word32

-- | /No description available in the introspection data./
pattern KEY_leftanglebracket = 2748 :: Word32

-- | /No description available in the introspection data./
pattern KEY_lcedilla = 950 :: Word32

-- | /No description available in the introspection data./
pattern KEY_lcaron = 437 :: Word32

-- | /No description available in the introspection data./
pattern KEY_lbelowdot = 16784951 :: Word32

-- | /No description available in the introspection data./
pattern KEY_latincross = 2777 :: Word32

-- | /No description available in the introspection data./
pattern KEY_lacute = 485 :: Word32

-- | /No description available in the introspection data./
pattern KEY_l = 108 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kra = 930 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kcedilla = 1011 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kappa = 930 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_yu = 1197 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_yo = 1198 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_ya = 1196 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_u = 1193 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_tu = 1199 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_tsu = 1199 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_switch = 65406 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_openingbracket = 1186 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_o = 1195 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_middledot = 1189 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_i = 1192 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_fullstop = 1185 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_e = 1194 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_conjunctive = 1189 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_comma = 1188 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_closingbracket = 1187 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_a = 1191 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_YU = 1237 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_YO = 1238 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_YA = 1236 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_WO = 1190 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_WA = 1244 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_U = 1203 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_TU = 1218 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_TSU = 1218 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_TO = 1220 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_TI = 1217 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_TE = 1219 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_TA = 1216 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_SU = 1213 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_SO = 1215 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_SHI = 1212 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_SE = 1214 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_SA = 1211 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_RU = 1241 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_RO = 1243 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_RI = 1240 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_RE = 1242 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_RA = 1239 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_O = 1205 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_NU = 1223 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_NO = 1225 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_NI = 1222 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_NE = 1224 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_NA = 1221 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_N = 1245 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_MU = 1233 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_MO = 1235 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_MI = 1232 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_ME = 1234 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_MA = 1231 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_KU = 1208 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_KO = 1210 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_KI = 1207 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_KE = 1209 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_KA = 1206 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_I = 1202 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_HU = 1228 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_HO = 1230 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_HI = 1227 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_HE = 1229 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_HA = 1226 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_FU = 1228 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_E = 1204 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_CHI = 1217 :: Word32

-- | /No description available in the introspection data./
pattern KEY_kana_A = 1201 :: Word32

-- | /No description available in the introspection data./
pattern KEY_k = 107 :: Word32

-- | /No description available in the introspection data./
pattern KEY_jot = 3018 :: Word32

-- | /No description available in the introspection data./
pattern KEY_jcircumflex = 700 :: Word32

-- | /No description available in the introspection data./
pattern KEY_j = 106 :: Word32

-- | /No description available in the introspection data./
pattern KEY_itilde = 949 :: Word32

-- | /No description available in the introspection data./
pattern KEY_iogonek = 999 :: Word32

-- | /No description available in the introspection data./
pattern KEY_intersection = 2268 :: Word32

-- | /No description available in the introspection data./
pattern KEY_integral = 2239 :: Word32

-- | /No description available in the introspection data./
pattern KEY_infinity = 2242 :: Word32

-- | /No description available in the introspection data./
pattern KEY_includes = 2267 :: Word32

-- | /No description available in the introspection data./
pattern KEY_includedin = 2266 :: Word32

-- | /No description available in the introspection data./
pattern KEY_implies = 2254 :: Word32

-- | /No description available in the introspection data./
pattern KEY_imacron = 1007 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ihook = 16785097 :: Word32

-- | /No description available in the introspection data./
pattern KEY_igrave = 236 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ifonlyif = 2253 :: Word32

-- | /No description available in the introspection data./
pattern KEY_idotless = 697 :: Word32

-- | /No description available in the introspection data./
pattern KEY_idiaeresis = 239 :: Word32

-- | /No description available in the introspection data./
pattern KEY_identical = 2255 :: Word32

-- | /No description available in the introspection data./
pattern KEY_icircumflex = 238 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ibreve = 16777517 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ibelowdot = 16785099 :: Word32

-- | /No description available in the introspection data./
pattern KEY_iacute = 237 :: Word32

-- | /No description available in the introspection data./
pattern KEY_iTouch = 269025120 :: Word32

-- | /No description available in the introspection data./
pattern KEY_i = 105 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hyphen = 173 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ht = 2530 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hstroke = 689 :: Word32

-- | /No description available in the introspection data./
pattern KEY_horizlinescan9 = 2547 :: Word32

-- | /No description available in the introspection data./
pattern KEY_horizlinescan7 = 2546 :: Word32

-- | /No description available in the introspection data./
pattern KEY_horizlinescan5 = 2545 :: Word32

-- | /No description available in the introspection data./
pattern KEY_horizlinescan3 = 2544 :: Word32

-- | /No description available in the introspection data./
pattern KEY_horizlinescan1 = 2543 :: Word32

-- | /No description available in the introspection data./
pattern KEY_horizconnector = 2211 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hexagram = 2778 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_zayin = 3302 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_zain = 3302 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_zadi = 3318 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_zade = 3318 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_yod = 3305 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_waw = 3301 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_teth = 3304 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_tet = 3304 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_taw = 3322 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_taf = 3322 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_shin = 3321 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_samekh = 3313 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_samech = 3313 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_resh = 3320 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_qoph = 3319 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_pe = 3316 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_nun = 3312 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_mem = 3310 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_lamed = 3308 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_kuf = 3319 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_kaph = 3307 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_het = 3303 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_he = 3300 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_gimmel = 3298 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_gimel = 3298 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_finalzadi = 3317 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_finalzade = 3317 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_finalpe = 3315 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_finalnun = 3311 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_finalmem = 3309 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_finalkaph = 3306 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_doublelowline = 3295 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_daleth = 3299 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_dalet = 3299 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_chet = 3303 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_beth = 3297 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_bet = 3297 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_ayin = 3314 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hebrew_aleph = 3296 :: Word32

-- | /No description available in the introspection data./
pattern KEY_heart = 2798 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hcircumflex = 694 :: Word32

-- | /No description available in the introspection data./
pattern KEY_hairspace = 2728 :: Word32

-- | /No description available in the introspection data./
pattern KEY_h = 104 :: Word32

-- | /No description available in the introspection data./
pattern KEY_guillemotright = 187 :: Word32

-- | /No description available in the introspection data./
pattern KEY_guillemotleft = 171 :: Word32

-- | /No description available in the introspection data./
pattern KEY_guillemetright = 187 :: Word32

-- | /No description available in the introspection data./
pattern KEY_guillemetleft = 171 :: Word32

-- | /No description available in the introspection data./
pattern KEY_greaterthanequal = 2238 :: Word32

-- | /No description available in the introspection data./
pattern KEY_greater = 62 :: Word32

-- | /No description available in the introspection data./
pattern KEY_grave = 96 :: Word32

-- | /No description available in the introspection data./
pattern KEY_gcircumflex = 760 :: Word32

-- | /No description available in the introspection data./
pattern KEY_gcedilla = 955 :: Word32

-- | /No description available in the introspection data./
pattern KEY_gcaron = 16777703 :: Word32

-- | /No description available in the introspection data./
pattern KEY_gbreve = 699 :: Word32

-- | /No description available in the introspection data./
pattern KEY_gabovedot = 757 :: Word32

-- | /No description available in the introspection data./
pattern KEY_g = 103 :: Word32

-- | /No description available in the introspection data./
pattern KEY_function = 2294 :: Word32

-- | /No description available in the introspection data./
pattern KEY_fourthroot = 16785948 :: Word32

-- | /No description available in the introspection data./
pattern KEY_foursuperior = 16785524 :: Word32

-- | /No description available in the introspection data./
pattern KEY_foursubscript = 16785540 :: Word32

-- | /No description available in the introspection data./
pattern KEY_fourfifths = 2741 :: Word32

-- | /No description available in the introspection data./
pattern KEY_fivesuperior = 16785525 :: Word32

-- | /No description available in the introspection data./
pattern KEY_fivesubscript = 16785541 :: Word32

-- | /No description available in the introspection data./
pattern KEY_fivesixths = 2743 :: Word32

-- | /No description available in the introspection data./
pattern KEY_fiveeighths = 2757 :: Word32

-- | /No description available in the introspection data./
pattern KEY_filledtribulletup = 2792 :: Word32

-- | /No description available in the introspection data./
pattern KEY_filledtribulletdown = 2793 :: Word32

-- | /No description available in the introspection data./
pattern KEY_filledrighttribullet = 2781 :: Word32

-- | /No description available in the introspection data./
pattern KEY_filledrectbullet = 2779 :: Word32

-- | /No description available in the introspection data./
pattern KEY_filledlefttribullet = 2780 :: Word32

-- | /No description available in the introspection data./
pattern KEY_figdash = 2747 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ff = 2531 :: Word32

-- | /No description available in the introspection data./
pattern KEY_femalesymbol = 2808 :: Word32

-- | /No description available in the introspection data./
pattern KEY_fabovedot = 16784927 :: Word32

-- | /No description available in the introspection data./
pattern KEY_f = 102 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ezh = 16777874 :: Word32

-- | /No description available in the introspection data./
pattern KEY_exclamdown = 161 :: Word32

-- | /No description available in the introspection data./
pattern KEY_exclam = 33 :: Word32

-- | /No description available in the introspection data./
pattern KEY_etilde = 16785085 :: Word32

-- | /No description available in the introspection data./
pattern KEY_eth = 240 :: Word32

-- | /No description available in the introspection data./
pattern KEY_equal = 61 :: Word32

-- | /No description available in the introspection data./
pattern KEY_eogonek = 490 :: Word32

-- | /No description available in the introspection data./
pattern KEY_enspace = 2722 :: Word32

-- | /No description available in the introspection data./
pattern KEY_enopensquarebullet = 2785 :: Word32

-- | /No description available in the introspection data./
pattern KEY_enopencircbullet = 2784 :: Word32

-- | /No description available in the introspection data./
pattern KEY_eng = 959 :: Word32

-- | /No description available in the introspection data./
pattern KEY_enfilledsqbullet = 2791 :: Word32

-- | /No description available in the introspection data./
pattern KEY_enfilledcircbullet = 2790 :: Word32

-- | /No description available in the introspection data./
pattern KEY_endash = 2730 :: Word32

-- | /No description available in the introspection data./
pattern KEY_emspace = 2721 :: Word32

-- | /No description available in the introspection data./
pattern KEY_emptyset = 16785925 :: Word32

-- | /No description available in the introspection data./
pattern KEY_emopenrectangle = 2767 :: Word32

-- | /No description available in the introspection data./
pattern KEY_emopencircle = 2766 :: Word32

-- | /No description available in the introspection data./
pattern KEY_emfilledrect = 2783 :: Word32

-- | /No description available in the introspection data./
pattern KEY_emfilledcircle = 2782 :: Word32

-- | /No description available in the introspection data./
pattern KEY_emdash = 2729 :: Word32

-- | /No description available in the introspection data./
pattern KEY_emacron = 954 :: Word32

-- | /No description available in the introspection data./
pattern KEY_em4space = 2724 :: Word32

-- | /No description available in the introspection data./
pattern KEY_em3space = 2723 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ellipsis = 2734 :: Word32

-- | /No description available in the introspection data./
pattern KEY_elementof = 16785928 :: Word32

-- | /No description available in the introspection data./
pattern KEY_eightsuperior = 16785528 :: Word32

-- | /No description available in the introspection data./
pattern KEY_eightsubscript = 16785544 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ehook = 16785083 :: Word32

-- | /No description available in the introspection data./
pattern KEY_egrave = 232 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ediaeresis = 235 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ecircumflextilde = 16785093 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ecircumflexhook = 16785091 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ecircumflexgrave = 16785089 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ecircumflexbelowdot = 16785095 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ecircumflexacute = 16785087 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ecircumflex = 234 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ecaron = 492 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ebelowdot = 16785081 :: Word32

-- | /No description available in the introspection data./
pattern KEY_eacute = 233 :: Word32

-- | /No description available in the introspection data./
pattern KEY_eabovedot = 1004 :: Word32

-- | /No description available in the introspection data./
pattern KEY_e = 101 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dstroke = 496 :: Word32

-- | /No description available in the introspection data./
pattern KEY_downtack = 3010 :: Word32

-- | /No description available in the introspection data./
pattern KEY_downstile = 3012 :: Word32

-- | /No description available in the introspection data./
pattern KEY_downshoe = 3030 :: Word32

-- | /No description available in the introspection data./
pattern KEY_downcaret = 2984 :: Word32

-- | /No description available in the introspection data./
pattern KEY_downarrow = 2302 :: Word32

-- | /No description available in the introspection data./
pattern KEY_doublelowquotemark = 2814 :: Word32

-- | /No description available in the introspection data./
pattern KEY_doubledagger = 2802 :: Word32

-- | /No description available in the introspection data./
pattern KEY_doubleacute = 445 :: Word32

-- | /No description available in the introspection data./
pattern KEY_doubbaselinedot = 2735 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dollar = 36 :: Word32

-- | /No description available in the introspection data./
pattern KEY_division = 247 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dintegral = 16785964 :: Word32

-- | /No description available in the introspection data./
pattern KEY_digitspace = 2725 :: Word32

-- | /No description available in the introspection data./
pattern KEY_diamond = 2797 :: Word32

-- | /No description available in the introspection data./
pattern KEY_diaeresis = 168 :: Word32

-- | /No description available in the introspection data./
pattern KEY_degree = 176 :: Word32

-- | /No description available in the introspection data./
pattern KEY_decimalpoint = 2749 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_voiced_sound = 65118 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_u = 65160 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_tilde = 65107 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_stroke = 65123 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_small_schwa = 65162 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_semivoiced_sound = 65119 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_schwa = 65162 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_psili = 65124 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_perispomeni = 65107 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_ogonek = 65116 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_o = 65158 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_macron = 65108 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_lowline = 65168 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_longsolidusoverlay = 65171 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_iota = 65117 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_invertedbreve = 65133 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_i = 65156 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_horn = 65122 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_hook = 65121 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_hamza = 65165 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_greek = 65164 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_grave = 65104 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_e = 65154 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_doublegrave = 65126 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_doubleacute = 65113 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_diaeresis = 65111 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_dasia = 65125 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_currency = 65135 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_circumflex = 65106 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_cedilla = 65115 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_caron = 65114 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_capital_schwa = 65163 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_breve = 65109 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_belowverticalline = 65170 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_belowtilde = 65130 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_belowring = 65127 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_belowmacron = 65128 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_belowdot = 65120 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_belowdiaeresis = 65132 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_belowcomma = 65134 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_belowcircumflex = 65129 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_belowbreve = 65131 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_acute = 65105 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_aboveverticalline = 65169 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_abovering = 65112 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_abovereversedcomma = 65125 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_abovedot = 65110 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_abovecomma = 65124 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_a = 65152 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_U = 65161 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_SCHWA = 65163 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_O = 65159 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_I = 65157 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_E = 65155 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dead_A = 65153 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dcaron = 495 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dagger = 2801 :: Word32

-- | /No description available in the introspection data./
pattern KEY_dabovedot = 16784907 :: Word32

-- | /No description available in the introspection data./
pattern KEY_d = 100 :: Word32

-- | /No description available in the introspection data./
pattern KEY_cursor = 2815 :: Word32

-- | /No description available in the introspection data./
pattern KEY_currency = 164 :: Word32

-- | /No description available in the introspection data./
pattern KEY_cuberoot = 16785947 :: Word32

-- | /No description available in the introspection data./
pattern KEY_crossinglines = 2542 :: Word32

-- | /No description available in the introspection data./
pattern KEY_cr = 2532 :: Word32

-- | /No description available in the introspection data./
pattern KEY_copyright = 169 :: Word32

-- | /No description available in the introspection data./
pattern KEY_containsas = 16785931 :: Word32

-- | /No description available in the introspection data./
pattern KEY_comma = 44 :: Word32

-- | /No description available in the introspection data./
pattern KEY_combining_tilde = 16777987 :: Word32

-- | /No description available in the introspection data./
pattern KEY_combining_hook = 16777993 :: Word32

-- | /No description available in the introspection data./
pattern KEY_combining_grave = 16777984 :: Word32

-- | /No description available in the introspection data./
pattern KEY_combining_belowdot = 16778019 :: Word32

-- | /No description available in the introspection data./
pattern KEY_combining_acute = 16777985 :: Word32

-- | /No description available in the introspection data./
pattern KEY_colon = 58 :: Word32

-- | /No description available in the introspection data./
pattern KEY_club = 2796 :: Word32

-- | /No description available in the introspection data./
pattern KEY_circle = 3023 :: Word32

-- | /No description available in the introspection data./
pattern KEY_checkmark = 2803 :: Word32

-- | /No description available in the introspection data./
pattern KEY_checkerboard = 2529 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ch = 65184 :: Word32

-- | /No description available in the introspection data./
pattern KEY_cent = 162 :: Word32

-- | /No description available in the introspection data./
pattern KEY_cedilla = 184 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ccircumflex = 742 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ccedilla = 231 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ccaron = 488 :: Word32

-- | /No description available in the introspection data./
pattern KEY_caron = 439 :: Word32

-- | /No description available in the introspection data./
pattern KEY_caret = 2812 :: Word32

-- | /No description available in the introspection data./
pattern KEY_careof = 2744 :: Word32

-- | /No description available in the introspection data./
pattern KEY_cacute = 486 :: Word32

-- | /No description available in the introspection data./
pattern KEY_cabovedot = 741 :: Word32

-- | /No description available in the introspection data./
pattern KEY_c_h = 65187 :: Word32

-- | /No description available in the introspection data./
pattern KEY_c = 99 :: Word32

-- | /No description available in the introspection data./
pattern KEY_brokenbar = 166 :: Word32

-- | /No description available in the introspection data./
pattern KEY_breve = 418 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_8 = 16787584 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_78 = 16787648 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_7 = 16787520 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_68 = 16787616 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_678 = 16787680 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_67 = 16787552 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_6 = 16787488 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_58 = 16787600 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_578 = 16787664 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_57 = 16787536 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_568 = 16787632 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_5678 = 16787696 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_567 = 16787568 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_56 = 16787504 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_5 = 16787472 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_48 = 16787592 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_478 = 16787656 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_47 = 16787528 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_468 = 16787624 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_4678 = 16787688 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_467 = 16787560 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_46 = 16787496 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_458 = 16787608 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_4578 = 16787672 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_457 = 16787544 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_4568 = 16787640 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_45678 = 16787704 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_4567 = 16787576 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_456 = 16787512 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_45 = 16787480 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_4 = 16787464 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_38 = 16787588 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_378 = 16787652 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_37 = 16787524 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_368 = 16787620 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_3678 = 16787684 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_367 = 16787556 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_36 = 16787492 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_358 = 16787604 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_3578 = 16787668 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_357 = 16787540 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_3568 = 16787636 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_35678 = 16787700 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_3567 = 16787572 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_356 = 16787508 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_35 = 16787476 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_348 = 16787596 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_3478 = 16787660 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_347 = 16787532 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_3468 = 16787628 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_34678 = 16787692 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_3467 = 16787564 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_346 = 16787500 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_3458 = 16787612 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_34578 = 16787676 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_3457 = 16787548 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_34568 = 16787644 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_345678 = 16787708 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_34567 = 16787580 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_3456 = 16787516 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_345 = 16787484 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_34 = 16787468 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_3 = 16787460 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_28 = 16787586 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_278 = 16787650 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_27 = 16787522 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_268 = 16787618 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2678 = 16787682 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_267 = 16787554 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_26 = 16787490 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_258 = 16787602 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2578 = 16787666 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_257 = 16787538 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2568 = 16787634 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_25678 = 16787698 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2567 = 16787570 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_256 = 16787506 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_25 = 16787474 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_248 = 16787594 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2478 = 16787658 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_247 = 16787530 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2468 = 16787626 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_24678 = 16787690 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2467 = 16787562 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_246 = 16787498 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2458 = 16787610 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_24578 = 16787674 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2457 = 16787546 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_24568 = 16787642 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_245678 = 16787706 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_24567 = 16787578 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2456 = 16787514 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_245 = 16787482 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_24 = 16787466 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_238 = 16787590 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2378 = 16787654 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_237 = 16787526 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2368 = 16787622 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_23678 = 16787686 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2367 = 16787558 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_236 = 16787494 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2358 = 16787606 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_23578 = 16787670 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2357 = 16787542 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_23568 = 16787638 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_235678 = 16787702 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_23567 = 16787574 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2356 = 16787510 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_235 = 16787478 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2348 = 16787598 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_23478 = 16787662 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2347 = 16787534 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_23468 = 16787630 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_234678 = 16787694 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_23467 = 16787566 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2346 = 16787502 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_23458 = 16787614 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_234578 = 16787678 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_23457 = 16787550 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_234568 = 16787646 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2345678 = 16787710 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_234567 = 16787582 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_23456 = 16787518 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2345 = 16787486 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_234 = 16787470 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_23 = 16787462 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_2 = 16787458 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_18 = 16787585 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_178 = 16787649 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_17 = 16787521 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_168 = 16787617 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1678 = 16787681 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_167 = 16787553 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_16 = 16787489 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_158 = 16787601 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1578 = 16787665 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_157 = 16787537 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1568 = 16787633 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_15678 = 16787697 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1567 = 16787569 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_156 = 16787505 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_15 = 16787473 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_148 = 16787593 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1478 = 16787657 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_147 = 16787529 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1468 = 16787625 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_14678 = 16787689 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1467 = 16787561 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_146 = 16787497 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1458 = 16787609 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_14578 = 16787673 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1457 = 16787545 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_14568 = 16787641 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_145678 = 16787705 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_14567 = 16787577 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1456 = 16787513 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_145 = 16787481 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_14 = 16787465 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_138 = 16787589 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1378 = 16787653 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_137 = 16787525 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1368 = 16787621 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_13678 = 16787685 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1367 = 16787557 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_136 = 16787493 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1358 = 16787605 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_13578 = 16787669 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1357 = 16787541 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_13568 = 16787637 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_135678 = 16787701 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_13567 = 16787573 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1356 = 16787509 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_135 = 16787477 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1348 = 16787597 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_13478 = 16787661 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1347 = 16787533 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_13468 = 16787629 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_134678 = 16787693 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_13467 = 16787565 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1346 = 16787501 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_13458 = 16787613 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_134578 = 16787677 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_13457 = 16787549 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_134568 = 16787645 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1345678 = 16787709 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_134567 = 16787581 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_13456 = 16787517 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1345 = 16787485 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_134 = 16787469 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_13 = 16787461 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_128 = 16787587 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1278 = 16787651 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_127 = 16787523 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1268 = 16787619 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12678 = 16787683 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1267 = 16787555 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_126 = 16787491 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1258 = 16787603 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12578 = 16787667 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1257 = 16787539 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12568 = 16787635 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_125678 = 16787699 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12567 = 16787571 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1256 = 16787507 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_125 = 16787475 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1248 = 16787595 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12478 = 16787659 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1247 = 16787531 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12468 = 16787627 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_124678 = 16787691 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12467 = 16787563 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1246 = 16787499 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12458 = 16787611 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_124578 = 16787675 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12457 = 16787547 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_124568 = 16787643 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1245678 = 16787707 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_124567 = 16787579 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12456 = 16787515 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1245 = 16787483 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_124 = 16787467 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1238 = 16787591 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12378 = 16787655 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1237 = 16787527 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12368 = 16787623 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_123678 = 16787687 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12367 = 16787559 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1236 = 16787495 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12358 = 16787607 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_123578 = 16787671 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12357 = 16787543 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_123568 = 16787639 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1235678 = 16787703 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_123567 = 16787575 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12356 = 16787511 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1235 = 16787479 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12348 = 16787599 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_123478 = 16787663 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12347 = 16787535 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_123468 = 16787631 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1234678 = 16787695 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_123467 = 16787567 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12346 = 16787503 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_123458 = 16787615 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1234578 = 16787679 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_123457 = 16787551 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1234568 = 16787647 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12345678 = 16787711 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1234567 = 16787583 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_123456 = 16787519 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12345 = 16787487 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1234 = 16787471 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_123 = 16787463 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_12 = 16787459 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dots_1 = 16787457 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dot_9 = 65529 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dot_8 = 65528 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dot_7 = 65527 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dot_6 = 65526 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dot_5 = 65525 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dot_4 = 65524 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dot_3 = 65523 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dot_2 = 65522 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dot_10 = 65530 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_dot_1 = 65521 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braille_blank = 16787456 :: Word32

-- | /No description available in the introspection data./
pattern KEY_bracketright = 93 :: Word32

-- | /No description available in the introspection data./
pattern KEY_bracketleft = 91 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braceright = 125 :: Word32

-- | /No description available in the introspection data./
pattern KEY_braceleft = 123 :: Word32

-- | /No description available in the introspection data./
pattern KEY_botvertsummationconnector = 2228 :: Word32

-- | /No description available in the introspection data./
pattern KEY_bott = 2550 :: Word32

-- | /No description available in the introspection data./
pattern KEY_botrightsummation = 2230 :: Word32

-- | /No description available in the introspection data./
pattern KEY_botrightsqbracket = 2218 :: Word32

-- | /No description available in the introspection data./
pattern KEY_botrightparens = 2222 :: Word32

-- | /No description available in the introspection data./
pattern KEY_botleftsummation = 2226 :: Word32

-- | /No description available in the introspection data./
pattern KEY_botleftsqbracket = 2216 :: Word32

-- | /No description available in the introspection data./
pattern KEY_botleftparens = 2220 :: Word32

-- | /No description available in the introspection data./
pattern KEY_botintegral = 2213 :: Word32

-- | /No description available in the introspection data./
pattern KEY_blank = 2527 :: Word32

-- | /No description available in the introspection data./
pattern KEY_because = 16785973 :: Word32

-- | /No description available in the introspection data./
pattern KEY_bar = 124 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ballotcross = 2804 :: Word32

-- | /No description available in the introspection data./
pattern KEY_backslash = 92 :: Word32

-- | /No description available in the introspection data./
pattern KEY_babovedot = 16784899 :: Word32

-- | /No description available in the introspection data./
pattern KEY_b = 98 :: Word32

-- | /No description available in the introspection data./
pattern KEY_atilde = 227 :: Word32

-- | /No description available in the introspection data./
pattern KEY_at = 64 :: Word32

-- | /No description available in the introspection data./
pattern KEY_asterisk = 42 :: Word32

-- | /No description available in the introspection data./
pattern KEY_asciitilde = 126 :: Word32

-- | /No description available in the introspection data./
pattern KEY_asciicircum = 94 :: Word32

-- | /No description available in the introspection data./
pattern KEY_aring = 229 :: Word32

-- | /No description available in the introspection data./
pattern KEY_approximate = 2248 :: Word32

-- | /No description available in the introspection data./
pattern KEY_approxeq = 16785992 :: Word32

-- | /No description available in the introspection data./
pattern KEY_apostrophe = 39 :: Word32

-- | /No description available in the introspection data./
pattern KEY_aogonek = 433 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ampersand = 38 :: Word32

-- | /No description available in the introspection data./
pattern KEY_amacron = 992 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ahook = 16785059 :: Word32

-- | /No description available in the introspection data./
pattern KEY_agrave = 224 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ae = 230 :: Word32

-- | /No description available in the introspection data./
pattern KEY_adiaeresis = 228 :: Word32

-- | /No description available in the introspection data./
pattern KEY_acute = 180 :: Word32

-- | /No description available in the introspection data./
pattern KEY_acircumflextilde = 16785067 :: Word32

-- | /No description available in the introspection data./
pattern KEY_acircumflexhook = 16785065 :: Word32

-- | /No description available in the introspection data./
pattern KEY_acircumflexgrave = 16785063 :: Word32

-- | /No description available in the introspection data./
pattern KEY_acircumflexbelowdot = 16785069 :: Word32

-- | /No description available in the introspection data./
pattern KEY_acircumflexacute = 16785061 :: Word32

-- | /No description available in the introspection data./
pattern KEY_acircumflex = 226 :: Word32

-- | /No description available in the introspection data./
pattern KEY_abrevetilde = 16785077 :: Word32

-- | /No description available in the introspection data./
pattern KEY_abrevehook = 16785075 :: Word32

-- | /No description available in the introspection data./
pattern KEY_abrevegrave = 16785073 :: Word32

-- | /No description available in the introspection data./
pattern KEY_abrevebelowdot = 16785079 :: Word32

-- | /No description available in the introspection data./
pattern KEY_abreveacute = 16785071 :: Word32

-- | /No description available in the introspection data./
pattern KEY_abreve = 483 :: Word32

-- | /No description available in the introspection data./
pattern KEY_abovedot = 511 :: Word32

-- | /No description available in the introspection data./
pattern KEY_abelowdot = 16785057 :: Word32

-- | /No description available in the introspection data./
pattern KEY_aacute = 225 :: Word32

-- | /No description available in the introspection data./
pattern KEY_a = 97 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Zstroke = 16777653 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ZoomReset = 268964260 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ZoomOut = 269025164 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ZoomIn = 269025163 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Zenkaku_Hankaku = 65322 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Zenkaku = 65320 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Zen_Koho = 65341 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Zcaron = 430 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Zacute = 428 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Zabovedot = 431 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Z = 90 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ytilde = 16785144 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Yhook = 16785142 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ygrave = 16785138 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Yellow = 269025189 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ydiaeresis = 5054 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ycircumflex = 16777590 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ybelowdot = 16785140 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Yacute = 221 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Y = 89 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Xfer = 269025162 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Xabovedot = 16785034 :: Word32

-- | /No description available in the introspection data./
pattern KEY_X = 88 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Word = 269025161 :: Word32

-- | /No description available in the introspection data./
pattern KEY_WonSign = 16785577 :: Word32

-- | /No description available in the introspection data./
pattern KEY_WindowClear = 269025109 :: Word32

-- | /No description available in the introspection data./
pattern KEY_WheelButton = 269025160 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Wgrave = 16785024 :: Word32

-- | /No description available in the introspection data./
pattern KEY_WebCam = 269025167 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Wdiaeresis = 16785028 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Wcircumflex = 16777588 :: Word32

-- | /No description available in the introspection data./
pattern KEY_WakeUp = 269025067 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Wacute = 16785026 :: Word32

-- | /No description available in the introspection data./
pattern KEY_WWW = 269025070 :: Word32

-- | /No description available in the introspection data./
pattern KEY_WWAN = 269025204 :: Word32

-- | /No description available in the introspection data./
pattern KEY_WPSButton = 268964369 :: Word32

-- | /No description available in the introspection data./
pattern KEY_WLAN = 269025173 :: Word32

-- | /No description available in the introspection data./
pattern KEY_W = 87 :: Word32

-- | /No description available in the introspection data./
pattern KEY_VoidSymbol = 16777215 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Voicemail = 268964268 :: Word32

-- | /No description available in the introspection data./
pattern KEY_VoiceCommand = 268964422 :: Word32

-- | /No description available in the introspection data./
pattern KEY_View = 269025185 :: Word32

-- | /No description available in the introspection data./
pattern KEY_VideoPhone = 268964256 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Video = 269025159 :: Word32

-- | /No description available in the introspection data./
pattern KEY_VendorHome = 269025076 :: Word32

-- | /No description available in the introspection data./
pattern KEY_VOD = 268964467 :: Word32

-- | /No description available in the introspection data./
pattern KEY_V = 86 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Utilde = 989 :: Word32

-- | /No description available in the introspection data./
pattern KEY_UserPB = 269025156 :: Word32

-- | /No description available in the introspection data./
pattern KEY_User2KB = 269025158 :: Word32

-- | /No description available in the introspection data./
pattern KEY_User1KB = 269025157 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Uring = 473 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Up = 65362 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Uogonek = 985 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Unmute = 268964468 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ungrab = 269024800 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Undo = 65381 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Umacron = 990 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukranian_yi = 1703 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukranian_je = 1700 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukranian_i = 1702 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukranian_YI = 1719 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukranian_JE = 1716 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukranian_I = 1718 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukrainian_yi = 1703 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukrainian_ie = 1700 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukrainian_i = 1702 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukrainian_ghe_with_upturn = 1709 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukrainian_YI = 1719 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukrainian_IE = 1716 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukrainian_I = 1718 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ukrainian_GHE_WITH_UPTURN = 1725 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Uhorntilde = 16785134 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Uhornhook = 16785132 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Uhorngrave = 16785130 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Uhornbelowdot = 16785136 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Uhornacute = 16785128 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Uhorn = 16777647 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Uhook = 16785126 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ugrave = 217 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Udoubleacute = 475 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Udiaeresis = 220 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ucircumflex = 219 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ubreve = 733 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ubelowdot = 16785124 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Uacute = 218 :: Word32

-- | /No description available in the introspection data./
pattern KEY_UWB = 269025174 :: Word32

-- | /No description available in the introspection data./
pattern KEY_U = 85 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Tslash = 940 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Travel = 269025154 :: Word32

-- | /No description available in the introspection data./
pattern KEY_TraditionalSonar = 268964485 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Touroku = 65323 :: Word32

-- | /No description available in the introspection data./
pattern KEY_TouchpadToggle = 269025193 :: Word32

-- | /No description available in the introspection data./
pattern KEY_TouchpadOn = 269025200 :: Word32

-- | /No description available in the introspection data./
pattern KEY_TouchpadOff = 269025201 :: Word32

-- | /No description available in the introspection data./
pattern KEY_TopMenu = 269025186 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Tools = 269025153 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ToDoList = 269025055 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Time = 269025183 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thorn = 222 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_yoying = 3501 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_yoyak = 3522 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_wowaen = 3527 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_totao = 3509 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_topatak = 3503 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_thothung = 3510 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_thothong = 3512 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_thothan = 3504 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_thothahan = 3511 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_thophuthao = 3506 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_thonangmontho = 3505 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_thanthakhat = 3564 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_sosua = 3530 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_soso = 3499 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_sosala = 3528 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_sorusi = 3529 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_sarauu = 3545 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_sarauee = 3543 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_saraue = 3542 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_sarau = 3544 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_sarao = 3554 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_saraii = 3541 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_sarai = 3540 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_sarae = 3552 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_saraam = 3539 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_saraaimaimuan = 3555 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_saraaimaimalai = 3556 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_saraae = 3553 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_saraaa = 3538 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_saraa = 3536 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_ru = 3524 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_rorua = 3523 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_popla = 3515 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_phosamphao = 3520 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_phophung = 3516 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_phophan = 3518 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_phinthu = 3546 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_paiyannoi = 3535 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_oang = 3533 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_nonu = 3513 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_nonen = 3507 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_nikhahit = 3565 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_ngongu = 3495 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_moma = 3521 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_maiyamok = 3558 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_maitri = 3562 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_maitho = 3561 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_maitaikhu = 3559 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_maihanakat_maitho = 3550 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_maihanakat = 3537 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_maiek = 3560 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_maichattawa = 3563 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_lu = 3526 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_loling = 3525 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_lochula = 3532 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_leksun = 3568 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_leksong = 3570 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_leksi = 3572 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_leksam = 3571 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_lekpaet = 3576 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_leknung = 3569 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_lekkao = 3577 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_lekhok = 3574 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_lekha = 3573 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_lekchet = 3575 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_lakkhangyao = 3557 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_kokai = 3489 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_khorakhang = 3494 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_khokhwai = 3492 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_khokhuat = 3491 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_khokhon = 3493 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_khokhai = 3490 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_honokhuk = 3534 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_hohip = 3531 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_fofan = 3519 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_fofa = 3517 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_dodek = 3508 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_dochada = 3502 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_chochoe = 3500 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_choching = 3497 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_chochang = 3498 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_chochan = 3496 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_bobaimai = 3514 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Thai_baht = 3551 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Terminate_Server = 65237 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Terminal = 269025152 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Tcedilla = 478 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Tcaron = 427 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Taskmanager = 268964417 :: Word32

-- | /No description available in the introspection data./
pattern KEY_TaskPane = 269025151 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Tabovedot = 16785002 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Tab = 65289 :: Word32

-- | /No description available in the introspection data./
pattern KEY_THORN = 222 :: Word32

-- | /No description available in the introspection data./
pattern KEY_T = 84 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sys_Req = 65301 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_9 = 269024777 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_8 = 269024776 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_7 = 269024775 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_6 = 269024774 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_5 = 269024773 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_4 = 269024772 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_3 = 269024771 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_2 = 269024770 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_12 = 269024780 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_11 = 269024779 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_10 = 269024778 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Switch_VT_1 = 269024769 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Suspend = 269025191 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Support = 269025150 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Super_R = 65516 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Super_L = 65515 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Subtitle = 269025178 :: Word32

-- | /No description available in the introspection data./
pattern KEY_StopRecord = 268964465 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Stop = 269025064 :: Word32

-- | /No description available in the introspection data./
pattern KEY_StickyKeys_Enable = 65141 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Start = 269025050 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Standby = 269025040 :: Word32

-- | /No description available in the introspection data./
pattern KEY_SplitScreen = 269025149 :: Word32

-- | /No description available in the introspection data./
pattern KEY_SpellCheck = 268964272 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Spell = 269025148 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sos = 268964479 :: Word32

-- | /No description available in the introspection data./
pattern KEY_SlowReverse = 268964470 :: Word32

-- | /No description available in the introspection data./
pattern KEY_SlowKeys_Enable = 65139 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sleep = 269025071 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ya = 16780730 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_va = 16780736 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_uu2 = 16780758 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_uu = 16780684 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_u2 = 16780756 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_u = 16780683 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ttha = 16780712 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_tta = 16780711 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_thha = 16780718 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_tha = 16780717 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ssha = 16780738 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_sha = 16780737 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_sa = 16780739 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ruu2 = 16780786 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ru2 = 16780760 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_rii = 16780686 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ri = 16780685 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ra = 16780731 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_pha = 16780725 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_pa = 16780724 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_oo2 = 16780765 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_oo = 16780693 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_o2 = 16780764 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_o = 16780692 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_nya = 16780708 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_nna = 16780715 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_nja = 16780710 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_nga = 16780703 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ng2 = 16780702 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ng = 16780674 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ndha = 16780723 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ndda = 16780716 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_na = 16780721 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_mba = 16780729 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ma = 16780728 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_luu2 = 16780787 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_luu = 16780688 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_lu2 = 16780767 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_lu = 16780687 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_lla = 16780741 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_la = 16780733 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_kunddaliya = 16780788 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_kha = 16780699 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ka = 16780698 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_jnya = 16780709 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_jha = 16780707 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ja = 16780706 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ii2 = 16780755 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ii = 16780682 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_i2 = 16780754 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_i = 16780681 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ha = 16780740 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_h2 = 16780675 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_gha = 16780701 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ga = 16780700 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_fa = 16780742 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ee2 = 16780762 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ee = 16780690 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_e2 = 16780761 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_e = 16780689 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_dhha = 16780720 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_dha = 16780719 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ddha = 16780714 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_dda = 16780713 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_cha = 16780705 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ca = 16780704 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_bha = 16780727 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ba = 16780726 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_au2 = 16780766 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_au = 16780694 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_al = 16780746 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ai2 = 16780763 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ai = 16780691 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_aee2 = 16780753 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_aee = 16780680 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ae2 = 16780752 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_ae = 16780679 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_aa2 = 16780751 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_aa = 16780678 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sinh_a = 16780677 :: Word32

-- | /No description available in the introspection data./
pattern KEY_SingleRangeRadar = 268964482 :: Word32

-- | /No description available in the introspection data./
pattern KEY_SingleCandidate = 65340 :: Word32

-- | /No description available in the introspection data./
pattern KEY_SidevuSonar = 268964487 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Shop = 269025078 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Shift_R = 65506 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Shift_Lock = 65510 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Shift_L = 65505 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_tshe = 1707 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_nje = 1706 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_lje = 1705 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_je = 1704 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_dze = 1711 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_dje = 1697 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_TSHE = 1723 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_NJE = 1722 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_LJE = 1721 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_JE = 1720 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_DZE = 1727 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Serbian_DJE = 1713 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Send = 269025147 :: Word32

-- | /No description available in the introspection data./
pattern KEY_SelectiveScreenshot = 268964474 :: Word32

-- | /No description available in the introspection data./
pattern KEY_SelectButton = 269025184 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Select = 65376 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Search = 269025051 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Scroll_Lock = 65300 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ScrollUp = 269025144 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ScrollDown = 269025145 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ScrollClick = 269025146 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Screensaver = 268964421 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ScreenSaver = 269025069 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Scircumflex = 734 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Scedilla = 426 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Scaron = 425 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Save = 269025143 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sacute = 422 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Sabovedot = 16784992 :: Word32

-- | /No description available in the introspection data./
pattern KEY_SCHWA = 16777615 :: Word32

-- | /No description available in the introspection data./
pattern KEY_S = 83 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RupeeSign = 16785576 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RotationPB = 269025141 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RotationLockToggle = 269025207 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RotationKB = 269025142 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RotateWindows = 269025140 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RootMenu = 268964458 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Romaji = 65316 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RockerUp = 269025059 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RockerEnter = 269025061 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RockerDown = 269025060 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RightUp = 268964454 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RightDown = 268964455 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Right = 65363 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Return = 65293 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Reply = 269025138 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RepeatKeys_Enable = 65138 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Reload = 269025139 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RefreshRateToggle = 268964402 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Refresh = 269025065 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Redo = 65382 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Red = 269025187 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Rcedilla = 931 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Rcaron = 472 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RadarOverlay = 268964484 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Racute = 448 :: Word32

-- | /No description available in the introspection data./
pattern KEY_RFKill = 269025205 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R9 = 65498 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R8 = 65497 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R7 = 65496 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R6 = 65495 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R5 = 65494 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R4 = 65493 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R3 = 65492 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R2 = 65491 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R15 = 65504 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R14 = 65503 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R13 = 65502 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R12 = 65501 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R11 = 65500 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R10 = 65499 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R1 = 65490 :: Word32

-- | /No description available in the introspection data./
pattern KEY_R = 82 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Q = 81 :: Word32

-- | /No description available in the introspection data./
pattern KEY_PrivacyScreenToggle = 268964473 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Prior = 65365 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Print = 65377 :: Word32

-- | /No description available in the introspection data./
pattern KEY_PreviousElement = 268964476 :: Word32

-- | /No description available in the introspection data./
pattern KEY_PreviousCandidate = 65342 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Prev_Virtual_Screen = 65233 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Prev_VMode = 269024803 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Presentation = 268964265 :: Word32

-- | /No description available in the introspection data./
pattern KEY_PowerOff = 269025066 :: Word32

-- | /No description available in the introspection data./
pattern KEY_PowerDown = 269025057 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_UpRight = 65253 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_UpLeft = 65252 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Up = 65250 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Right = 65249 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Left = 65248 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_EnableKeys = 65273 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Drag_Dflt = 65268 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Drag5 = 65277 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Drag4 = 65272 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Drag3 = 65271 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Drag2 = 65270 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Drag1 = 65269 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_DownRight = 65255 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_DownLeft = 65254 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Down = 65251 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_DfltBtnPrev = 65276 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_DfltBtnNext = 65275 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_DblClick_Dflt = 65262 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_DblClick5 = 65267 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_DblClick4 = 65266 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_DblClick3 = 65265 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_DblClick2 = 65264 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_DblClick1 = 65263 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Button_Dflt = 65256 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Button5 = 65261 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Button4 = 65260 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Button3 = 65259 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Button2 = 65258 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Button1 = 65257 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pointer_Accelerate = 65274 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pictures = 269025169 :: Word32

-- | /No description available in the introspection data./
pattern KEY_PickupPhone = 268964285 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Phone = 269025134 :: Word32

-- | /No description available in the introspection data./
pattern KEY_PesetaSign = 16785575 :: Word32

-- | /No description available in the introspection data./
pattern KEY_PauseRecord = 268964466 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pause = 65299 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Paste = 269025133 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Page_Up = 65365 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Page_Down = 65366 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Pabovedot = 16784982 :: Word32

-- | /No description available in the introspection data./
pattern KEY_P = 80 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Overlay2_Enable = 65145 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Overlay1_Enable = 65144 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Otilde = 213 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Oslash = 216 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Option = 269025132 :: Word32

-- | /No description available in the introspection data./
pattern KEY_OpenURL = 269025080 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Open = 269025131 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ooblique = 216 :: Word32

-- | /No description available in the introspection data./
pattern KEY_OnScreenKeyboard = 268964472 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Omacron = 978 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ohorntilde = 16785120 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ohornhook = 16785118 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ohorngrave = 16785116 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ohornbelowdot = 16785122 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ohornacute = 16785114 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ohorn = 16777632 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ohook = 16785102 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ograve = 210 :: Word32

-- | /No description available in the introspection data./
pattern KEY_OfficeHome = 269025130 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Odoubleacute = 469 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Odiaeresis = 214 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ocircumflextilde = 16785110 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ocircumflexhook = 16785108 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ocircumflexgrave = 16785106 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ocircumflexbelowdot = 16785112 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ocircumflexacute = 16785104 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ocircumflex = 212 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ocaron = 16777681 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Obelowdot = 16785100 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Obarred = 16777631 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Oacute = 211 :: Word32

-- | /No description available in the introspection data./
pattern KEY_OE = 5052 :: Word32

-- | /No description available in the introspection data./
pattern KEY_O = 79 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NumericStar = 268964362 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NumericPound = 268964363 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NumericD = 268964367 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NumericC = 268964366 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NumericB = 268964365 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NumericA = 268964364 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric9 = 268964361 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric8 = 268964360 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric7 = 268964359 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric6 = 268964358 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric5 = 268964357 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric4 = 268964356 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric3 = 268964355 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric2 = 268964354 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric12 = 268964461 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric11 = 268964460 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric1 = 268964353 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Numeric0 = 268964352 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Num_Lock = 65407 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ntilde = 209 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NotificationCenter = 268964284 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Next_Virtual_Screen = 65234 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Next_VMode = 269024802 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NextFavorite = 268964464 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NextElement = 268964475 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Next = 65366 :: Word32

-- | /No description available in the introspection data./
pattern KEY_News = 269025129 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NewSheqelSign = 16785578 :: Word32

-- | /No description available in the introspection data./
pattern KEY_New = 269025128 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ncedilla = 977 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ncaron = 466 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NavInfo = 268964488 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NavChart = 268964480 :: Word32

-- | /No description available in the introspection data./
pattern KEY_NairaSign = 16785574 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Nacute = 465 :: Word32

-- | /No description available in the introspection data./
pattern KEY_N = 78 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MySites = 269025127 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MyComputer = 269025075 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Music = 269025170 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MultipleCandidate = 65341 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Multi_key = 65312 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Muhenkan = 65314 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MouseKeys_Enable = 65142 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MouseKeys_Accel_Enable = 65143 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MonBrightnessUp = 269025026 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MonBrightnessDown = 269025027 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MonBrightnessCycle = 269025031 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Mode_switch = 65406 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ModeLock = 269025025 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MillSign = 16785573 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Meta_R = 65512 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Meta_L = 65511 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Messenger = 269025166 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MenuPB = 269025126 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MenuKB = 269025125 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Menu = 65383 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Memo = 269025054 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Meeting = 269025123 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MediaTopMenu = 268964459 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MediaRepeat = 268964279 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Massyo = 65324 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Market = 269025122 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MarkWaypoint = 268964478 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MailForward = 269025168 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Mail = 269025049 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Mae_Koho = 65342 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MacroRecordStop = 268964529 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MacroRecordStart = 268964528 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MacroPresetCycle = 268964530 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MacroPreset3 = 268964533 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MacroPreset2 = 268964532 :: Word32

-- | /No description available in the introspection data./
pattern KEY_MacroPreset1 = 268964531 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro9 = 268964504 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro8 = 268964503 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro7 = 268964502 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro6 = 268964501 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro5 = 268964500 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro4 = 268964499 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro30 = 268964525 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro3 = 268964498 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro29 = 268964524 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro28 = 268964523 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro27 = 268964522 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro26 = 268964521 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro25 = 268964520 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro24 = 268964519 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro23 = 268964518 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro22 = 268964517 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro21 = 268964516 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro20 = 268964515 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro2 = 268964497 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro19 = 268964514 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro18 = 268964513 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro17 = 268964512 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro16 = 268964511 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro15 = 268964510 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro14 = 268964509 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro13 = 268964508 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro12 = 268964507 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro11 = 268964506 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro10 = 268964505 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macro1 = 268964496 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macedonia_kje = 1708 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macedonia_gje = 1698 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macedonia_dse = 1701 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macedonia_KJE = 1724 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macedonia_GJE = 1714 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Macedonia_DSE = 1717 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Mabovedot = 16784960 :: Word32

-- | /No description available in the introspection data./
pattern KEY_M = 77 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Lstroke = 419 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LogWindowTree = 269024804 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LogOff = 269025121 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LogGrabInfo = 269024805 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LiraSign = 16785572 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Linefeed = 65290 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LightsToggle = 268964382 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LightBulb = 269025077 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LeftUp = 268964456 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LeftDown = 268964457 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Left = 65361 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Lcedilla = 934 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Lcaron = 421 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Lbelowdot = 16784950 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LaunchF = 269025103 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LaunchE = 269025102 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LaunchD = 269025101 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LaunchC = 269025100 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LaunchB = 269025099 :: Word32

-- | /No description available in the introspection data./
pattern KEY_LaunchA = 269025098 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Launch9 = 269025097 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Launch8 = 269025096 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Launch7 = 269025095 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Launch6 = 269025094 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Launch5 = 269025093 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Launch4 = 269025092 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Launch3 = 269025091 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Launch2 = 269025090 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Launch1 = 269025089 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Launch0 = 269025088 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Last_Virtual_Screen = 65236 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Lacute = 453 :: Word32

-- | /No description available in the introspection data./
pattern KEY_L9 = 65488 :: Word32

-- | /No description available in the introspection data./
pattern KEY_L8 = 65487 :: Word32

-- | /No description available in the introspection data./
pattern KEY_L7 = 65486 :: Word32

-- | /No description available in the introspection data./
pattern KEY_L6 = 65485 :: Word32

-- | /No description available in the introspection data./
pattern KEY_L5 = 65484 :: Word32

-- | /No description available in the introspection data./
pattern KEY_L4 = 65483 :: Word32

-- | /No description available in the introspection data./
pattern KEY_L3 = 65482 :: Word32

-- | /No description available in the introspection data./
pattern KEY_L2 = 65481 :: Word32

-- | /No description available in the introspection data./
pattern KEY_L10 = 65489 :: Word32

-- | /No description available in the introspection data./
pattern KEY_L1 = 65480 :: Word32

-- | /No description available in the introspection data./
pattern KEY_L = 76 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Korean_Won = 3839 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Keyboard = 269025203 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Kcedilla = 979 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdLightOnOff = 269025028 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdLcdMenu5 = 268964540 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdLcdMenu4 = 268964539 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdLcdMenu3 = 268964538 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdLcdMenu2 = 268964537 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdLcdMenu1 = 268964536 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdInputAssistPrevgroup = 268964450 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdInputAssistPrev = 268964448 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdInputAssistNextgroup = 268964451 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdInputAssistNext = 268964449 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdInputAssistCancel = 268964453 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdInputAssistAccept = 268964452 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdBrightnessUp = 269025029 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KbdBrightnessDown = 269025030 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Katakana = 65318 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Kanji_Bangou = 65335 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Kanji = 65313 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Kana_Shift = 65326 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Kana_Lock = 65325 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Up = 65431 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Tab = 65417 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Subtract = 65453 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Space = 65408 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Separator = 65452 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Right = 65432 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Prior = 65434 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Page_Up = 65434 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Page_Down = 65435 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Next = 65435 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Multiply = 65450 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Left = 65430 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Insert = 65438 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Home = 65429 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_F4 = 65428 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_F3 = 65427 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_F2 = 65426 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_F1 = 65425 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Equal = 65469 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Enter = 65421 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_End = 65436 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Down = 65433 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Divide = 65455 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Delete = 65439 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Decimal = 65454 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Begin = 65437 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_Add = 65451 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_9 = 65465 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_8 = 65464 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_7 = 65463 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_6 = 65462 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_5 = 65461 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_4 = 65460 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_3 = 65459 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_2 = 65458 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_1 = 65457 :: Word32

-- | /No description available in the introspection data./
pattern KEY_KP_0 = 65456 :: Word32

-- | /No description available in the introspection data./
pattern KEY_K = 75 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Journal = 268964418 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Jcircumflex = 684 :: Word32

-- | /No description available in the introspection data./
pattern KEY_J = 74 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Itilde = 933 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Iogonek = 967 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Insert = 65379 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Info = 268964198 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Images = 268964282 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Imacron = 975 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ihook = 16785096 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Igrave = 204 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Idiaeresis = 207 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Icircumflex = 206 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ibreve = 16777516 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ibelowdot = 16785098 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Iacute = 205 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Iabovedot = 681 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Set_Margin_Right = 65064 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Set_Margin_Left = 65063 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Release_Margin_Right = 65066 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Release_Margin_Left = 65065 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Release_Both_Margins = 65067 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Prev_Group_Lock = 65035 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Prev_Group = 65034 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Partial_Space_Right = 65062 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Partial_Space_Left = 65061 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Partial_Line_Up = 65059 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Partial_Line_Down = 65060 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Next_Group_Lock = 65033 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Next_Group = 65032 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Move_Line_Up = 65057 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Move_Line_Down = 65058 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Lock = 65025 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Level5_Shift = 65041 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Level5_Lock = 65043 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Level5_Latch = 65042 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Level3_Shift = 65027 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Level3_Lock = 65029 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Level3_Latch = 65028 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Level2_Latch = 65026 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Left_Tab = 65056 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Last_Group_Lock = 65039 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Last_Group = 65038 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Group_Shift = 65406 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Group_Lock = 65031 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Group_Latch = 65030 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_First_Group_Lock = 65037 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_First_Group = 65036 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Fast_Cursor_Up = 65070 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Fast_Cursor_Right = 65069 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Fast_Cursor_Left = 65068 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Fast_Cursor_Down = 65071 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Enter = 65076 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Emphasize = 65074 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Discontinuous_Underline = 65073 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Continuous_Underline = 65072 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ISO_Center_Object = 65075 :: Word32

-- | /No description available in the introspection data./
pattern KEY_I = 73 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hyper_R = 65518 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hyper_L = 65517 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hstroke = 673 :: Word32

-- | /No description available in the introspection data./
pattern KEY_HotLinks = 269025082 :: Word32

-- | /No description available in the introspection data./
pattern KEY_HomePage = 269025048 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Home = 65360 :: Word32

-- | /No description available in the introspection data./
pattern KEY_History = 269025079 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hiragana_Katakana = 65319 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hiragana = 65317 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hibernate = 269025192 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Henkan_Mode = 65315 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Henkan = 65315 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Help = 65386 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hebrew_switch = 65406 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hcircumflex = 678 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hankaku = 65321 :: Word32

-- | /No description available in the introspection data./
pattern KEY_HangupPhone = 268964286 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_switch = 65406 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_YeorinHieuh = 3829 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_YU = 3792 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_YO = 3787 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_YI = 3794 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_YEO = 3781 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_YE = 3782 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_YAE = 3778 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_YA = 3777 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_WI = 3791 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_WEO = 3789 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_WE = 3790 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_WAE = 3785 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_WA = 3784 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_U = 3788 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Tieut = 3772 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_SunkyeongeumPieub = 3825 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_SunkyeongeumPhieuf = 3828 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_SunkyeongeumMieum = 3824 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Start = 65330 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_SsangSios = 3766 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_SsangPieub = 3763 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_SsangKiyeog = 3746 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_SsangJieuj = 3769 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_SsangDikeud = 3752 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Special = 65343 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Sios = 3765 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_SingleCandidate = 65340 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Romaja = 65334 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_RieulYeorinHieuh = 3823 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_RieulTieut = 3758 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_RieulSios = 3757 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_RieulPieub = 3756 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_RieulPhieuf = 3759 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_RieulMieum = 3755 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_RieulKiyeog = 3754 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_RieulHieuh = 3760 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Rieul = 3753 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_PreviousCandidate = 65342 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_PreHanja = 65338 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_PostHanja = 65339 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_PieubSios = 3764 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Pieub = 3762 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Phieuf = 3773 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_PanSios = 3826 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_OE = 3786 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_O = 3783 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_NieunJieuj = 3749 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_NieunHieuh = 3750 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Nieun = 3748 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_MultipleCandidate = 65341 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Mieum = 3761 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_KkogjiDalrinIeung = 3827 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_KiyeogSios = 3747 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Kiyeog = 3745 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Khieuq = 3771 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Jieuj = 3768 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Jeonja = 65336 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Jamo = 65333 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_YeorinHieuh = 3834 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Tieut = 3820 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_SsangSios = 3815 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_SsangKiyeog = 3797 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Sios = 3814 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_RieulTieut = 3808 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_RieulSios = 3807 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_RieulPieub = 3806 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_RieulPhieuf = 3809 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_RieulMieum = 3805 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_RieulKiyeog = 3804 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_RieulHieuh = 3810 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Rieul = 3803 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_PieubSios = 3813 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Pieub = 3812 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Phieuf = 3821 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_PanSios = 3832 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_NieunJieuj = 3800 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_NieunHieuh = 3801 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Nieun = 3799 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Mieum = 3811 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_KkogjiDalrinIeung = 3833 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_KiyeogSios = 3798 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Kiyeog = 3796 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Khieuq = 3819 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Jieuj = 3817 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Ieung = 3816 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Hieuh = 3822 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Dikeud = 3802 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_J_Cieuc = 3818 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Ieung = 3767 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_I = 3795 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Hieuh = 3774 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Hanja = 65332 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_End = 65331 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_EU = 3793 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_EO = 3779 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_E = 3780 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Dikeud = 3751 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Codeinput = 65335 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Cieuc = 3770 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_Banja = 65337 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_AraeAE = 3831 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_AraeA = 3830 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_AE = 3776 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul_A = 3775 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Hangul = 65329 :: Word32

-- | /No description available in the introspection data./
pattern KEY_H = 72 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Green = 269025188 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_zeta = 2022 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_xi = 2030 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_upsilondieresis = 1977 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_upsilonaccentdieresis = 1978 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_upsilonaccent = 1976 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_upsilon = 2037 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_theta = 2024 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_tau = 2036 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_switch = 65406 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_sigma = 2034 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_rho = 2033 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_psi = 2040 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_pi = 2032 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_phi = 2038 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_omicronaccent = 1975 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_omicron = 2031 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_omegaaccent = 1979 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_omega = 2041 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_nu = 2029 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_mu = 2028 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_lamda = 2027 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_lambda = 2027 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_kappa = 2026 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_iotadieresis = 1973 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_iotaaccentdieresis = 1974 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_iotaaccent = 1972 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_iota = 2025 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_horizbar = 1967 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_gamma = 2019 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_finalsmallsigma = 2035 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_etaaccent = 1971 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_eta = 2023 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_epsilonaccent = 1970 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_epsilon = 2021 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_delta = 2020 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_chi = 2039 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_beta = 2018 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_alphaaccent = 1969 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_alpha = 2017 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_accentdieresis = 1966 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_ZETA = 1990 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_XI = 1998 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_UPSILONdieresis = 1961 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_UPSILONaccent = 1960 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_UPSILON = 2005 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_THETA = 1992 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_TAU = 2004 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_SIGMA = 2002 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_RHO = 2001 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_PSI = 2008 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_PI = 2000 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_PHI = 2006 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_OMICRONaccent = 1959 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_OMICRON = 1999 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_OMEGAaccent = 1963 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_OMEGA = 2009 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_NU = 1997 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_MU = 1996 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_LAMDA = 1995 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_LAMBDA = 1995 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_KAPPA = 1994 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_IOTAdieresis = 1957 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_IOTAdiaeresis = 1957 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_IOTAaccent = 1956 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_IOTA = 1993 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_GAMMA = 1987 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_ETAaccent = 1955 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_ETA = 1991 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_EPSILONaccent = 1954 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_EPSILON = 1989 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_DELTA = 1988 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_CHI = 2007 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_BETA = 1986 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_ALPHAaccent = 1953 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Greek_ALPHA = 1985 :: Word32

-- | /No description available in the introspection data./
pattern KEY_GraphicsEditor = 268964264 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Go = 269025119 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_zhar = 16781535 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_zen = 16781526 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_xan = 16781550 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_we = 16781555 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_vin = 16781525 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_un = 16781539 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_tar = 16781538 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_tan = 16781527 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_shin = 16781544 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_san = 16781537 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_rae = 16781536 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_qar = 16781543 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_phar = 16781540 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_par = 16781534 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_on = 16781533 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_nar = 16781532 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_man = 16781531 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_las = 16781530 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_khar = 16781541 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_kan = 16781529 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_jil = 16781547 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_jhan = 16781551 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_in = 16781528 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_hoe = 16781557 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_hie = 16781554 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_he = 16781553 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_har = 16781556 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_hae = 16781552 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_ghan = 16781542 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_gan = 16781522 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_fi = 16781558 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_en = 16781524 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_don = 16781523 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_cil = 16781548 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_chin = 16781545 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_char = 16781549 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_can = 16781546 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_ban = 16781521 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Georgian_an = 16781520 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Gcircumflex = 728 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Gcedilla = 939 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Gcaron = 16777702 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Gbreve = 683 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Game = 269025118 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Gabovedot = 725 :: Word32

-- | /No description available in the introspection data./
pattern KEY_G = 71 :: Word32

-- | /No description available in the introspection data./
pattern KEY_FullScreen = 269025208 :: Word32

-- | /No description available in the introspection data./
pattern KEY_FrameForward = 269025182 :: Word32

-- | /No description available in the introspection data./
pattern KEY_FrameBack = 269025181 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Forward = 269025063 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Fn_Esc = 268964305 :: Word32

-- | /No description available in the introspection data./
pattern KEY_FnRightShift = 268964325 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Fn = 268964304 :: Word32

-- | /No description available in the introspection data./
pattern KEY_FishingChart = 268964481 :: Word32

-- | /No description available in the introspection data./
pattern KEY_First_Virtual_Screen = 65232 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Find = 65384 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Finance = 269025084 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Favorites = 269025072 :: Word32

-- | /No description available in the introspection data./
pattern KEY_FastReverse = 268964469 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Farsi_yeh = 16778956 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Farsi_9 = 16779001 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Farsi_8 = 16779000 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Farsi_7 = 16778999 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Farsi_6 = 16778998 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Farsi_5 = 16778997 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Farsi_4 = 16778996 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Farsi_3 = 16778995 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Farsi_2 = 16778994 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Farsi_1 = 16778993 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Farsi_0 = 16778992 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Fabovedot = 16784926 :: Word32

-- | /No description available in the introspection data./
pattern KEY_FFrancSign = 16785571 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F9 = 65478 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F8 = 65477 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F7 = 65476 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F6 = 65475 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F5 = 65474 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F4 = 65473 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F35 = 65504 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F34 = 65503 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F33 = 65502 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F32 = 65501 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F31 = 65500 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F30 = 65499 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F3 = 65472 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F29 = 65498 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F28 = 65497 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F27 = 65496 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F26 = 65495 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F25 = 65494 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F24 = 65493 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F23 = 65492 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F22 = 65491 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F21 = 65490 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F20 = 65489 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F2 = 65471 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F19 = 65488 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F18 = 65487 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F17 = 65486 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F16 = 65485 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F15 = 65484 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F14 = 65483 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F13 = 65482 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F12 = 65481 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F11 = 65480 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F10 = 65479 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F1 = 65470 :: Word32

-- | /No description available in the introspection data./
pattern KEY_F = 70 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Explorer = 269025117 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Execute = 65378 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Excel = 269025116 :: Word32

-- | /No description available in the introspection data./
pattern KEY_EuroSign = 8364 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Etilde = 16785084 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Eth = 208 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Escape = 65307 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Eogonek = 458 :: Word32

-- | /No description available in the introspection data./
pattern KEY_End = 65367 :: Word32

-- | /No description available in the introspection data./
pattern KEY_EmojiPicker = 268964425 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Emacron = 938 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Eject = 269025068 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Eisu_toggle = 65328 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Eisu_Shift = 65327 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ehook = 16785082 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Egrave = 200 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Editor = 268964262 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ediaeresis = 203 :: Word32

-- | /No description available in the introspection data./
pattern KEY_EcuSign = 16785568 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ecircumflextilde = 16785092 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ecircumflexhook = 16785090 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ecircumflexgrave = 16785088 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ecircumflexbelowdot = 16785094 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ecircumflexacute = 16785086 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ecircumflex = 202 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ecaron = 460 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ebelowdot = 16785080 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Eacute = 201 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Eabovedot = 972 :: Word32

-- | /No description available in the introspection data./
pattern KEY_EZH = 16777655 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ETH = 208 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ENG = 957 :: Word32

-- | /No description available in the introspection data./
pattern KEY_E = 69 :: Word32

-- | /No description available in the introspection data./
pattern KEY_DualRangeRadar = 268964483 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Dstroke = 464 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Down = 65364 :: Word32

-- | /No description available in the introspection data./
pattern KEY_DongSign = 16785579 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Documents = 269025115 :: Word32

-- | /No description available in the introspection data./
pattern KEY_DoNotDisturb = 268964431 :: Word32

-- | /No description available in the introspection data./
pattern KEY_DisplayToggle = 268964271 :: Word32

-- | /No description available in the introspection data./
pattern KEY_DisplayOff = 268964085 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Display = 269025113 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Dictate = 268964426 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Delete = 65535 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Dcaron = 463 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Database = 268964266 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Data = 268964471 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Dabovedot = 16784906 :: Word32

-- | /No description available in the introspection data./
pattern KEY_DVD = 268964229 :: Word32

-- | /No description available in the introspection data./
pattern KEY_DOS = 269025114 :: Word32

-- | /No description available in the introspection data./
pattern KEY_D = 68 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_zhe_descender = 16778391 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_zhe = 1750 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ze = 1754 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_yu = 1728 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_yeru = 1753 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ya = 1745 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ve = 1751 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_u_straight_bar = 16778417 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_u_straight = 16778415 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_u_macron = 16778479 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_u = 1749 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_tse = 1731 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_te = 1748 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_softsign = 1752 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_shorti = 1738 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_shha = 16778427 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_shcha = 1757 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_sha = 1755 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_schwa = 16778457 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_pe = 1744 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_o_bar = 16778473 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_o = 1743 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_nje = 1706 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_lje = 1705 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ka_vertstroke = 16778397 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ka_descender = 16778395 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ka = 1739 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_je = 1704 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_io = 1699 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ie = 1733 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_i_macron = 16778467 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_i = 1737 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_hardsign = 1759 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ha_descender = 16778419 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ha = 1736 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ghe_bar = 16778387 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ghe = 1735 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_es = 1747 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_er = 1746 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_en_descender = 16778403 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_en = 1742 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_em = 1741 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_el = 1740 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ef = 1734 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_e = 1756 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_dzhe = 1711 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_de = 1732 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_che_vertstroke = 16778425 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_che_descender = 16778423 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_che = 1758 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_be = 1730 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_a = 1729 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ZHE_descender = 16778390 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ZHE = 1782 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ZE = 1786 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_YU = 1760 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_YERU = 1785 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_YA = 1777 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_VE = 1783 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_U_straight_bar = 16778416 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_U_straight = 16778414 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_U_macron = 16778478 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_U = 1781 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_TSE = 1763 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_TE = 1780 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_SOFTSIGN = 1784 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_SHORTI = 1770 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_SHHA = 16778426 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_SHCHA = 1789 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_SHA = 1787 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_SCHWA = 16778456 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_PE = 1776 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_O_bar = 16778472 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_O = 1775 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_NJE = 1722 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_LJE = 1721 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_KA_vertstroke = 16778396 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_KA_descender = 16778394 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_KA = 1771 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_JE = 1720 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_I_macron = 16778466 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_IO = 1715 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_IE = 1765 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_I = 1769 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_HA_descender = 16778418 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_HARDSIGN = 1791 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_HA = 1768 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_GHE_bar = 16778386 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_GHE = 1767 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ES = 1779 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_ER = 1778 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_EN_descender = 16778402 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_EN = 1774 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_EM = 1773 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_EL = 1772 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_EF = 1766 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_E = 1788 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_DZHE = 1727 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_DE = 1764 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_CHE_vertstroke = 16778424 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_CHE_descender = 16778422 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_CHE = 1790 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_BE = 1762 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cyrillic_A = 1761 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CycleAngle = 269025180 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cut = 269025112 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CruzeiroSign = 16785570 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Copy = 269025111 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Control_R = 65508 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Control_L = 65507 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ControlPanel = 268964419 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ContrastAdjust = 269025058 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ContextMenu = 268964278 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Community = 269025085 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ColonSign = 16785569 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Codeinput = 65335 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Close = 269025110 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ClearvuSonar = 268964486 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ClearGrab = 269024801 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Clear = 65291 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ChannelUp = 268964242 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ChannelDown = 268964243 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ch = 65185 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ccircumflex = 710 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ccedilla = 199 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ccaron = 456 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Caps_Lock = 65509 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cancel = 65385 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CameraZoomOut = 268964374 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CameraZoomIn = 268964373 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CameraUp = 268964375 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CameraRight = 268964378 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CameraLeft = 268964377 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CameraFocus = 268964368 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CameraDown = 268964376 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CameraAccessToggle = 268964429 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CameraAccessEnable = 268964427 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CameraAccessDisable = 268964428 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Calendar = 269025056 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Calculator = 269025053 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cacute = 454 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Cabovedot = 709 :: Word32

-- | /No description available in the introspection data./
pattern KEY_C_h = 65188 :: Word32

-- | /No description available in the introspection data./
pattern KEY_C_H = 65189 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CH = 65186 :: Word32

-- | /No description available in the introspection data./
pattern KEY_CD = 269025107 :: Word32

-- | /No description available in the introspection data./
pattern KEY_C = 67 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Byelorussian_shortu = 1710 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Byelorussian_SHORTU = 1726 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Buttonconfig = 268964416 :: Word32

-- | /No description available in the introspection data./
pattern KEY_BrightnessMin = 268964432 :: Word32

-- | /No description available in the introspection data./
pattern KEY_BrightnessMax = 268964433 :: Word32

-- | /No description available in the introspection data./
pattern KEY_BrightnessAuto = 268964084 :: Word32

-- | /No description available in the introspection data./
pattern KEY_BrightnessAdjust = 269025083 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Break = 65387 :: Word32

-- | /No description available in the introspection data./
pattern KEY_BounceKeys_Enable = 65140 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Book = 269025106 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Bluetooth = 269025172 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Blue = 269025190 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Begin = 65368 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Battery = 269025171 :: Word32

-- | /No description available in the introspection data./
pattern KEY_BackSpace = 65288 :: Word32

-- | /No description available in the introspection data./
pattern KEY_BackForward = 269025087 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Back = 269025062 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Babovedot = 16784898 :: Word32

-- | /No description available in the introspection data./
pattern KEY_B = 66 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Away = 269025165 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AutopilotEngageToggle = 268964477 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioStop = 269025045 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioRewind = 269025086 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioRepeat = 269025176 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioRecord = 269025052 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioRandomPlay = 269025177 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioRaiseVolume = 269025043 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioPrev = 269025046 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioPreset = 269025206 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioPlay = 269025044 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioPause = 269025073 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioNext = 269025047 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioMute = 269025042 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioMicMute = 269025202 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioMedia = 269025074 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioLowerVolume = 269025041 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioForward = 269025175 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioDesc = 268964462 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudioCycleTrack = 269025179 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Audio = 268964232 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AudibleBell_Enable = 65146 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AttendantToggle = 268964381 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AttendantOn = 268964379 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AttendantOff = 268964380 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Atilde = 195 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Assistant = 268964423 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AspectRatio = 268964215 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_zhe = 16778602 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_za = 16778598 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_yentamna = 16778634 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_yech = 16778597 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_vyun = 16778626 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_vo = 16778616 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_vev = 16778622 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_verjaket = 16778633 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_tyun = 16778623 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_tso = 16778625 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_tsa = 16778606 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_to = 16778601 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_tche = 16778611 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_shesht = 16778587 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_sha = 16778615 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_separation_mark = 16778589 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_se = 16778621 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_re = 16778624 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_ra = 16778620 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_question = 16778590 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_pyur = 16778627 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_pe = 16778618 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_paruyk = 16778590 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_o = 16778629 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_nu = 16778614 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_men = 16778612 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_lyun = 16778604 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_ligature_ew = 16778631 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_khe = 16778605 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_ken = 16778607 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_ke = 16778628 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_je = 16778619 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_ini = 16778603 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_hyphen = 16778634 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_ho = 16778608 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_hi = 16778613 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_gim = 16778595 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_ghat = 16778610 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_full_stop = 16778633 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_fe = 16778630 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_exclam = 16778588 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_e = 16778599 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_dza = 16778609 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_da = 16778596 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_cha = 16778617 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_but = 16778589 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_ben = 16778594 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_ayb = 16778593 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_at = 16778600 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_apostrophe = 16778586 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_amanak = 16778588 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_accent = 16778587 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_ZHE = 16778554 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_ZA = 16778550 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_YECH = 16778549 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_VYUN = 16778578 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_VO = 16778568 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_VEV = 16778574 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_TYUN = 16778575 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_TSO = 16778577 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_TSA = 16778558 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_TO = 16778553 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_TCHE = 16778563 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_SHA = 16778567 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_SE = 16778573 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_RE = 16778576 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_RA = 16778572 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_PYUR = 16778579 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_PE = 16778570 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_O = 16778581 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_NU = 16778566 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_MEN = 16778564 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_LYUN = 16778556 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_KHE = 16778557 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_KEN = 16778559 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_KE = 16778580 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_JE = 16778571 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_INI = 16778555 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_HO = 16778560 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_HI = 16778565 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_GIM = 16778547 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_GHAT = 16778562 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_FE = 16778582 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_E = 16778551 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_DZA = 16778561 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_DA = 16778548 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_CHA = 16778569 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_BEN = 16778546 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_AYB = 16778545 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Armenian_AT = 16778552 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Aring = 197 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_zain = 1490 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_zah = 1496 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_yeh_baree = 16778962 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_yeh = 1514 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_waw = 1512 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_veh = 16778916 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_tteh = 16778873 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_theh = 1483 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_thal = 1488 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_tehmarbuta = 1481 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_teh = 1482 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_tcheh = 16778886 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_tatweel = 1504 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_tah = 1495 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_switch = 65406 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_superscript_alef = 16778864 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_sukun = 1522 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_sheen = 1492 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_shadda = 1521 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_semicolon = 1467 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_seen = 1491 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_sad = 1493 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_rreh = 16778897 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_ra = 1489 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_question_mark = 1471 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_qaf = 1506 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_percent = 16778858 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_peh = 16778878 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_noon_ghunna = 16778938 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_noon = 1510 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_meem = 1509 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_maddaonalef = 1474 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_madda_above = 16778835 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_lam = 1508 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_khah = 1486 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_keheh = 16778921 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_kasratan = 1517 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_kasra = 1520 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_kaf = 1507 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_jeh = 16778904 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_jeem = 1484 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_heh_goal = 16778945 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_heh_doachashmee = 16778942 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_heh = 1511 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_hamzaunderalef = 1477 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_hamzaonyeh = 1478 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_hamzaonwaw = 1476 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_hamzaonalef = 1475 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_hamza_below = 16778837 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_hamza_above = 16778836 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_hamza = 1473 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_hah = 1485 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_ha = 1511 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_ghain = 1498 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_gaf = 16778927 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_fullstop = 16778964 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_feh = 1505 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_fathatan = 1515 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_fatha = 1518 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_farsi_yeh = 16778956 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_ddal = 16778888 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_dammatan = 1516 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_damma = 1519 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_dal = 1487 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_dad = 1494 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_comma = 1452 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_beh = 1480 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_alefmaksura = 1513 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_alef = 1479 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_ain = 1497 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_9 = 16778857 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_8 = 16778856 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_7 = 16778855 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_6 = 16778854 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_5 = 16778853 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_4 = 16778852 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_3 = 16778851 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_2 = 16778850 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_1 = 16778849 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Arabic_0 = 16778848 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ApplicationRight = 269025105 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ApplicationLeft = 269025104 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AppSelect = 268964420 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Aogonek = 417 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Amacron = 960 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Alt_R = 65514 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Alt_L = 65513 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Ahook = 16785058 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Agrave = 192 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Adiaeresis = 196 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Addressbook = 268964269 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AddFavorite = 269025081 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Acircumflextilde = 16785066 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Acircumflexhook = 16785064 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Acircumflexgrave = 16785062 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Acircumflexbelowdot = 16785068 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Acircumflexacute = 16785060 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Acircumflex = 194 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Accessibility = 268964430 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AccessX_Feedback_Enable = 65137 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AccessX_Enable = 65136 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Abrevetilde = 16785076 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Abrevehook = 16785074 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Abrevegrave = 16785072 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Abrevebelowdot = 16785078 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Abreveacute = 16785070 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Abreve = 451 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Abelowdot = 16785056 :: Word32

-- | /No description available in the introspection data./
pattern KEY_Aacute = 193 :: Word32

-- | /No description available in the introspection data./
pattern KEY_ALSToggle = 268964400 :: Word32

-- | /No description available in the introspection data./
pattern KEY_AE = 198 :: Word32

-- | /No description available in the introspection data./
pattern KEY_A = 65 :: Word32

-- | /No description available in the introspection data./
pattern KEY_9 = 57 :: Word32

-- | /No description available in the introspection data./
pattern KEY_8 = 56 :: Word32

-- | /No description available in the introspection data./
pattern KEY_7 = 55 :: Word32

-- | /No description available in the introspection data./
pattern KEY_6 = 54 :: Word32

-- | /No description available in the introspection data./
pattern KEY_5 = 53 :: Word32

-- | /No description available in the introspection data./
pattern KEY_4 = 52 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3DMode = 268964463 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Test = 64781 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Setup = 64791 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Rule = 64788 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Right2 = 64771 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Reset = 64776 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Record = 64792 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Quit = 64777 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_PrintScreen = 64797 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Play = 64790 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_PA3 = 64780 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_PA2 = 64779 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_PA1 = 64778 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Left2 = 64772 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_KeyClick = 64785 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Jump = 64786 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Ident = 64787 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_FieldMark = 64770 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_ExSelect = 64795 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_EraseInput = 64775 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_EraseEOF = 64774 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Enter = 64798 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Duplicate = 64769 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_DeleteWord = 64794 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_CursorSelect = 64796 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_CursorBlink = 64783 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Copy = 64789 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_ChangeScreen = 64793 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_BackTab = 64773 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_Attn = 64782 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3270_AltCursor = 64784 :: Word32

-- | /No description available in the introspection data./
pattern KEY_3 = 51 :: Word32

-- | /No description available in the introspection data./
pattern KEY_2 = 50 :: Word32

-- | /No description available in the introspection data./
pattern KEY_10ChannelsUp = 268964280 :: Word32

-- | /No description available in the introspection data./
pattern KEY_10ChannelsDown = 268964281 :: Word32

-- | /No description available in the introspection data./
pattern KEY_1 = 49 :: Word32

-- | /No description available in the introspection data./
pattern KEY_0 = 48 :: Word32

-- | Use this macro as the return value for stopping the propagation of
-- an event handler.
pattern EVENT_STOP = P.True :: Bool

-- | Use this macro as the return value for continuing the propagation of
-- an event handler.
pattern EVENT_PROPAGATE = P.False :: Bool

-- | Represents the current time, and can be used anywhere a time is expected.
pattern CURRENT_TIME = 0 :: Int32

-- | The secondary button. This is typically the right mouse button, or the
-- left button in a left-handed setup.
pattern BUTTON_SECONDARY = 3 :: Int32

-- | The primary button. This is typically the left mouse button, or the
-- right button in a left-handed setup.
pattern BUTTON_PRIMARY = 1 :: Int32

-- | The middle button.
pattern BUTTON_MIDDLE = 2 :: Int32

-- | Defines all possible DND actions.
-- 
-- This can be used in 'GI.Gdk.Objects.Drop.dropStatus' messages when any drop
-- can be accepted or a more specific drop method is not yet known.
pattern ACTION_ALL = 7 :: Int32


